package aoc3.dark;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: Magnus Jahre
 * Date: 13.jul.2006
 * Time: 11:58:00
 *
 * @author Magnus Jahre
 */
public final class AssemblyEditor extends JTextPane implements Runnable, DocumentListener
{

  /**
   * Tells the editor to do syntax highlighting for the Dark Stack Machine
   */
  public static final int STACK_EDITOR = 0;
  /**
   * Tells the editor to do syntax highlighting for the Dark Load-Store Machine
   */
  public static final int LOAD_STORE_EDITOR = 1;

  private static final Color COMMENT_COLOR = Color.GRAY;
  private static final Color LABEL_COLOR = Color.BLACK;
  private static final Color KEYWORD_COLOR = Color.BLUE;
  private static final Color REGISTER_COLOR = Color.RED;

  private static final String KEYWORD_FILE_PATH = "data/settings/";
  private static final String LOAD_STORE_KEYWORD_FILE = "loadStoreKeywords.txt";
  private static final String STACK_KEYWORD_FILE = "stackKeywords.txt";

  private final float[] tabStopVals = {20, 40, 60, 80, 100, 120, 140, 160, 180, 200};

  private int editorType;
  private final Map<String, String> keywords;
  private final String host;

  private final AttributeSet labelAttributes;
  private final AttributeSet commentAttributes;
  private final AttributeSet keywordAttributes;
  private AttributeSet registerAttributes;

  /**
   * This constructor creates an AssemblyEditor which in an extension to JTextPane. The main feature added
   * is syntax highlighting for the Dark assembly language.
   *
   * @param type   - The constant STACK_EDITOR for the Dark Stack Machine or LOAD_STORE_EDITOR for the
   *               Dark Load-Store machine
   * @param server - The address of the current server (for example aoc.idi.ntnu.no)
   * @see JTextPane
   */
  public AssemblyEditor(int type, String server)
  {

    setEditorKit(new NumberedEditorKit());
    host = server;

    StyledDocument doc = getStyledDocument();
    doc.addDocumentListener(this);

    TabStop[] tabStops = new TabStop[tabStopVals.length];
    for (int i = 0; i < tabStopVals.length; i++)
      tabStops[i] = new TabStop(tabStopVals[i], TabStop.ALIGN_LEFT, TabStop.LEAD_NONE);
    TabSet tabSet = new TabSet(tabStops);

    StyleContext sc = StyleContext.getDefaultStyleContext();
    AttributeSet mainAttrs = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.TabSet, tabSet);
    mainAttrs = sc.addAttribute(mainAttrs, StyleConstants.FontFamily, "Courier");
    mainAttrs = sc.addAttribute(mainAttrs, StyleConstants.FontSize, 12);
    mainAttrs = sc.addAttribute(mainAttrs, StyleConstants.LeftIndent, (float)25); // Do not write text on line numbers
    setParagraphAttributes(mainAttrs, true);

    labelAttributes = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, LABEL_COLOR);
    commentAttributes = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, COMMENT_COLOR);
    keywordAttributes = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, KEYWORD_COLOR);
    registerAttributes = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, REGISTER_COLOR);
    registerAttributes = sc.addAttribute(registerAttributes, StyleConstants.Bold, true);

    keywords = new HashMap<String, String>();
    editorType = type;
    try {
      setArchitecture(editorType);
    }
    catch (Exception e) {
      e.printStackTrace();
    }

  }

  // --------------------- Begin Interface Runnable ---------------------

  /**
   * This method does the actual syntax highlighting
   */
  public void run()
  {

    StyledDocument doc = getStyledDocument();
    String text = "";
    int len = doc.getLength();
    if (len <= 0) {
      return; // avoids a BadLocationException that sometimes occur
    }

    try {
      text = doc.getText(0, len);
    }
    catch (BadLocationException ble) {
      ble.printStackTrace();
    }

    int currentOffset = 0;
    String[] lines = text.split("\\n");

    for (String currentLine : lines) {

      int lineLength = currentLine.length();
      int lineStart = currentOffset;
      int lineEnd = currentOffset + lineLength + 1;

      String[] tokens = currentLine.split("\\b");

      boolean isComment = false;
      for (int i = 0; i < tokens.length; i++) {

        int tokenLength = tokens[i].length();
        tokens[i] = AssemblyEditor.removeTabs(tokens[i]);

        if (tokens[i].contains(";")) {
          isComment = true;
          int commentStart = currentOffset;
          for (int j = 0; j < tokens[i].length(); j++) {
            if (tokens[i].charAt(j) == ';') {
              commentStart += j;
            }
          }
          if (commentStart - 1 > lineStart) {
            AssemblyEditor.setStyle(doc, currentOffset, commentStart - 1, labelAttributes);
          }
          AssemblyEditor.setStyle(doc, commentStart, lineEnd, commentAttributes);

          currentOffset = lineEnd;

          break; // The rest of the line is a comment and we are finished
        }
        else if (keywords.containsKey(tokens[i])) {
          AssemblyEditor.setStyle(doc, currentOffset, currentOffset + tokenLength, keywordAttributes);
          currentOffset += tokenLength;
        }
        else if (tokens[i].contains("$")) {
          // registers are only allowed in the load-store architecture
          if (editorType == LOAD_STORE_EDITOR) {
            for (int j = 0; j < tokens[i].length(); j++) {
              if (tokens[i].charAt(j) == '$') {
                // special formating for the register character
                doc.setCharacterAttributes(currentOffset + j, 1, registerAttributes, true);
              }
              else {
                doc.setCharacterAttributes(currentOffset + j, 1, labelAttributes, true);
              }
            }
          }
          else {
            AssemblyEditor.setStyle(doc, currentOffset, currentOffset + tokenLength, labelAttributes);
          }
          currentOffset += tokenLength;
        }
        else {
          AssemblyEditor.setStyle(doc, currentOffset, currentOffset + tokenLength, labelAttributes);
          currentOffset += tokenLength;
        }
      }

      if (!isComment) {
        // add an extra character as line feeds are not counted
        currentOffset++;
      }
    }

  }
  // -------------------- End Interface Runnable ---------------------

  // --------------------- Begin Interface DocumentListener ---------------------

  public void insertUpdate(DocumentEvent e)
  {
    SwingUtilities.invokeLater(this); // calls the run method
  }

  public void removeUpdate(DocumentEvent e)
  {
    SwingUtilities.invokeLater(this); // calls the run method
  }

  public void changedUpdate(DocumentEvent e)
  {
    // Do nothing
  }
// --------------------- End Interface DocumentListener ---------------------

  public int getArchitecture()
  {
    return editorType;
  }

  /**
   * This method updates for which Dark architecture syntax highlightning is done.
   *
   * @param newArchitecture - The constant STACK_EDITOR for the Dark Stack Machine or LOAD_STORE_EDITOR for the
   *                        Dark Load-Store machine
   */
  public void setArchitecture(int newArchitecture) throws UnknownArchitectureException
  {

    if (editorType > 1) {
      throw new UnknownArchitectureException("Unknown architecture.");
    }

    editorType = newArchitecture;
    keywords.clear();


    String serverString = "http://" + host + '/';
    try {

      BufferedReader reader;

      if (editorType == STACK_EDITOR) {
        URL tempURL = new URL(serverString + KEYWORD_FILE_PATH + STACK_KEYWORD_FILE);
        reader = new BufferedReader(new InputStreamReader(tempURL.openStream()));
      }
      else {
        URL tempURL = new URL(serverString + KEYWORD_FILE_PATH + LOAD_STORE_KEYWORD_FILE);
        reader = new BufferedReader(new InputStreamReader(tempURL.openStream()));
      }

      String line;
      while ((line = reader.readLine()) != null) {
        line = line.trim();
        keywords.put(line, line);
      }
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    // make sure the syntax highlighting is updated
    SwingUtilities.invokeLater(this);
  }

  private static void setStyle(StyledDocument doc, int fromIndex, int toIndex, AttributeSet attrs)
  {
    for (int i = fromIndex; i < toIndex; i++) {
      doc.setCharacterAttributes(i, 1, attrs, true);
    }
  }

  private static String removeTabs(String str)
  {
    str = str.replaceAll("\t", "");
    return str;
  }

  /**
   * Inner class for exception handling
   */
  private static final class UnknownArchitectureException extends Exception
  {
    UnknownArchitectureException(String message)
    {
      super(message);
    }
  }
}
