package aoc3.dark.architecture;


import aoc3.dark.ExecuteException;
import aoc3.dark.IExecutionListener;
import aoc3.dark.architecture.load.Parser;

import javax.swing.*;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Erik Axel Nielsen <erikaxel@stud.ntnu.no>
 * @version 07.jun.2004
 */
public final class Architecture
{
  private AbstractParser parser;
  private final Run run;
  private final Mem mem = new Mem();
  private int lineNumber;
  private ExecuteThread thread;
  private int architecture;
  private final List<IExecutionListener> executionListeners = new ArrayList<IExecutionListener>();
  private boolean showErrors = true;

  public static final int ARCH_ACCUMULATOR = 0;
  public static final int ARCH_INDEX = 1;
  public static final int ARCH_LOAD = 2;
  public static final int ARCH_MEMORY = 3;
  public static final int ARCH_STACK = 4;

  public static final int STEPS_ONE = 0;
  public static final int STEPS_ALL = 1;

  private static final String[] ARCH_NAMES = {"Accumulator", "Index", "Load", "Memory", "Stack"};

  public Architecture()
  {
    run = new Run(mem);
  }

  public void parse(String source) throws IOException, ParseException
  {
    // Reset mem
    mem.empty();
    switch (architecture) {
      case ARCH_STACK:
        parser = new aoc3.dark.architecture.stack.Parser(source, mem);
        break;
      case ARCH_ACCUMULATOR:
        parser = new aoc3.dark.architecture.accu.Parser(source, mem);
        break;
      case ARCH_INDEX:
        parser = new aoc3.dark.architecture.index.Parser(source, mem);
        break;
      case ARCH_MEMORY:
        parser = new aoc3.dark.architecture.memory.Parser(source, mem);
        break;
      case ARCH_LOAD:
        parser = new Parser(source, mem);
        break;
    }

    parser.setRun(run);
    parser.parse();
    // Initialize the codemachine:
    run.clear();
    // Checks that symbols are correct.
    Architecture.checkSymbols();
  }

  public void setShowErrors(boolean showErrors)
  {
    this.showErrors = showErrors;
  }

  private static void checkSymbols() throws ParseException
  {
    SymbolTable st = SymbolTable.getInstance();

    // Check SymbolTable
    if (!st.isOK()) {
      int unresolvedSize = st.getUnresolved().size();
      if (unresolvedSize == 1)
        throw new ParseException("Uløste foroverreferanser til: '" +
                                 st.getUnresolved().get(0) +
                                 "' i kildefil");
      else {
        StringBuilder sb = new StringBuilder();
        sb.append("Uløste foroverreferanser til::\n");

        for (int i = 0; i < unresolvedSize; i++) {
          sb.append(st.getUnresolved().get(i)).append('\n');
        }
        sb.append("i kildefil.");
        throw new ParseException(sb.toString());
      }
      //clearAll();
    }
  }

  private void lineChanged(int line)
  {
    lineNumber = line;
    for (IExecutionListener executionListener : executionListeners) executionListener.lineChanged(lineNumber);
  }

  private void executionPaused()
  {
    for (IExecutionListener executionListener : executionListeners) executionListener.executionPaused();
  }

  /**
   * Executes a number of logical steps. One step is the same as one codeline.
   * Will start a new thread and return immediately.
   * It will signal is's LineChangeListeners each time a line changes and it's
   * ExecutionListeners when it has stopped.
   *
   * @param steps Either STEPS_ONE or STEPS_ALL
   */
  public void execute(int steps)
  {
    thread = new ExecuteThread(steps, steps == STEPS_ONE);
    Thread t = new Thread(thread);
    t.start();
  }

  public void setArchitecture(int architecture)
  {
    this.architecture = architecture;
  }

  public int getArchitecture()
  {
    return architecture;
  }

  public int getLineNumber()
  {
    return lineNumber;
  }

  public static int getArchitectureNumber(String name)
  {
    for (int i = 0; i < ARCH_NAMES.length; i++) {
      if (ARCH_NAMES[i].equals(name)) return i;
    }
    return 16;
  }

  public void pauseExecution()
  {
    thread.stop = true;
  }

  public Run getRun()
  {
    return run;
  }

  public Mem getMem()
  {
    return mem;
  }

  public void addExecutionListener(IExecutionListener l)
  {
    executionListeners.add(l);
  }

  private final class ExecuteThread implements Runnable
  {
    private final int steps;
    public boolean stop;
    public final boolean notify;

    ExecuteThread(int steps, boolean notify)
    {
      this.steps = steps;
      stop = false;
      this.notify = notify;
    }

    public void run()
    {
      if (steps == STEPS_ONE)
        executeSingleLine();
      else {
        while (!run.stop && !stop) {
          executeSingleLine();
        }
      }
      executionPaused();
    }

    public void executeSingleLine()
    {
      int oldLine = run.getLineNo();
      int newLine = 0;
      if (oldLine == 0)
        return;
      try {
        // run.executeOne performs one internal step only.
        // We want to perform one codeline.
        boolean cont = true;
        while (cont) {

          cont = run.executeOne(true);

          newLine = run.getLineNo();
          if (oldLine != newLine || run.stop) // We have advanced one codeline
            break;
        }
      }
      catch (ExecuteException e) {
        //Stack underflow error reporting is done in DarkPanel. THIS IS WRONG, no stack underflow reporting in DarkPanel!!!

        //if(!e.getMessage().equals("Stakk underflyt")){ // Old code

        if (showErrors) {
          if (e.getMessage().equals("Stakk underflyt")) {
            JOptionPane.showMessageDialog(null, "Du prøvde å hente en verdi fra en tom stakk (stakk underflyt). Vennligst sjekk koden din.", "Error", JOptionPane.ERROR_MESSAGE);
          }
          else {
            JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
          }
        }

        // Old code, solves that the system seems to not respond to the user pressing the "Ett steg" button
        //}
        //else{
        //  if (notify) {
        //    lineChanged(newLine);  // DarkPanel needs to be notified
        //  }
        //}

        return;
      }
      catch (ArithmeticException e) {
        if (showErrors) {
          JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
        }
        return;

      }
      catch (ArrayIndexOutOfBoundsException e) {
        if (architecture == ARCH_STACK) {
          if (showErrors) {
            JOptionPane.showMessageDialog(null, "Du prøvde å hente en verdi fra en tom stakk (stakk underflyt). Vennligst sjekk koden din.", "Error", JOptionPane.ERROR_MESSAGE);
          }
          return;
        }
      }

      if (notify)
        lineChanged(newLine);
    }
  }
}
