package aoc3.dark.validation;

import aoc3.dark.architecture.Architecture;
import aoc3.dark.architecture.ParseException;
import org.w3c.dom.*;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.swing.*;
import javax.xml.parsers.*;
import java.io.IOException;
import java.io.StringReader;
import java.util.Random;

/**
 * @author Erik Axel Nielsen <erikaxel@stud.ntnu.no>
 * @version 09.jun.2004
 */
public final class Validator
{
  public static final int SOLUTION_OK = 0;
  public static final int SOLUTION_ERROR = 1;

  private static final String parseErrorMessage =
    "Valideringsfunksjonen i Dark må deaktiviseres\n" +
    "fordi det oppsto en feil i parsing av oppgaven.\n" +
    "Denne oppgaven kan ikke løses før feilen er rettet.\n" +
    "Vennligst meld følgende feil til fagstaben:\n";
  private final Architecture architecture;
  private final Random pseudoRand;
  private Test[] testList;
  private boolean questionCorrectlyParsed;
  private int architectureInt;

  private static void showParseError(String msg)
  {
    JOptionPane.showMessageDialog(null, msg, "Feil i parsing av oppgave.", JOptionPane.ERROR_MESSAGE);
  }

  public Validator(String xml, Architecture architecture)
  {
    pseudoRand = new Random(System.currentTimeMillis());
    questionCorrectlyParsed = false;
    this.architecture = architecture;
    try {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      //Document doc = builder.parse( test );//builder.parse(new InputSource(new StringReader(xml)));
      Document doc = builder.parse(new InputSource(new StringReader(xml)));
      parseFromXml(doc.getDocumentElement());
      questionCorrectlyParsed = true;
    } catch (SAXException e) {
      Validator.showParseError(parseErrorMessage + e);
      e.printStackTrace();
    } catch (ParserConfigurationException e) {
      Validator.showParseError(parseErrorMessage + e);
      e.printStackTrace();
    } catch (IOException e) {
      Validator.showParseError(parseErrorMessage + e);
      e.printStackTrace();
    }
  }

  public int getArchitectureNumber()
  {
    return architectureInt;
  }

  public boolean isQuestionCorrectlyParsed()
  {
    return questionCorrectlyParsed;
  }

  /**
   * Will validate the source. We expect the architecture to be set up properly.
   *
   * @param source
   * @throws ValidationException
   */
  public void validate(String source) throws ValidationException
  {
    try {
      int permutation[] = randPermutation(testList.length);
      for (int i = 0; i < testList.length; i++) {
//        System.err.println("Validator: validate i==" + permutation[i]);
        architecture.parse(source);
        testList[permutation[i]].validate();
      }
    } catch (IOException e) {
      throw new ValidationException(e.getMessage());
    } catch (ParseException parseError) {
      throw new ValidationException(parseError.getMessage());
    }
  }

  private void parseFromXml(Element elem)
  {
    architectureInt = Architecture.getArchitectureNumber(elem.getAttribute("arch"));

    NodeList nodeList = elem.getElementsByTagName("Test");
    testList = new Test[nodeList.getLength()];
    for (int i = 0; i < nodeList.getLength(); i++) {
      switch (architectureInt) {
        case Architecture.ARCH_STACK:
          testList[i] = new StackTest(architecture, (Element)nodeList.item(i));
          break;
        case Architecture.ARCH_LOAD:
          testList[i] = new LoadTest(architecture, (Element)nodeList.item(i));
          break;
      }
    }
  }

  /**
   * Return a random permutation of 0..(n-1).
   *
   * @param n
   */
  private int[] randPermutation(int n)
  {
    int source_index = 0;
    int permutation[] = new int[n];
    boolean used[] = new boolean[n];

    for (int i = 0; i < n; i++)
      used[i] = false;

    for (int i = 0; i < n; i++) {
      source_index = (source_index + pseudoRand.nextInt(n)) % n;
      while (used[source_index]) source_index = (source_index + 1) % n;
      permutation[i] = source_index;
      used[source_index] = true;
    }
    return permutation;
  }
}

