package aoc3.db;

import aoc3.dark.DarkParameterGenerator;
import aoc3.icons.Icons;

import javax.swing.Timer;
import java.awt.event.*;
import java.io.IOException;
import java.sql.*;
import java.util.*;


/**
 * Created by IntelliJ IDEA.
 * User: Jon Olav Hauglid
 * Date: 10.apr.2005
 * Time: 18:58:15
 * To change this template use File | Settings | File Templates.
 */
public final class Database
{
  private static final String SERVER = "localhost";
  private static final String DATABASE = "aoc3";
  private static final String USERNAME = "aoc";
  private static final String PASSWORD = "your_password_here";
  private static final int PING_DELAY = 6 * 60 * 60 * 1000; // 6 timer
  private static final int MOVEFLUSH_DELAY = 5 * 1000; // 5 sekunder
  private final Connection connection;
  private final List<User> userPosts = new ArrayList<User>();
  private final Map<Integer, Level> levelsByID = new HashMap<Integer, Level>();
  private final Map<Integer, List<Integer>> teleportRequirementsByTeleporter = new HashMap<Integer, List<Integer>>();
  private final Map<Integer, List<LevelItem>> messagesByLevel = new HashMap<Integer, List<LevelItem>>();
  private final Map<Integer, List<LevelItem>> teleportDestsByLevel = new HashMap<Integer, List<LevelItem>>();
  private final Map<Integer, List<LevelItem>> teleportSourcesByLevel = new HashMap<Integer, List<LevelItem>>();
  private final Map<Integer, List<QuestionCategory>> questionCategoriesByLevel = new HashMap<Integer, List<QuestionCategory>>();
  private final Map<Integer, List<UserQuestion>> userQuestionsByUser = new HashMap<Integer, List<UserQuestion>>();
  private final Map<Integer, List<UserText>> userTextsByUser = new HashMap<Integer, List<UserText>>();
  private final Map<Integer, Question> questionsByID = new HashMap<Integer, Question>();
  private final Map<Integer, QuestionCategory> questionCategoriesByID = new HashMap<Integer, QuestionCategory>();
  private final Map<Integer, QuestionGroup> questionGroupsByID = new HashMap<Integer, QuestionGroup>();
  private final Map<Integer, TeleportDest> teleportDestsByID = new HashMap<Integer, TeleportDest>();
  private final Map<Integer, Text> textsByID = new HashMap<Integer, Text>();
  private final Map<Integer, TextGroup> textGroupsByID = new HashMap<Integer, TextGroup>();
  private final Map<QuestionCategory, List<Question>> questionsByCategory = new HashMap<QuestionCategory, List<Question>>();
  private final Set<User> usersMoved = new HashSet<User>();
  private final Statement statement;
  private final Timer dbPingTimer;
  private final Timer dbMoveFlushTimer;
  private Settings settings;

  private static UserQuestion makeQuestionAnswer(User user, Question question)
  {
    if (question instanceof AutoGenQuestion) {
      AutoGenQuestionGenerator generator = ((AutoGenQuestion)question).makeQuestionGenerator();
      Object[] params = generator.generateParameters();
      return new UserQuestion(user.getId(), question.getId(), null, true, params, null, 0);
    }
    else if (question instanceof MultiChoiceQuestion) {
      return new UserQuestion(user.getId(), question.getId(), null, true, null,
                              ((MultiChoiceQuestion)question).getAnswerID(), 0);
    }
    else if (question instanceof InputQuestion) {
      return new UserQuestion(user.getId(), question.getId(), null, true, null,
                              ((InputQuestion)question).getAnswer(), 0);
    }
    else if (question instanceof DarkQuestion) {
      DarkQuestion tempDarkQuestion = (DarkQuestion)question;

      if (tempDarkQuestion.getCode().length() != 0) { // A DarkQuestion without code is not a valid question
        DarkQuestionGenerator generator = tempDarkQuestion.makeQuestionGenerator();
        Object[] params = {DarkParameterGenerator.generateXMLParameters(generator),
                           tempDarkQuestion.getTestCode()};
        return new UserQuestion(user.getId(), question.getId(), null, true, params,
                                "; Du har ikke svart på denne oppgaven\n; Den er synlig pga eksamensmodus", 0);
      }
    }
    // This should not happen
    System.out.println("Unknown question in db.Database.getCompletedQuestions");
    return null;
  }

  public Database() throws SQLException, ClassNotFoundException, IOException
  {
    Class.forName("com.mysql.jdbc.Driver");
    connection = DriverManager.getConnection("jdbc:mysql://" + SERVER + '/' +
                                             DATABASE + "?autoReconnect=true", USERNAME, PASSWORD);
    statement = connection.createStatement();
    statement.setFetchSize(2000);

    readPosts();

    dbPingTimer = new Timer(PING_DELAY, new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        pingDatabase();
      }
    });
    dbPingTimer.start();

    dbMoveFlushTimer = new Timer(MOVEFLUSH_DELAY, new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        flushUsersMoved();
      }
    });
  }

  public void answeredQuestion(User user, Question question, boolean wasCorrect, Object parameters[],
                               Object answer)
  {
    synchronized (userQuestionsByUser) {
      List<UserQuestion> userQuestions = userQuestionsByUser.get(user.getId());
      for (UserQuestion userQuestion : userQuestions) {
        if (userQuestion.getQuestionID() == question.getId() &&
            (!(question instanceof AutoGenQuestion) ||
             Arrays.equals(parameters, userQuestion.getParams()))) {
          if (!userQuestion.isCorrect()) {
            userQuestion.update(wasCorrect, parameters, answer, userQuestion.getTimesAnswered() + 1);
            makeUserQuestionPersistent(userQuestion, true);
          }
          return; // Har uansett svart på denne får
        }
      }
      // Ikke svart på spørsmål før (eller autogen-spørsmål med nye parametre) ==> ny post
      UserQuestion userQuestion = new UserQuestion(user.getId(), question.getId(),
                                                   new Timestamp(System.currentTimeMillis()),
                                                   wasCorrect, parameters, answer, 1);
      userQuestions.add(userQuestion);
      makeUserQuestionPersistent(userQuestion, false);
    }
  }

  public boolean fulfillsTeleportRequirements(User user, TeleportSource teleportSource)
  {
    List<Integer> requirements = teleportRequirementsByTeleporter.get(teleportSource.getId());
    if (requirements == null)
      return true;
    for (Integer requiredCategoryID : requirements)
      if (!isQuestionCategoryCompleted(user, questionCategoriesByID.get(requiredCategoryID)))
        return false;
    return true;
  }

  public Map<UserQuestion, Question> getCompletedQuestions(User user)
  {
    if (isExamMode(user)) { // Exam mode --> return all questions
      Set<Question> unusedQuestions = getUnusedQuestions();
      Map<UserQuestion, Question> questions = new HashMap<UserQuestion, Question>();

      synchronized (questionsByID) {
        for (Question question : questionsByID.values()) {
          if (!unusedQuestions.contains(question)) { // Hide questions that are not used in the game
            List<UserQuestion> correctAnswers = findCorrectAnswers(question, user);
            if (!correctAnswers.isEmpty()) {
              // the user has answered the questions
              for (UserQuestion userQuestion : correctAnswers)
                questions.put(userQuestion, question); // findCorrectAnswers checks if the answer is correct
            }
            else {
              // the user has not answered this question --> create new UserQuestion object
              UserQuestion userQuestion = Database.makeQuestionAnswer(user, question);
              questions.put(userQuestion, question);
            }
          }
        }
      }
      return questions;
    }
    else { // Not exam mode --> return correctly answered questions only
      Map<UserQuestion, Question> questions = new HashMap<UserQuestion, Question>();
      synchronized (userQuestionsByUser) {
        for (UserQuestion userQuestion : userQuestionsByUser.get(user.getId()))
          if (userQuestion.isCorrect())
            questions.put(userQuestion, questionsByID.get(userQuestion.getQuestionID()));
      }
      return questions;
    }
  }

  public List<Highscore> getHighscoreList()
  {
    List<Highscore> highScore = new ArrayList<Highscore>();
    synchronized (connection) {
      try {
        ResultSet rs = statement.executeQuery("SELECT u.username, SUM(q.value) AS poengsum " +
                                              "FROM User u, UserQuestion uq, Question q " +
                                              "WHERE u.id = uq.userid " +
                                              "AND uq.correct = 1 " +
                                              "AND uq.questionid = q.id " +
                                              "AND u.admin = 0 " +
                                              "GROUP BY u.username " +
                                              "ORDER BY poengsum DESC " +
                                              "LIMIT 10");

        while (rs.next())
          highScore.add(new Highscore(rs.getString(1), rs.getInt(2)));
        rs.close();
      } catch (SQLException e) {
        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      }
    }
    return highScore;
  }

  public List<Highscore> getHighscoreWeekList()
  {
    List<Highscore> highScore = new ArrayList<Highscore>();
    synchronized (connection) {
      try {
        ResultSet rs = statement.executeQuery("SELECT u.username, SUM(q.value) AS poengsum " +
                                              "FROM User u, UserQuestion uq, Question q " +
                                              "WHERE u.id = uq.userid " +
                                              "AND uq.correct = 1 " +
                                              "AND uq.questionid = q.id " +
                                              "AND DATE_SUB(CURDATE(),INTERVAL 7 DAY) <= uq.date " +
                                              "AND u.admin = 0 " +
                                              "GROUP BY u.username " +
                                              "ORDER BY poengsum DESC " +
                                              "LIMIT 10");

        while (rs.next())
          highScore.add(new Highscore(rs.getString(1), rs.getInt(2)));
        rs.close();
      } catch (SQLException e) {
        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      }
    }
    return highScore;
  }

  public Map<Integer, List<LevelItem>> getLevelItems(User user, int levelid)
  {
    Map<Integer, List<LevelItem>> levelItems = new HashMap<Integer, List<LevelItem>>();

    Collection<LevelItem> messages = messagesByLevel.get(levelid);
    if (messages == null) {
      levelItems.put(Icons.ICON_MESSAGE, new ArrayList<LevelItem>());
    }
    else {
      levelItems.put(Icons.ICON_MESSAGE, new ArrayList<LevelItem>(messages));
    }

    Collection<LevelItem> destinations = teleportDestsByLevel.get(levelid);
    if (destinations == null) {
      levelItems.put(Icons.ICON_TELEPORTDEST, new ArrayList<LevelItem>());
    }
    else {
      levelItems.put(Icons.ICON_TELEPORTDEST, new ArrayList<LevelItem>(destinations));
    }

    List<LevelItem> teleportersOpen = new ArrayList<LevelItem>();
    List<LevelItem> teleportersClosed = new ArrayList<LevelItem>();
    for (LevelItem post : teleportSourcesByLevel.get(levelid)) {
      if (fulfillsTeleportRequirements(user, (TeleportSource)post))
        teleportersOpen.add(post);
      else
        teleportersClosed.add(post);
    }
    levelItems.put(Icons.ICON_TELEPORTSOURCE, teleportersOpen);
    levelItems.put(Icons.ICON_TELEPORTSOURCE_STATIC, teleportersClosed);

    List<LevelItem> facts = new ArrayList<LevelItem>();
    List<LevelItem> histories = new ArrayList<LevelItem>();
    for (Text post : textsByID.values()) {
      if (post.getLevelid() == levelid && !hasVisited(user, post)) {
        if (post.getType() == Text.TYPE_FACT)
          facts.add(post);
        else
          histories.add(post);
      }
    }
    levelItems.put(Icons.ICON_FACT, facts);
    levelItems.put(Icons.ICON_HISTORY, histories);

    List<LevelItem> categoriesDone = new ArrayList<LevelItem>();
    List<LevelItem> categoriesRemaining = new ArrayList<LevelItem>();
    for (QuestionCategory post : questionCategoriesByLevel.get(levelid)) {
      if (isQuestionCategoryCompleted(user, post))
        categoriesDone.add(post);
      else
        categoriesRemaining.add(post);
    }
    levelItems.put(Icons.ICON_QUESTION_GREEN, categoriesDone);
    levelItems.put(Icons.ICON_QUESTION, categoriesRemaining);

    return levelItems;
  }

  public Level getLevelPost(int id)
  {
    return levelsByID.get(id);
  }

  public QuestionCategory getQuestionCategoryPost(int id)
  {
    return questionCategoriesByID.get(id);
  }

  public Map<Integer, QuestionGroup> getQuestionGroupPosts()
  {
    return questionGroupsByID;
  }

  public Question getQuestionPost(int id)
  {
    return questionsByID.get(id);

    // Caching er på
    /*
  	try {
      synchronized (connection) {
      	return Question.readPost(statement, id);
			}
    } catch (SQLException e) {
      e.printStackTrace();
    }
    return null;
    */
  }

  public List<Question> getQuestions(QuestionCategory questionCategory)
  {
    List<Question> questions = questionsByCategory.get(questionCategory);
    if (questions == null || questions.isEmpty()) {
      System.err.println("Fant ingen spørsmål i kategori: " + questionCategory.getId());
      return new ArrayList<Question>();
    }
    return new ArrayList<Question>(questions);
  }

  public int getScore(User user)
  {
    int score = 0;
    synchronized (userQuestionsByUser) {
      List<UserQuestion> userQuestions = userQuestionsByUser.get(user.getId());
      for (UserQuestion userQuestion : userQuestions) {
        if (userQuestion.isCorrect()) {
          Question question = questionsByID.get(userQuestion.getQuestionID());
          score += question.getValue();
        }
      }
    }
    return score;
  }

  public List<LevelItem> getTeleporterDestination(int levelID)
  {
    return teleportDestsByLevel.get(levelID);
  }

  public TeleportDest getTeleporterDestionation(TeleportSource teleportSource)
  {
    return teleportDestsByID.get(teleportSource.getTeleportdestid());
  }

  public Map<Integer, TextGroup> getTextGroupPosts()
  {
    return textGroupsByID;
  }

  public Text getTextPost(int id)
  {
    return textsByID.get(id);

    // Caching er på
    /*
  	try {
      synchronized(connection){
      	return Text.readPost(statement, id);
      }
    } catch (SQLException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
    return null;
    */
  }

  public User getUserPost(String username)
  {
    synchronized (userPosts) {
      for (User user : userPosts)
        if (user.getUsername().equalsIgnoreCase(username))
          return user;
      // Sjekk om brukeren har blitt lagt til databasen siden serveroppstart
      synchronized (connection) {
        try {
          User user = User.readPost(statement, username);
          if (user != null) {
            userPosts.add(user);
            // Kan trygt annta at nye brukere ikke har besøkt spørsmål eller tekster.
            synchronized (userQuestionsByUser) {
              userQuestionsByUser.put(user.getId(), new ArrayList<UserQuestion>());
            }
            synchronized (userTextsByUser) {
              userTextsByUser.put(user.getId(), new ArrayList<UserText>());
            }
          }
          return user;
        } catch (SQLException e) {
          e.printStackTrace();
        }
      }
      return null;
    }
  }

  public Object[] getUserQuestionParameters(User user, Question question)
  {
    synchronized (userQuestionsByUser) {
      List<UserQuestion> userQuestions = userQuestionsByUser.get(user.getId());
      for (UserQuestion userQuestion : userQuestions) {
        if (userQuestion.getQuestionID() == question.getId() && !userQuestion.isCorrect())
          return userQuestion.getParams();
      }
    }
    return null;
  }

  public List<UserQuestion> getUserQuestions(User user)
  {
    synchronized (userQuestionsByUser) {
      return userQuestionsByUser.get(user.getId());
    }
  }

  public List<Text> getVisitedTexts(int type, User user)
  {
    List<Text> texts = new ArrayList<Text>();
    synchronized (userTextsByUser) {
      for (UserText userText : userTextsByUser.get(user.getId())) {
        Text text = textsByID.get(userText.getTextID());
        if (type == text.getType())
          texts.add(text);
      }
    }
    return texts;
  }

  public boolean isQuestionCategoryCompleted(User user, QuestionCategory post)
  {
    int numAnsweredCorrectly = 0;
    synchronized (userQuestionsByUser) {
      List<UserQuestion> userQuestions = userQuestionsByUser.get(user.getId());
      for (UserQuestion userQuestion : userQuestions) {
        if (userQuestion.isCorrect()) {
          Question question = questionsByID.get(userQuestion.getQuestionID());
          if (question.getQuestionCategoryID() == post.getId())
            numAnsweredCorrectly++;
        }
      }
    }
    return numAnsweredCorrectly >= post.getQuestionsrequired();
  }

  public void makeUserUpdatesPersistent(User user)
  {
    synchronized (connection) {
      dbPingTimer.restart();
      try {
        user.makeUpdatesPersistent();
      } catch (SQLException e) {
        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      }
    }
  }

  public void reloadDatabase()
  {
    dbMoveFlushTimer.stop();
    flushUsersMoved();
    dbPingTimer.restart();
    synchronized (connection) {
      try {
        readPosts();
      } catch (SQLException e) {
        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      } catch (IOException e) {
        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      } catch (ClassNotFoundException e) {
        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      }
    }
    dbMoveFlushTimer.start();
  }

  public void userMoved(User user)
  {
    synchronized (usersMoved) {
      usersMoved.add(user);
      if (!dbMoveFlushTimer.isRunning())
        dbMoveFlushTimer.restart();
    }
  }

  public void visitedText(User user, Text text)
  {
    UserText userText = new UserText(user.getId(), text.getId());
    synchronized (userTextsByUser) {
      List<UserText> userTexts = userTextsByUser.get(user.getId());
      userTexts.add(userText);
    }
    synchronized (connection) {
      dbPingTimer.restart();
      try {
        userText.makePostPersistent();
      } catch (SQLException e) {
        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      }
    }
  }

  // Midlertidig kode
  public void writeHTML(Question question, String result)
  {
    try {
      PreparedStatement ps = connection.prepareStatement("UPDATE Question SET Text = ? WHERE id = ?");
      ps.setString(1, result);
      ps.setInt(2, question.getId());
      ps.executeUpdate();
      ps.close();
    } catch (SQLException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
  }

  private List<UserQuestion> findCorrectAnswers(Question question, User user)
  {
    List<UserQuestion> answeredQuestions = new ArrayList<UserQuestion>();
    synchronized (userQuestionsByUser) {
      List<UserQuestion> userQuestions = userQuestionsByUser.get(user.getId());
      for (UserQuestion userQuestion : userQuestions) {
        if (userQuestion.getQuestionID() == question.getId() && userQuestion.isCorrect()) {
          answeredQuestions.add(userQuestion);
        }
      }
    }
    return answeredQuestions;
  }

  private void flushUsersMoved()
  {
    synchronized (usersMoved) {
      synchronized (connection) {
        try {
          for (User user : usersMoved)
            user.makeUpdatesPersistent();
        } catch (SQLException e) {
          e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        if (!usersMoved.isEmpty())
          dbPingTimer.restart();
      }
      usersMoved.clear();
    }
    dbMoveFlushTimer.stop();
  }

  private Set<Question> getUnusedQuestions()
  {
    Map<QuestionGroup, List<QuestionGroup>> groupChildren = new HashMap<QuestionGroup, List<QuestionGroup>>();
    QuestionGroup unusedQuestionRoot;

    synchronized (questionGroupsByID) { // Map: QuestionGroup => subgroups
      for (QuestionGroup group : questionGroupsByID.values()) {
        List<QuestionGroup> childList = groupChildren.get(group);
        if (childList == null) {
          childList = new ArrayList<QuestionGroup>();
          groupChildren.put(group, childList);
        }
        childList.add(group);
      }

      unusedQuestionRoot = questionGroupsByID.get(settings.getDoNotShowQuestionGroup());
    }

    List<QuestionGroup> doNotShowGroups = new ArrayList<QuestionGroup>();
    doNotShowGroups.add(unusedQuestionRoot);

    List<QuestionGroup> childQueue = groupChildren.get(unusedQuestionRoot);
    while (!childQueue.isEmpty()) {
      QuestionGroup child = childQueue.remove(0);
      doNotShowGroups.add(child);

      if (groupChildren.containsKey(child)) { // check if the group has children
        childQueue.addAll(groupChildren.get(child));
      }
    }
    // Har nå funnet alle subgrupper til settings.getDoNowShowQuestionGroup()

    Map<Integer, List<QuestionCategory>> questionCategoriesByGroupID = new HashMap<Integer, List<QuestionCategory>>();
    synchronized (questionCategoriesByID) {
      for (QuestionCategory category : questionCategoriesByID.values()) {
        List<QuestionCategory> childList = questionCategoriesByGroupID.get(category.getQuestionGroupid());
        if (childList == null) {
          childList = new ArrayList<QuestionCategory>();
          questionCategoriesByGroupID.put(category.getQuestionGroupid(), childList);
        }
        childList.add(category);
      }
    }

    Set<Question> doNotShowQuestions = new HashSet<Question>();

    synchronized (questionsByCategory) {
      for (QuestionGroup hideGroup : doNotShowGroups) {
        List<QuestionCategory> hideCategories = questionCategoriesByGroupID.get(hideGroup.getId());
        for (QuestionCategory hideCategory : hideCategories)
          doNotShowQuestions.addAll(questionsByCategory.get(hideCategory));
      }
    }

    return doNotShowQuestions;
  }

  private boolean hasVisited(User user, Text post)
  {
    synchronized (userTextsByUser) {
      List<UserText> userTexts = userTextsByUser.get(user.getId());
      for (UserText userText : userTexts)
        if (userText.getTextID() == post.getId())
          return true;
    }
    return false;
  }

  private boolean isExamMode(User localUser)
  {
    if (!settings.isExamMode())
      return false; // no need to do the expensive check if the system is not in exam mode

    int lastQuestion = settings.getLastQuestionID();
    synchronized (userQuestionsByUser) {
      for (UserQuestion userQuestion : userQuestionsByUser.get(localUser.getId()))
        if (userQuestion.getQuestionID() == lastQuestion && userQuestion.isCorrect())
          return true;
    }
    return false;
  }

  private void makeUserQuestionPersistent(UserQuestion userQuestion, boolean update)
  {
    synchronized (connection) {
      dbPingTimer.restart();
      try {
        if (update)
          userQuestion.makeUpdatesPersistent();
        else
          userQuestion.makePostPersistent();
      } catch (SQLException e) {
        e.printStackTrace();
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
  }

  private void pingDatabase()
  {
    synchronized (connection) {
      try {
        ResultSet rs = statement.executeQuery("SELECT 1");
        rs.close();
        System.out.println("DB Ping");
      } catch (SQLException e) {
        e.printStackTrace();
      }
    }
  }

  private void readPosts() throws SQLException, IOException, ClassNotFoundException
  {
    System.out.println("Reading posts from database");
    levelsByID.clear();
    teleportRequirementsByTeleporter.clear();
    userQuestionsByUser.clear();
    userTextsByUser.clear();
    messagesByLevel.clear();
    questionsByID.clear();
    questionCategoriesByID.clear();
    questionGroupsByID.clear();
    teleportDestsByID.clear();
    teleportDestsByID.clear();
    textsByID.clear();
    textGroupsByID.clear();
    userPosts.clear();

    synchronized (connection) {
      Level.readPosts(statement, levelsByID);
      TeleportRequirement.readPosts(statement, teleportRequirementsByTeleporter);
      UserQuestion.readPosts(statement, userQuestionsByUser);
      UserText.readPosts(statement, userTextsByUser);
      Message.readPosts(statement, messagesByLevel);
      Question.readPosts(statement, questionsByID);
      QuestionCategory.readPosts(statement, questionCategoriesByID);
      QuestionGroup.readPosts(statement, questionGroupsByID);
      TeleportDest.readPosts(statement, teleportDestsByID);
      TeleportSource.readPosts(statement, teleportSourcesByLevel);
      Text.readPosts(statement, textsByID);
      TextGroup.readPosts(statement, textGroupsByID);
      User.readPosts(statement, userPosts);
      settings = Settings.readPosts(statement);
    }

    questionsByCategory.clear();
    for (QuestionCategory questionCategory : questionCategoriesByID.values()) {
      List<Question> questions = new ArrayList<Question>();
      for (Question question : questionsByID.values())
        if (question.getQuestionCategoryID() == questionCategory.getId())
          questions.add(question);
      questionsByCategory.put(questionCategory, questions);
    }

    questionCategoriesByLevel.clear();
    for (QuestionCategory questionCategory : questionCategoriesByID.values()) {
      List<QuestionCategory> categories = questionCategoriesByLevel.get(questionCategory.getLevelid());
      if (categories == null) {
        categories = new ArrayList<QuestionCategory>();
        questionCategoriesByLevel.put(questionCategory.getLevelid(), categories);
      }
      categories.add(questionCategory);
    }

    teleportDestsByLevel.clear();
    for (TeleportDest teleportDest : teleportDestsByID.values()) {
      List<LevelItem> dests = teleportDestsByLevel.get(teleportDest.getLevelid());
      if (dests == null) {
        dests = new ArrayList<LevelItem>();
        teleportDestsByLevel.put(teleportDest.getLevelid(), dests);
      }
      dests.add(teleportDest);
    }

    // Kode for å unngå NullPointerExceptions senere
    List<LevelItem> emptyItemList = Collections.emptyList();
    List<QuestionCategory> emptyCategoryList = Collections.emptyList();
    for (int levelID : levelsByID.keySet()) {
      if (messagesByLevel.get(levelID) == null)
        messagesByLevel.put(levelID, emptyItemList);
      if (teleportDestsByLevel.get(levelID) == null)
        teleportDestsByLevel.put(levelID, emptyItemList);
      if (teleportSourcesByLevel.get(levelID) == null)
        teleportSourcesByLevel.put(levelID, emptyItemList);
      if (questionCategoriesByLevel.get(levelID) == null)
        questionCategoriesByLevel.put(levelID, emptyCategoryList);
    }
    for (User user : userPosts) {
      if (userQuestionsByUser.get(user.getId()) == null)
        userQuestionsByUser.put(user.getId(), new ArrayList<UserQuestion>());
      if (userTextsByUser.get(user.getId()) == null)
        userTextsByUser.put(user.getId(), new ArrayList<UserText>());
    }
    System.out.println("Posts read");
  }
}

