package aoc3.net;

import aoc3.db.Database;
import aoc3.db.User;
import aoc3.event.*;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.sql.SQLException;
import java.util.*;
import java.util.concurrent.locks.*;
import java.util.Calendar;


/**
 * Created by IntelliJ IDEA.
 * User: Jon Olav Hauglid
 * Date: 10.apr.2005
 * Time: 15:11:24
 * To change this template use File | Settings | File Templates.
 */
public final class Server implements ChatEventListener
{
  public static final int PORT = 2901;
  private static final int CHAT_EVENTS_TO_REMEMBER = 16;
  private final List<ChatEvent> lastChatEvents = new LinkedList<ChatEvent>();
  private final List<ServerUser> clients = new ArrayList<ServerUser>();
  private Database database;
  private final ReadWriteLock rwlock = new ReentrantReadWriteLock();

  public static void main(String args[])
  {
    new Server();
  }

  private Server()
  {
    try {
      database = new Database();
    } catch (SQLException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      System.exit(20);
    } catch (ClassNotFoundException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      System.exit(20);
    } catch (IOException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      System.exit(20);
    }

    try {
      ServerSocket server = new ServerSocket(PORT);
      while (true) {
        try {
          Socket socketToClient = server.accept();

          System.out.println(getTimestamp()+": New connection from: " + socketToClient);
          ServerConnection connection = new ServerConnection(this, socketToClient,
                                                             database.getHighscoreList(),
                                                             database.getHighscoreWeekList());
          ServerUser client = new ServerUser(this, database, connection);
          connection.start(client);
        } catch (IOException e) {
          e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
      }
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

// --------------------- Begin Interface ChatEventListener ---------------------

  public void chatLine(ChatEvent event)
  {
    event.setTimeToCurrent();
    synchronized(lastChatEvents) {
      lastChatEvents.add(event);
      while (lastChatEvents.size() > CHAT_EVENTS_TO_REMEMBER)
        lastChatEvents.remove(0);
    }

    ServerUser[] localClients;
    try {
      rwlock.readLock().lock();
      localClients = clients.toArray(new ServerUser[0]);
    } finally {
      rwlock.readLock().unlock();
    }

    // Arbeider på en lokal kopi for å sikre at critical section er kort
    if(localClients != null){
      for (ServerUser connection : localClients) {
        try{
          if (!connection.getUser().getUsername().equals(event.getUsername())){
            // System.out.println("Notifying other users of new chat line");
            connection.getConnection().fireRemoteChatEvent(event);
          }
        }
        catch(Exception e){
          System.err.println(getTimestamp()+": Exception i chatLine() der exception er forventet");
          e.printStackTrace();
        }
      }
    }
    else{
      System.err.println(getTimestamp()+": localClients er null i chatLine");
    }
  }

// --------------------- End Interface ChatEventListener ---------------------

  void broadcastConnect(ServerUser client)
  {
    UserEvent event = new UserEvent(client.getUser(), UserEvent.USER_CONNECTED);


    ServerUser[] localClients;
    try {
      rwlock.readLock().lock();
      localClients = clients.toArray(new ServerUser[0]);
    } finally {
      rwlock.readLock().unlock();
    }

    // Arbeider på en lokal kopi for å sikre at critical section er kort
    if(localClients != null){
      for (ServerUser otherClient : localClients) {
          try{
            //System.out.println("Notifying other users of connect");
            // Gi andre beskjed om ny bruker
            otherClient.getConnection().fireUserEvent(event);
            // Gi ny bruker beskjed om andre
            client.getConnection().fireUserEvent(new UserEvent(otherClient.getUser(), UserEvent.USER_CONNECTED));
          }
          catch(Exception e){
            System.err.println(getTimestamp()+": Exception i broadcastConnect() der exception er forventet");
            e.printStackTrace();
          }
      }
    }
    else{
      System.err.println(getTimestamp()+": localClients er null i broadcastConnect");
    }


    try {
      rwlock.writeLock().lock();
      clients.add(client);
    } finally {
      rwlock.writeLock().unlock();
    }

    client.getConnection().addChatListener(this);

    synchronized (lastChatEvents) {
      // Send de siste chatEvent'ene
      for (ChatEvent chatEvent : lastChatEvents)
        client.getConnection().fireRemoteChatEvent(chatEvent);
    }
  }

  void broadcastDisconnect(ServerUser client)
  {
    try {
      rwlock.writeLock().lock();
      clients.remove(client);
    } finally {
      rwlock.writeLock().unlock();
    }
    if (client.getUser() == null)
      return;

    UserEvent event = new UserEvent(client.getUser(), UserEvent.USER_DISCONNECTED);
    ServerUser[] localClients;
    try {
      rwlock.readLock().lock();
      localClients = clients.toArray(new ServerUser[0]);
    } finally {
      rwlock.readLock().unlock();
    }

    // Arbeider på en lokal kopi for å sikre at critical section er kort
    if(localClients != null){
      for (ServerUser otherClient : localClients){
        try{
          if (otherClient.getUser() != client.getUser()){
            // System.out.println("Notifying other users of disconnect");
            otherClient.getConnection().fireUserEvent(event);
          }
        }
        catch(Exception e){
          System.err.println(getTimestamp()+": Exception i broadcastDisconnect() der exception er forventet");
          e.printStackTrace();
        }
      }
    }
    else{
      System.err.println(getTimestamp()+": localClients er null i broadcastDisconnect");
    }
  }

  void broadcastPositionChange(ServerUser client, User user)
  {

    ServerUser[] localClients;
    try {
      rwlock.readLock().lock();
      localClients = clients.toArray(new ServerUser[0]);
    } finally {
      rwlock.readLock().unlock();
    }

    // Arbeider på en lokal kopi for å sikre at critical section er kort
    if(localClients != null){
      for (ServerUser otherClient : localClients){
        try{
          if (otherClient != client &&
              otherClient.getUser() != null &&
              otherClient.getUser().getLevelid() == user.getLevelid()){
            // System.out.println("Notifying user "+otherClient.getUser().getId()+" of position change");
            otherClient.getConnection().firePositionChange(user.getId(), user.getPosx(), user.getPosy());
          }
        }
        catch(Exception e){
          System.err.println(getTimestamp()+": Exception i broadcastPositionChange() der exception er forventet");
          e.printStackTrace();
        }
      }
    }
    else{
      System.err.println(getTimestamp()+": localClients er null i broadCastPositionChange");
    }
  }

  void broadcastLevelChange(User user)
  {
    ServerUser[] localClients;
    try {
      rwlock.readLock().lock();
      localClients = clients.toArray(new ServerUser[0]);
    } finally {
      rwlock.readLock().unlock();
    }

    // Arbeider på en lokal kopi for å sikre at critical section er kort
    if(localClients != null){
      for (ServerUser client : localClients){
        try{
          // System.out.println("Notifying user "+client.getUser().getId()+" of level change");
          client.getConnection().fireLevelChange(user.getId(), user.getLevelid(),
                                                 user.getPosx(), user.getPosy(), user.getPosz());
        }
        catch(Exception e){
          System.err.println(getTimestamp()+": Exception i broadcastLevelChange() der exception er forventet");
          e.printStackTrace();
        }
      }
    }
    else{
      System.err.println(getTimestamp()+": localClients er null i broadcastLevelChange");
    }
  }

  boolean isAlreadyConnected(User user)
  {
    try {
      rwlock.readLock().lock();
      for (ServerUser connectedUser : clients)
        if (connectedUser.getUser() == user)
          return true;
    } finally {
      rwlock.readLock().unlock();
    }
    return false;
  }

  ServerUser getServerUser(User user)
  {
    try {
      rwlock.readLock().lock();
      for (ServerUser serverUser : clients)
        if (serverUser.getUser() == user)
          return serverUser;
    } finally {
      rwlock.readLock().unlock();
    }
    return null;
  }

  void forceDisconnnect(ServerUser client)
  {
    try {
      rwlock.writeLock().lock();
      clients.remove(client);
    } finally {
      rwlock.writeLock().unlock();
    }
    client.getConnection().disconnect();
  }

  private String getTimestamp(){
    Calendar now = Calendar.getInstance();
    return now.get(Calendar.DAY_OF_MONTH)+"."
        +now.get(Calendar.MONTH)+"."+now.get(Calendar.YEAR)+"-"
        +now.get(Calendar.HOUR_OF_DAY)+":"+now.get(Calendar.MINUTE);
  }

}

