package aoc3.net;

import aoc3.dark.DarkParameterGenerator;
import aoc3.dark.architecture.Architecture;
import aoc3.dark.validation.ValidationException;
import aoc3.dark.validation.Validator;
import aoc3.db.*;
import aoc3.event.*;
import aoc3.icons.Icons;
import com.alienfactory.javamappy.Layer;
import com.alienfactory.javamappy.loader.MapLoader;

import java.awt.*;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.net.URL;
import java.sql.Date;
import java.sql.Timestamp;
import java.util.*;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 01.mai.2005
 * Time: 16:31:10
 * To change this template use File | Settings | File Templates.
 */
final class ServerUser
{
  private static final int QUESTION_DELAY = 10 * 60 * 1000; // 10 minutter i ms.
  private final Architecture architecture = new Architecture();
  private final Database database;
  private final Server server;
  private final ServerToClient connection;
  private LevelItem currentCollidedItem;
  private Map<Integer, List<LevelItem>> levelItems;
  private Object lastQuestionParameters[];
  private User localUser;

  ServerUser(Server server, Database database, ServerToClient connection)
  {
    this.server = server;
    this.database = database;
    this.connection = connection;
  }

  private void collidedWithMessage(Message message)
  {
    fireSystemChatEvent(message.getText());
  }

  private void collidedWithQuestionCategory(QuestionCategory questionCategory) throws IOException
  {
    if (database.isQuestionCategoryCompleted(localUser, questionCategory))
      return;

    List<Question> categoryQuestions = database.getQuestions(questionCategory);
    List<UserQuestion> userQuestions = database.getUserQuestions(localUser);
    boolean generateNewParameters = false;

    // 1: Fjern spørsmå som har blitt besvart riktig før
    for (Iterator<Question> i = categoryQuestions.iterator(); i.hasNext();) {
      Question question = i.next();
      for (UserQuestion userQuestion : userQuestions) {
        if (userQuestion.getQuestionID() == question.getId() && userQuestion.isCorrect()) {
          if (question instanceof AutoGenQuestion)
            generateNewParameters = true; // AutoGen-spørsmål skal kunne besvares flere ganger
          else
            i.remove();
          break;
        }
      }
    }
    if (categoryQuestions.isEmpty()) {
      fireSystemChatEvent("Fant ingen ubesvarte spørsmål å stille!");
      return;
    }

    // 2: Fjern sp�rsm�l som nylig har blitt besvart
    Timestamp latestTime = new Timestamp(System.currentTimeMillis() - QUESTION_DELAY);
    for (Iterator<Question> i = categoryQuestions.iterator(); i.hasNext();) {
      Question question = i.next();
      if (!(question instanceof AutoGenQuestion)) { // Dette gjelder ikke AutoGen
        for (UserQuestion userQuestion : userQuestions) {
          if (userQuestion.getQuestionID() == question.getId() &&
              userQuestion.getTimestamp().compareTo(latestTime) > 0) {
            i.remove();
            break;
          }
        }
      }
    }
    if (categoryQuestions.isEmpty()) {
      fireSystemChatEvent("Du må vente litt før du kan få nye spørsmål fra denne kategorien");
    }
    else {
      Random random = new Random(System.currentTimeMillis());
      Question question = categoryQuestions.get(random.nextInt(categoryQuestions.size()));
      sendQuestion(question, questionCategory, generateNewParameters);
    }
  }

  private void collidedWithTeleportSource(TeleportSource teleportSource) throws IOException
  {
    if (database.fulfillsTeleportRequirements(localUser, teleportSource)) {
      if (teleportSource.getText() != null)
        fireSystemChatEvent(teleportSource.getText());
      TeleportDest dest = database.getTeleporterDestionation(teleportSource);
      localUser.moveTo(dest.getLevelid(), dest.getPosx(), dest.getPosy(), dest.getPosz());
      database.makeUserUpdatesPersistent(localUser);
      server.broadcastLevelChange(localUser);
      levelItems = database.getLevelItems(localUser, dest.getLevelid());
      connection.fireLevelEvent(database.getLevelPost(dest.getLevelid()), levelItems);
      if (dest.getText() != null)
        fireSystemChatEvent(dest.getText());
    }
    else
      fireSystemChatEvent("Du har ikke besvart nok spørsmål til å kunne bruke denne teleporteren");
  }

  private void collidedWithText(Text text) throws IOException
  {
    connection.fireTextEvent(text);
    database.visitedText(localUser, text);
    connection.fireLevelEvent(text);
  }

  private boolean collidesWithItem(LevelItem item, int animNr)
  {
    Rectangle itemSize = Icons.getFrameSize(animNr);
    Rectangle avatarSize = Icons.getFrameSize(localUser.getAvatar());

    int tw = avatarSize.width + localUser.getPosx();
    int th = avatarSize.height + localUser.getPosy();
    int rw = itemSize.width + item.getPosx();
    int rh = itemSize.height + item.getPosy();

    return (rw < item.getPosx() || rw > localUser.getPosx()) &&
           (rh < item.getPosy() || rh > localUser.getPosy()) &&
           (tw < localUser.getPosx() || tw > item.getPosx()) &&
           (th < localUser.getPosy() || th > item.getPosy());
  }

  private void fireSystemChatEvent(String message)
  {
    connection.fireRemoteChatEvent(new ChatEvent("#AOC", new Date(System.currentTimeMillis()), message));
  }

  ServerToClient getConnection()
  {
    return connection;
  }

  synchronized User getUser()
  {
    return localUser;
  }

  void notifyMove(int posx, int posy) throws IOException
  {
    localUser.moveTo(posx, posy);
    database.userMoved(localUser);
    server.broadcastPositionChange(this, localUser);
    boolean hasCollided = false;
    for (Integer animNr : levelItems.keySet()) {
      List<LevelItem> items = levelItems.get(animNr);
      for (Iterator<LevelItem> i = items.iterator(); i.hasNext();) {
        LevelItem item = i.next();
        if (collidesWithItem(item, animNr)) {
          hasCollided = true;
          if (item != currentCollidedItem) {
            currentCollidedItem = item;
            if (item instanceof Message)
              collidedWithMessage((Message)item);
            else if (item instanceof Text) {
              collidedWithText((Text)item);
              i.remove();
            }
            else if (item instanceof QuestionCategory)
              collidedWithQuestionCategory((QuestionCategory)item);
            else if (item instanceof TeleportSource)
              collidedWithTeleportSource((TeleportSource)item);
          }
        }
      }
    }
    if (!hasCollided)
      currentCollidedItem = null;
  }

  void requestBookQuestion() throws IOException
  {
    Map<UserQuestion, Question> completedQuestions = database.getCompletedQuestions(localUser);
    Map<Integer, QuestionCategory> questionCategories = new HashMap<Integer, QuestionCategory>();
    for (Question question : completedQuestions.values()) {
      questionCategories.put(question.getQuestionCategoryID(),
                             database.getQuestionCategoryPost(question.getQuestionCategoryID()));
    }
    Map<UserQuestion, String> autogenQuestionTexts = new HashMap<UserQuestion, String>();
    Map<UserQuestion, String> autogenFormulas = new HashMap<UserQuestion, String>();
    Map<UserQuestion, String> darkSuggestedSolutions = new HashMap<UserQuestion, String>();

    for (UserQuestion userQuestion : completedQuestions.keySet()) {
      Question question = completedQuestions.get(userQuestion);

      if (question instanceof AutoGenQuestion) {
        AutoGenQuestionGenerator generator = ((AutoGenQuestion)question).makeQuestionGenerator();
        if (generator == null)
          autogenQuestionTexts.put(userQuestion, null);
        else {
          autogenQuestionTexts.put(userQuestion, generator.getQuestionText(userQuestion.getParams()));
        }
        autogenFormulas.put(userQuestion, ((AutoGenQuestion)question).getFormula());
      }

      else if (question instanceof DarkQuestion) {
        darkSuggestedSolutions.put(userQuestion, ((DarkQuestion)question).getSuggestedSolution());
      }
    }

    connection.fireBookEvent(completedQuestions,
                             questionCategories,
                             database.getQuestionGroupPosts(),
                             autogenQuestionTexts,
                             autogenFormulas,
                             darkSuggestedSolutions);
  }

  void requestBookText(int type) throws IOException
  {
    connection.fireBookEvent(type, database.getVisitedTexts(type, localUser),
                             database.getTextGroupPosts());
  }

  void requestLevel(int levelID) throws IOException
  {
    if (localUser.getLevelid() == levelID) { // Kan bare få info om brett brukeren har nådd
      levelItems = database.getLevelItems(localUser, levelID);
      connection.fireLevelEvent(database.getLevelPost(levelID), levelItems);
    }
  }

  synchronized void requestLogin(String username, byte passkey[]) throws IOException
  {
    User user = database.getUserPost(username);
    if (user == null)
      connection.fireLoginEvent(new LoginEvent(LoginEvent.LOGIN_UNKNOWN_USER));
    else if (!connection.passwordMatches(user.getPassword(), passkey))
      connection.fireLoginEvent(new LoginEvent(LoginEvent.LOGIN_WRONG_PASSWORD));
    else {
      if (server.isAlreadyConnected(user))
        server.forceDisconnnect(server.getServerUser(user));

      //Sjekk om bruker er i gokk og flytt hvis han er det
      Level currentLevel = database.getLevelPost(user.getLevelid());
      if(currentLevel == null){
        System.out.println("User was on a non-existing level, restart required (level = " + user.getLevelid() + ')');
        connection.fireLoginEvent(new LoginEvent(LoginEvent.LOGIN_ERROR));
        return;
      }

      URL mapUrl = new URL("http://127.0.0.1/data/map/" + currentLevel.getMap());
      com.alienfactory.javamappy.Map currentMap = null;
      try {
        BufferedInputStream inputStream = new BufferedInputStream(mapUrl.openStream());
        currentMap = MapLoader.loadMap(inputStream);
        inputStream.close();
      }
      catch (Exception e) {
        e.printStackTrace();
      }

      if (currentMap != null) {
        Layer[] layers = currentMap.getLayers();
        Layer userLayer;
        if (user.getPosz() < layers.length) {
          userLayer = layers[user.getPosz()];
        }
        else {
          System.out.println("User was on a non-existing layer (layer = " + user.getPosz() + ')');
          connection.fireLoginEvent(new LoginEvent(LoginEvent.LOGIN_ERROR));
          return;
        }


        int blockWidth = currentMap.getMapHeader().getBlockWidth();
        int blockHeight = currentMap.getMapHeader().getBlockHeight();
        int tileX = user.getPosx() / blockWidth;
        int tileY = user.getPosy() / blockHeight;

        try {
          // Står brukeren på en ulovlig plass?
          if (userLayer.getBlockIndex(tileX, tileY) == 0) {
            giveNewPosistion(user);
          }
        }
        catch (IllegalArgumentException illArg) {
          //bruker er utenfor brettet
          giveNewPosistion(user);
        }
        catch (Exception e) {
          e.printStackTrace();
        }
      }

      localUser = user;
      connection.fireLoginEvent(new LoginEvent(LoginEvent.LOGIN_SUCCESS, user, database.getScore(user)));
      server.broadcastConnect(this);
    }
  }

  private void giveNewPosistion(User user)
  {
    List<LevelItem> destinations = database.getTeleporterDestination(user.getLevelid());
    TeleportDest dest = (TeleportDest)destinations.get(0);

    //Skriv ut informasjon om hendelsen
    Calendar cal = Calendar.getInstance();
    String timestamp = cal.get(Calendar.DATE) + "." +
                       cal.get(Calendar.MONTH) + '.' +
                       cal.get(Calendar.YEAR) + ' ' +
                       cal.get(Calendar.HOUR_OF_DAY) + ':' +
                       cal.get(Calendar.MINUTE) + ':' +
                       cal.get(Calendar.SECOND);
    System.out.println("\n---- Information (" + timestamp + "): ");
    System.out.println("User " + user.getUsername() + " is at an illegal position at log on.");
    System.out.println("Position:\t Level " + user.getLevelid() + " (" + user.getPosx() + ", " + user.getPosy() + ", " + user.getPosz() + ')');
    System.out.println("Moved to:\t Level " + user.getLevelid() + " (" + dest.getPosx() + ", " + dest.getPosy() + ", " + user.getPosz() + ')');
    System.out.println("---------");

    user.moveTo(dest.getPosx(), dest.getPosy());
  }

  private void sendQuestion(Question question, QuestionCategory questionCategory,
                            boolean generateNewParameters) throws IOException
  {
    if (question instanceof AutoGenQuestion) {
      AutoGenQuestionGenerator generator = ((AutoGenQuestion)question).makeQuestionGenerator();
      if (generator == null)
        connection.fireQuestionEvent(new QuestionEvent(question, questionCategory));
      else {
        Object parameters[] = database.getUserQuestionParameters(localUser, question);
        if (parameters == null || generateNewParameters)
          parameters = generator.generateParameters();
        lastQuestionParameters = parameters;
        connection.fireQuestionEvent(new QuestionEvent((AutoGenQuestion)question, questionCategory,
                                                       generator.getQuestionText(parameters)));
      }
    }
    else if (question instanceof DarkQuestion) {
      Object parameters[] = database.getUserQuestionParameters(localUser, question);
      if (parameters == null) {
        DarkQuestionGenerator generator = ((DarkQuestion)question).makeQuestionGenerator();
        if (generator == null) {
          connection.fireQuestionEvent(new QuestionEvent((DarkQuestion)question, questionCategory, null, null));
          return;
        }
        parameters = new Object[] {DarkParameterGenerator.generateXMLParameters(generator),
                                   ((DarkQuestion)question).getTestCode()};
      }
      lastQuestionParameters = parameters;
      connection.fireQuestionEvent(new QuestionEvent((DarkQuestion)question, questionCategory,
                                                     (String)parameters[0], (String)parameters[1]));
    }
    else
      connection.fireQuestionEvent(new QuestionEvent(question, questionCategory));
  }

  void submitAnswerAutoGen(int questionid, String answer) throws IOException
  {
    Question question = database.getQuestionPost(questionid);
    AutoGenQuestionGenerator generator = ((AutoGenQuestion)question).makeQuestionGenerator();
    boolean wasCorrect = generator.checkSolution(lastQuestionParameters, answer);
    database.answeredQuestion(localUser, question, wasCorrect, lastQuestionParameters, answer);
    connection.fireAnswerEvent(wasCorrect, false, database.getScore(localUser));
    QuestionCategory questionCategory = database.getQuestionCategoryPost(question.getQuestionCategoryID());
    if (wasCorrect && database.isQuestionCategoryCompleted(localUser, questionCategory))
      connection.fireLevelEvent(questionCategory, Icons.ICON_QUESTION_GREEN);
    if (wasCorrect)
      checkActivatedTeleporters();
  }

  void submitAnswerDark(int questionid, String answer) throws IOException
  {
    Question question = database.getQuestionPost(questionid);
    Validator validator = new Validator((String)lastQuestionParameters[0], architecture);
    architecture.setArchitecture(validator.getArchitectureNumber());
    boolean wasCorrect = true;
    try {
      validator.validate(answer);
    } catch (ValidationException e) {
      wasCorrect = false;
      e.printStackTrace();
    }
    database.answeredQuestion(localUser, question, wasCorrect, lastQuestionParameters, answer);
    connection.fireAnswerEvent(wasCorrect, true, database.getScore(localUser));
    QuestionCategory questionCategory = database.getQuestionCategoryPost(question.getQuestionCategoryID());
    if (wasCorrect && database.isQuestionCategoryCompleted(localUser, questionCategory))
      connection.fireLevelEvent(questionCategory, Icons.ICON_QUESTION_GREEN);
    if (wasCorrect)
      checkActivatedTeleporters();
  }

  void submitAnswerStatic(int questionid, Object answer) throws IOException
  {
    Question question = database.getQuestionPost(questionid);
    boolean wasCorrect = false;
    if (question instanceof MultiChoiceQuestion) {
      int answerid = (Integer)answer;
      wasCorrect = ((MultiChoiceQuestion)question).getAnswerID() == answerid;
    }
    else if (question instanceof InputQuestion) {
      String answerString = answer.toString();
      wasCorrect = ((InputQuestion)question).getAnswer().equalsIgnoreCase(answerString);
    }
    database.answeredQuestion(localUser, question, wasCorrect, null, answer);
    connection.fireAnswerEvent(wasCorrect, false, database.getScore(localUser));
    QuestionCategory questionCategory = database.getQuestionCategoryPost(question.getQuestionCategoryID());
    if (wasCorrect && database.isQuestionCategoryCompleted(localUser, questionCategory))
      connection.fireLevelEvent(questionCategory, Icons.ICON_QUESTION_GREEN);
    if (wasCorrect)
      checkActivatedTeleporters();
  }

  private void checkActivatedTeleporters() throws IOException
  {
    List<LevelItem> staticTeleporters = levelItems.get(Icons.ICON_TELEPORTSOURCE_STATIC);
    List<LevelItem> activeTeleporters = levelItems.get(Icons.ICON_TELEPORTSOURCE);
    for (Iterator<LevelItem> i = staticTeleporters.iterator(); i.hasNext();) {
      TeleportSource teleporterSource = (TeleportSource)i.next();
      if (database.fulfillsTeleportRequirements(localUser, teleporterSource)) {
        connection.fireLevelEvent(teleporterSource, Icons.ICON_TELEPORTSOURCE);
        i.remove();
        activeTeleporters.add(teleporterSource);
      }
    }
  }

  void submitCommand(ChatEvent event) throws IOException
  {
    try {
      StringTokenizer st = new StringTokenizer(event.getMessage());
      String command = st.nextToken();
      if (command.equalsIgnoreCase("/help")) {
        if (localUser.isAdmin())
          fireSystemChatEvent("Følgende kommandoer finnes:\n" +
                              "/loc - Nåværende posisjon\n" +
                              "/locuser <username> - Posisjonen til oppgitt bruker\n" +
                              "/highscore - Vis 10 beste poengsummer\n" +
                              "/highscoreweek - Vis 10 beste poengsummer siste 7 dager\n" +
                              "/question <nr> - Vis spørsmål med id <nr>\n" +
                              "/text <nr> - Vis fakta-/historieark med id <nr>\n" +
                              "/reload - Les alle data inn fra database på nytt (MERK: IKKE THREAD SAFE)\n" +
                              "/goto <levelid> <posx> <posy> <posz> - Hopp til posisjon\n" +
                              "/moveuser <username> <levelid> <posx> <posy> <posz> - Flytt bruker\n" +
                              "/answer <nr> - Vis fasit på spørsmål med id <nr> (hvis fasit finnes)");
        else
          fireSystemChatEvent("Følgende kommandoer finnes:\n" +
                              "/loc - Nåværende posisjon\n" +
                              "/highscore - Vis 10 beste poengsummer\n" +
                              "/highscoreweek - Vis 10 beste poengsummer siste 7 dager");
      }
      else if (command.equalsIgnoreCase("/loc"))
        fireSystemChatEvent("Posisjon: (" + localUser.getPosx() + ',' + localUser.getPosy() + ',' +
                            localUser.getPosz() + ") på brett " + localUser.getLevelid());
      else if (command.equalsIgnoreCase("/highscore")) {
        List<Highscore> highscore = database.getHighscoreList();
        for (int i = 0; i < highscore.size(); i++)
          fireSystemChatEvent((i + 1) + " : " + highscore.get(i));
      }
      else if (command.equalsIgnoreCase("/highscoreweek")) {
        List<Highscore> highscoreWeek = database.getHighscoreWeekList();
        for (int i = 0; i < highscoreWeek.size(); i++)
          fireSystemChatEvent((i + 1) + " : " + highscoreWeek.get(i));
      }
      else if (localUser.isAdmin() && command.equalsIgnoreCase("/locuser")) {
        String username = st.nextToken();
        User thisUser = database.getUserPost(username);

        if (thisUser == null) {
          fireSystemChatEvent("Fant ikke brukernavn");
        }
        else {
          StringBuilder outString = new StringBuilder();
          outString.append(username).append(" har posisjon (");
          outString.append(thisUser.getPosx()).append(", ");
          outString.append(thisUser.getPosy()).append(", ");
          outString.append(thisUser.getPosz()).append(") ");
          outString.append("på brett ").append(thisUser.getLevelid());

          fireSystemChatEvent(outString.toString());
        }

      }
      else if (localUser.isAdmin() && command.equalsIgnoreCase("/question")) {
        int questionid = Integer.parseInt(st.nextToken());
        Question question = database.getQuestionPost(questionid);
        if (question == null)
          fireSystemChatEvent("Fant ikke spørsmål");
        else
          sendQuestion(question, database.getQuestionCategoryPost(question.getQuestionCategoryID()), false);
      }
      else if (localUser.isAdmin() && command.equalsIgnoreCase("/text")) {
        int textid = Integer.parseInt(st.nextToken());
        Text text = database.getTextPost(textid);
        connection.fireTextEvent(text);
      }
      else if (localUser.isAdmin() && command.equalsIgnoreCase("/reload"))
        database.reloadDatabase();
      else if (localUser.isAdmin() && command.equalsIgnoreCase("/goto")) {
        int levelid = Integer.parseInt(st.nextToken());
        int posx = Integer.parseInt(st.nextToken());
        int posy = Integer.parseInt(st.nextToken());
        int posz = Integer.parseInt(st.nextToken());
        localUser.moveTo(levelid, posx, posy, posz);
        database.makeUserUpdatesPersistent(localUser);
        server.broadcastLevelChange(localUser);
        levelItems = database.getLevelItems(localUser, levelid);
        connection.fireLevelEvent(database.getLevelPost(levelid), levelItems);
      }
      else if (localUser.isAdmin() && command.equalsIgnoreCase("/moveuser")) {
        String username = st.nextToken();
        int levelid = Integer.parseInt(st.nextToken());
        int posx = Integer.parseInt(st.nextToken());
        int posy = Integer.parseInt(st.nextToken());
        int posz = Integer.parseInt(st.nextToken());
        User user = database.getUserPost(username);
        if (user == null)
          fireSystemChatEvent("Ukjent bruker: " + username);
        else {
          user.moveTo(levelid, posx, posy, posz);
          database.makeUserUpdatesPersistent(user);
          ServerUser serverUser = server.getServerUser(user);
          if (serverUser != null) { // Bruker innlogget
            server.broadcastLevelChange(user);
            serverUser.requestLevel(levelid);
            fireSystemChatEvent("Bruker flyttet (bruker er innlogget)");
          }
          else
            fireSystemChatEvent("Bruker flyttet (bruker er ikke innlogget)");
        }
      }
      else if (localUser.isAdmin() && command.equalsIgnoreCase("/answer")) {
        int questionid = Integer.parseInt(st.nextToken());
        Question question = database.getQuestionPost(questionid);
        if (question == null)
          fireSystemChatEvent("Fant ikke spørsmål");
        else if (question instanceof InputQuestion)
          fireSystemChatEvent("Riktig svar på spørsmål " + questionid + " er " +
                              ((InputQuestion)question).getAnswer());
        else if (question instanceof MultiChoiceQuestion)
          fireSystemChatEvent("Riktig svar på spørsmål " + questionid + " er alternativ " +
                              (1 + ((MultiChoiceQuestion)question).getAnswerID()));
        else
          fireSystemChatEvent("Har ikke fasit på spørsmål " + questionid);
      }


    } catch (NumberFormatException e) {
      fireSystemChatEvent("Feil under parsing: Kunne ikke konvertere fra tekst til tall");
    } catch (NoSuchElementException e) {
      fireSystemChatEvent("Feil under parsing: For få argumenter");
    }
  }
}

