package gui;

import gui.listeners.AdminPanelListener;
import sys.datatypes.User;
import sys.models.AdminModel;
import sys.models.UserTableModel;

import javax.swing.*;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import java.awt.*;
import java.util.ArrayList;

/**
 * This is the panel that contains everything in the admin view.
 * It gets its data from a UserTableModel and a AdminModel.
 *
 * @author Tarje Killingberg - 19.jun.2006
 */
public class AdminPanel extends JPanel implements Saver, TableModelListener {


    private JPanel settingsPanel;
    private JPanel userPanel;
    private JScrollPane userTableScrollPane;
    public JTable userTable;

    private JButton examModeButton;
    private JTextField styleSheetField;
    private JComboBox lastQuestionComboBox;
    private JCheckBox adminBox;
    private JCheckBox nonAdminBox;
    private JTextField searchField;
    private JComboBox fieldsComboBox;

    private AdminPanelListener adminPanelListener;
    private UserTableModel userTableModel;
    private AdminModel adminModel;


    /**
     * Creates a new AdminPanel
     */
    public AdminPanel() {

        this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        this.setBorder(new EmptyBorder(20, 10, 10, 10));




        adminPanelListener = new AdminPanelListener(this);

        add(Box.createHorizontalGlue());

        settingsPanel = new JPanel();
        initSettingsPanel();
        add(settingsPanel);
        add(Box.createRigidArea(new Dimension(30, 0)));

        userPanel = new JPanel();
        initUserPanel();
        add(userPanel);
        add(Box.createHorizontalGlue());

        resetFilter();
    }


    /**
     * Called when the "New"-button has been pressed.
     * Tells the AdminModel to create a new user.
     */
    public void newUser() {
        resetFilter();
        adminModel.createNewUser();
    }


    /**
     * Called when the "Delete"-button has been pressed.
     * Gets the selected rows from the table, translates
     * them to users using the UserTableModel, and tells
     * the AdminModel to delete them.
     */
    public void deleteButtonPressed() {

        int editingRow = userTable.getEditingRow();
        int[] selectedRows = userTable.getSelectedRows();
        if (editingRow != -1) {
            userTable.getCellEditor().stopCellEditing();
            userTable.getSelectionModel().addSelectionInterval(editingRow, editingRow);
            for (int i : selectedRows) {
                userTable.getSelectionModel().addSelectionInterval(i, i);
            }
        }

        int choice;
        int[] viewIndices = userTable.getSelectedRows();
        if (viewIndices.length > 0) {
            int[] modelIndices = new int[viewIndices.length];
            for (int i = 0; i < viewIndices.length; i++) {
                modelIndices[i] = ((TableSorter)userTable.getModel()).modelIndex(viewIndices[i]);
            }
            User[] users = userTableModel.getUsers(modelIndices);


            choice = JOptionPane.showConfirmDialog(this,
                                          "You are about to delete " + (users.length > 1 ? (users.length + " users") : users[0].getUsername()) + " from database.\n" +
                                          "Continue?",
                                          "Confirm",
                                          JOptionPane.YES_NO_OPTION);

            if (choice != JOptionPane.YES_OPTION) {
                return;
            }

            adminModel.deleteUsers(users);
        }
    }


    /**
     * Called when the "Change Password"-button has been pressed.
     * Displays a JOptionPane where the user can input the new
     * password, and if everything is OK, updates the user
     * with the new password.
     */
    public void changePassword() {
        User user;
        int selection;
        String newPassword1, newPassword2;

        int index = userTable.getSelectedRow();

        if (index == -1) {
            JOptionPane.showMessageDialog(this,
                                          "Select a user first.",
                                          "No user selected",
                                          JOptionPane.INFORMATION_MESSAGE);

            return;
        }

        index = ((TableSorter)userTable.getModel()).modelIndex(index);
        user = userTableModel.getUser(index);

        if (user == null) {
            return;
        }

        JPanel passwordOptionPaneContentPane = new JPanel();
        passwordOptionPaneContentPane.setLayout(new BoxLayout(passwordOptionPaneContentPane, BoxLayout.Y_AXIS));

        passwordOptionPaneContentPane.add(Box.createVerticalGlue());
        JLabel newPasswordLabel = new JLabel("Select and retype new password for user " + user.getUsername() + ".");
        passwordOptionPaneContentPane.add(newPasswordLabel);
        passwordOptionPaneContentPane.add(Box.createRigidArea(new Dimension(0, 10)));
        JPasswordField passwordField1 = new JPasswordField(20);
        passwordOptionPaneContentPane.add(passwordField1);
        passwordOptionPaneContentPane.add(Box.createRigidArea(new Dimension(0, 10)));
        JPasswordField passwordField2 = new JPasswordField(20);
        passwordOptionPaneContentPane.add(passwordField2);
        passwordOptionPaneContentPane.add(Box.createVerticalGlue());

        selection = JOptionPane.showConfirmDialog(this,
                                    passwordOptionPaneContentPane,
                                    "New password",
                                    JOptionPane.OK_CANCEL_OPTION,
                                    JOptionPane.PLAIN_MESSAGE);

        if (selection != JOptionPane.OK_OPTION) {
            return;
        }

        newPassword1 = new String(passwordField1.getPassword());
        newPassword2 = new String(passwordField2.getPassword());

        if (!newPassword1.equals(newPassword2)) {
            JOptionPane.showMessageDialog(this,
                                          "Password was not changed because\n" +
                                          "the input values were not equal.",
                                          "Values did not match",
                                          JOptionPane.ERROR_MESSAGE);

            return;

        }

        if (newPassword1.equals("")) {
            JOptionPane.showMessageDialog(this,
                                          "Password was not changed because\n" +
                                          "the input values were not legal.",
                                          "Illegal password",
                                          JOptionPane.ERROR_MESSAGE);

            return;

        }

        user.setPassword(newPassword1);
        getAdminModel().userChanged(user);
    }


    /**
     * Called by the AdminPanelListener when something on the
     * filterPanel has produced an ActionEvent.
     * Reloads the content of the table to fit the
     * current filter(s).
     */
    public void fireFilterChanged() {
        userTableModel.filterChanged(adminBox.isSelected(),
                                     nonAdminBox.isSelected(),
                                     searchField.getText(),
                                     (String)fieldsComboBox.getSelectedItem());

    }


    /**
     * Called when the "Clear filters"-button has been pressed.
     * Resets the filter-components and refreshes the table.
     */
    public void resetFilter() {
        adminBox.removeActionListener(adminPanelListener);
        nonAdminBox.removeActionListener(adminPanelListener);
        searchField.removeActionListener(adminPanelListener);
        fieldsComboBox.removeActionListener(adminPanelListener);
        adminBox.setSelected(true);
        nonAdminBox.setSelected(true);
        searchField.setText("");
        fieldsComboBox.setSelectedIndex(0);
        adminBox.addActionListener(adminPanelListener);
        nonAdminBox.addActionListener(adminPanelListener);
        searchField.addActionListener(adminPanelListener);
        fieldsComboBox.addActionListener(adminPanelListener);
        fireFilterChanged();
    }


    /**
     * Called when the "Exam Mode"-button has been pressed.
     * Asks the user to confirm the switch, and tells the
     * AdminModel to switch exam mode on or off.
     */
    public void switchExamMode() {
        int choice = JOptionPane.showConfirmDialog(this,
                                      "Are you sure you want to set exam mode to: " + (!adminModel.isExamMode() ? "on" : "off") + "?",
                                      "Confirm!",
                                      JOptionPane.YES_NO_OPTION,
                                      JOptionPane.QUESTION_MESSAGE);

        if (choice == JOptionPane.YES_OPTION) {
            adminModel.setExamMode(!adminModel.isExamMode());
        }
    }


    /**
     * Called when enter has been pressed
     * while the styleSheet-textfield
     * was focused.
     */
    public void styleSheetChanged() {
        String newValue = styleSheetField.getText();
        String oldValue = adminModel.getStyleSheet();

        if (newValue.equals(oldValue)) {
            return;
        }

        int choice = JOptionPane.showConfirmDialog(this,
                                      "The stylesheet will be set to: \"" + newValue + "\"\n" +
                                      "Proceed?",
                                      "Confirm!",
                                      JOptionPane.YES_NO_OPTION,
                                      JOptionPane.QUESTION_MESSAGE);

        if (choice == JOptionPane.YES_OPTION) {
            adminModel.setStyleSheet(newValue);
        }
        else {
            settingsChanged();
        }
    }


    /**
     * Called when a new value has been
     * selected in the Last-Question-ID-
     * ComboBox.
     */
    public void lastQuestionIDChanged() {
        Object selectedItem = lastQuestionComboBox.getSelectedItem();
        Integer newValue;
        Integer oldValue;

        if (!(selectedItem instanceof Integer)) {
            return;
        }

        newValue = (Integer)selectedItem;
        oldValue = adminModel.getLastQuestionID();
        if (newValue.equals(oldValue)) {
            return;
        }


        int choice = JOptionPane.showConfirmDialog(this,
                                      "Last question ID will be set to: " + newValue + "\n" +
                                      "Proceed?",
                                      "Confirm!",
                                      JOptionPane.YES_NO_OPTION,
                                      JOptionPane.QUESTION_MESSAGE);

        if (choice == JOptionPane.YES_OPTION) {
            adminModel.setLastQuestionID(newValue);
        }
        else {
            settingsChanged();
        }
    }


    /**
     * Called by the AdminModel when settings has been
     * changed and the data in the view needs to be
     * refreshed.
     */
    public void settingsChanged() {
        lastQuestionComboBox.removeActionListener(adminPanelListener);

        Integer lastQuestionID;
        ArrayList<Integer> questionIDs;
        boolean lastQuestionIDExists = false;

        /* Exam mode */
        examModeButton.setText(adminModel.isExamMode() ? "on" : "off");

        /* Style sheet */
        styleSheetField.setText(adminModel.getStyleSheet());

        /* Last Question ID */
        lastQuestionComboBox.removeAllItems();
        lastQuestionID = adminModel.getLastQuestionID();
        questionIDs = adminModel.getAllQuestionIDs();
        for (Integer questionID : questionIDs) {
            if (lastQuestionID.equals(questionID)) {
                lastQuestionIDExists = true;
            }
        }
        if (!lastQuestionIDExists) {
            lastQuestionComboBox.addItem("Not set");
        }
        for (Integer id : questionIDs) {
            lastQuestionComboBox.addItem(id);
        }
        lastQuestionComboBox.setSelectedItem(lastQuestionID);

        lastQuestionComboBox.addActionListener(adminPanelListener);


    }



    /* ----- Getters and setters ----- */


    public AdminModel getAdminModel() {
        return adminModel;
    }


    public void setAdminModel(AdminModel adminModel) {
        this.adminModel = adminModel;
        this.adminModel.setAdminPanel(this);
        this.userTableModel.setAdminModel(adminModel);
        this.levelIdsChanged();
        this.settingsChanged();
    }

    public void levelIdsChanged() {
        userTable.getColumn("Level").setCellEditor(new DefaultCellEditor(new JComboBox(adminModel.getLevelIdArray())));
    }



    /* ----- Initialization methods ----- */


    private void initSettingsPanel() {

        JLabel examModeLabel;
        JLabel styleSheetLabel;
        JLabel lastQuestionLabel;
        JPanel emptyFillPanel;

        GridBagLayout gridBagLayout;
        GridBagConstraints constraints;

        gridBagLayout = new GridBagLayout();
        constraints = new GridBagConstraints();

        settingsPanel.setLayout(gridBagLayout);
        settingsPanel.setBorder(new TitledBorder("Settings") {

            @Override
            public Insets getBorderInsets(Component c) {
                return new Insets(30, 15, 20, 15);
            }

        });

        /* Exam Mode */
        examModeLabel = new JLabel("Exam mode:");
        constraints.anchor = GridBagConstraints.WEST;
        constraints.insets = new Insets(5, 3, 5, 3);
        gridBagLayout.setConstraints(examModeLabel, constraints);
        settingsPanel.add(examModeLabel);
        examModeButton = new JButton();
        examModeButton.setActionCommand("EXAM MODE");
        examModeButton.addActionListener(adminPanelListener);
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        gridBagLayout.setConstraints(examModeButton, constraints);
        settingsPanel.add(examModeButton);

        /* Style sheet */
        styleSheetLabel = new JLabel("Style sheet:");
        constraints.gridwidth = 1;
        constraints.fill = GridBagConstraints.NONE;
        gridBagLayout.setConstraints(styleSheetLabel, constraints);
        settingsPanel.add(styleSheetLabel);
        styleSheetField = new JTextField(8);
        styleSheetField.setName("styleSheetField");
        styleSheetField.addActionListener(adminPanelListener);
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        gridBagLayout.setConstraints(styleSheetField, constraints);
        settingsPanel.add(styleSheetField);

        /* Last Question */
        lastQuestionLabel = new JLabel("Last question: ");
        constraints.gridwidth = 1;
        constraints.fill = GridBagConstraints.NONE;
        gridBagLayout.setConstraints(lastQuestionLabel, constraints);
        settingsPanel.add(lastQuestionLabel);
        lastQuestionComboBox = new JComboBox();
        lastQuestionComboBox.setName("lastQuestionComboBox");
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagLayout.setConstraints(lastQuestionComboBox, constraints);
        settingsPanel.add(lastQuestionComboBox);

        /* Fill for proper resizing behaviour */
        emptyFillPanel = new JPanel();
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        constraints.fill = GridBagConstraints.BOTH;
        constraints.weighty = 1;
        gridBagLayout.setConstraints(emptyFillPanel, constraints);
        settingsPanel.add(emptyFillPanel);


        settingsPanel.setMaximumSize(new Dimension(216, (int)(settingsPanel.getMaximumSize().getHeight())));
    }


    private void initUserPanel() {

        EmptyBorder userPanelEmptyBorder = new EmptyBorder(5, 20, 10, 20);
        JPanel buttonPanel, selectionPanel, filterPanel;
        JButton newButton, deleteButton, changePasswordButton;
        JButton selectAllButton, selectNoneButton;
        ButtonPanel bottomButtonPanel;

        userPanel.setBorder(new CompoundBorder(new TitledBorder("User administration"), userPanelEmptyBorder));
        userPanel.setLayout(new BoxLayout(userPanel, BoxLayout.Y_AXIS));
        userPanel.add(Box.createRigidArea(new Dimension(0, 10)));

        /* ButtonPanel */
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(Box.createHorizontalGlue());
        newButton = new JButton("New");
        newButton.addActionListener(adminPanelListener);
        newButton.setPreferredSize(new Dimension(137, 26));
        buttonPanel.add(newButton);
        buttonPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        deleteButton = new JButton("Delete");
        deleteButton.addActionListener(adminPanelListener);
        deleteButton.setPreferredSize(new Dimension(137, 26));
        buttonPanel.add(deleteButton);
        buttonPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        changePasswordButton = new JButton("Change password");
        changePasswordButton.addActionListener(adminPanelListener);
        changePasswordButton.setPreferredSize(new Dimension(137, 26));
        buttonPanel.add(changePasswordButton);
        buttonPanel.add(Box.createHorizontalGlue());
        userPanel.add(buttonPanel);
        userPanel.add(Box.createRigidArea(new Dimension(0, 10)));

        /* UserTableScrollPane */
        userTableScrollPane = new JScrollPane();
        initUserTableScrollPane();
        userPanel.add(userTableScrollPane);
        userPanel.add(Box.createRigidArea(new Dimension(0, 10)));

        /* SelectionPanel */
        selectionPanel = new JPanel();
        selectionPanel.setLayout(new BoxLayout(selectionPanel, BoxLayout.X_AXIS));
        selectionPanel.setBorder(new CompoundBorder(new TitledBorder("Selection"), userPanelEmptyBorder));

        selectionPanel.add(Box.createHorizontalGlue());
        selectAllButton = new JButton("Select All");
        selectAllButton.addActionListener(adminPanelListener);
        selectAllButton.setPreferredSize(new Dimension(102, 26));
        selectionPanel.add(selectAllButton);
        selectionPanel.add(Box.createRigidArea(new Dimension(10, 0)));
        selectNoneButton = new JButton("Select None");
        selectNoneButton.addActionListener(adminPanelListener);
        selectNoneButton.setPreferredSize(new Dimension(102, 26));
        selectionPanel.add(selectNoneButton);
        selectionPanel.add(Box.createHorizontalGlue());
        userPanel.add(selectionPanel);
        userPanel.add(Box.createRigidArea(new Dimension(0, 10)));

        /* FilterPanel */
        JLabel showLabel;
        JLabel searchForLabel;
        JLabel inLabel;
        JButton searchButton;
        JButton clearButton;
        GridBagLayout gridBagLayout;
        GridBagConstraints constraints;
        JPanel emptyFillPanel;

        filterPanel = new JPanel();
        gridBagLayout = new GridBagLayout();
        constraints = new GridBagConstraints();
        constraints.fill = GridBagConstraints.BOTH;
        filterPanel.setLayout(gridBagLayout);
        filterPanel.setBorder(new CompoundBorder(new TitledBorder("Filter"), userPanelEmptyBorder));

        showLabel = new JLabel("Show");
        constraints.anchor = GridBagConstraints.WEST;
        gridBagLayout.setConstraints(showLabel, constraints);
        filterPanel.add(showLabel);
        adminBox = new JCheckBox("Admin");
        adminBox.setPreferredSize(new Dimension(95, 26));
        gridBagLayout.setConstraints(adminBox, constraints);
        filterPanel.add(adminBox);
        nonAdminBox = new JCheckBox("Non-admin");
        nonAdminBox.setPreferredSize(new Dimension(95, 26));
        constraints.anchor = GridBagConstraints.EAST;
        gridBagLayout.setConstraints(nonAdminBox, constraints);
        filterPanel.add(nonAdminBox);
        emptyFillPanel = new JPanel();
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        gridBagLayout.setConstraints(emptyFillPanel, constraints);
        filterPanel.add(emptyFillPanel);
        searchForLabel = new JLabel("Search for");
        constraints.anchor = GridBagConstraints.WEST;
        constraints.gridwidth = 1;
        constraints.insets = new Insets(0, 0, 0, 7);
        gridBagLayout.setConstraints(searchForLabel, constraints);
        filterPanel.add(searchForLabel);
        searchField = new JTextField(15);
        searchField.setName("searchTextField");
        constraints.gridwidth = 2;
        constraints.insets = new Insets(0, 0, 0, 0);
        gridBagLayout.setConstraints(searchField, constraints);
        filterPanel.add(searchField);
        inLabel = new JLabel("in");
        constraints.anchor = GridBagConstraints.CENTER;
        constraints.insets = new Insets(0, 7, 0, 7);
        constraints.gridwidth = 1;
        gridBagLayout.setConstraints(inLabel, constraints);
        filterPanel.add(inLabel);
        fieldsComboBox = new JComboBox(new String[]{"Usernames", "Levels"});
        fieldsComboBox.setName("searchComboBox");
        constraints.insets = new Insets(0, 0, 0, 0);
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        gridBagLayout.setConstraints(fieldsComboBox, constraints);
        filterPanel.add(fieldsComboBox);
        emptyFillPanel = new JPanel();
        constraints.gridwidth = 1;
        gridBagLayout.setConstraints(emptyFillPanel, constraints);
        filterPanel.add(emptyFillPanel);
        searchButton = new JButton("Search");
        searchButton.addActionListener(adminPanelListener);
        searchButton.setPreferredSize(new Dimension(75, 26));
        constraints.anchor = GridBagConstraints.WEST;
        constraints.insets = new Insets(0, 7, 0, 7);
        gridBagLayout.setConstraints(searchButton, constraints);
        filterPanel.add(searchButton);
        clearButton = new JButton("Clear");
        clearButton.addActionListener(adminPanelListener);
        clearButton.setPreferredSize(new Dimension(75, 26));
        constraints.anchor = GridBagConstraints.EAST;
        gridBagLayout.setConstraints(clearButton, constraints);
        filterPanel.add(clearButton);
        emptyFillPanel = new JPanel();
        constraints.gridwidth = GridBagConstraints.REMAINDER;
        constraints.insets = new Insets(0, 0, 0, 0);
        gridBagLayout.setConstraints(emptyFillPanel, constraints);
        filterPanel.add(emptyFillPanel);

        filterPanel.setMaximumSize(new Dimension((int)(filterPanel.getMaximumSize().getWidth()), 225));
        userPanel.add(filterPanel);
        userPanel.add(Box.createRigidArea(new Dimension(0, 10)));

        bottomButtonPanel = new ButtonPanel(this);
        userPanel.add(bottomButtonPanel);
    }


    private void initUserTableScrollPane() {

        TableSorter sorter;

        userTableModel = new UserTableModel();
        userTableModel.addTableModelListener(this);
        sorter = new TableSorter(userTableModel);
        sorter.setSortingStatus(0, TableSorter.ASCENDING);
        userTable = new JTable(sorter);
        userTable.setAutoResizeMode(JTable.AUTO_RESIZE_NEXT_COLUMN);
        userTable.setCellSelectionEnabled(false);
        userTable.setColumnSelectionAllowed(false);
        userTable.setRowSelectionAllowed(true);
        userTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        sorter.setTableHeader(userTable.getTableHeader());

        userTableScrollPane.setViewportView(userTable);
    }



    /* ----- Other methods ----- */


    /**
     * Called when the "Select All"-button has been pressed.
     * Selects all rows in the userTable.
     */
    public void selectAll() {
        userTable.selectAll();
        userTable.requestFocusInWindow();
    }


    /**
     * Called when the "Select none"-button has been pressed.
     * Clears the selection in the userTable.
     */
    public void selectNone() {
        userTable.clearSelection();
    }



    /* ----- Methods from interface Saver ----- */


    /**
     * Called when the "Save"-button has been pressed.
     * Tells the AdminModel to save all changed objects
     * to database.
     */
    public void saveButtonPressed() {
        int editingRow = userTable.getEditingRow();
        if (editingRow != -1) {
            userTable.getCellEditor().stopCellEditing();
            userTable.getSelectionModel().setSelectionInterval(editingRow, editingRow);
        }
        adminModel.saveButtonPressed();
    }



    public void discardButtonPressed() {
        int choice;
        int editingRow = userTable.getEditingRow();
        if (editingRow != -1) {
            userTable.getCellEditor().stopCellEditing();
            userTable.getSelectionModel().setSelectionInterval(editingRow, editingRow);
        }

        choice = JOptionPane.showConfirmDialog(this,
                                               "This will reset changed users.\n" +
                                               "It will not delete created users or recreate deleted users.\n" +
                                               "Continue?",
                                               "Confirm",
                                               JOptionPane.YES_NO_OPTION);

        if (choice != JOptionPane.YES_OPTION) {
            return;
        }


        adminModel.discardButtonPressed();
    }


    /* ----- Methods from interface TableModelListener ----- */


    /**
     * This fine grain notification tells listeners the exact range
     * of cells, rows, or columns that changed.
     */
    public void tableChanged(TableModelEvent e) {
        switch (e.getType()) {
            case TableModelEvent.INSERT:
                int modelIndex = e.getFirstRow();
                try {
                    int viewIndex = ((TableSorter)userTable.getModel()).viewIndex(modelIndex);
                    userTable.getSelectionModel().setSelectionInterval(viewIndex, viewIndex);
                } catch (ArrayIndexOutOfBoundsException aioobe) {
                    aioobe.printStackTrace();
                }
                break;
            default:
                break;
        }

    }

    public String toString() {
        return "Admin Panel";
    }
}
