package gui;

import com.DatabaseCommunicator;
import com.ServerCommunicator;
import sys.AoCAdmin;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.util.ArrayList;

/**
 * Login-window
 *
 * @author Tarje Killingberg - 21.jun.2006
 */
public class LoginFrame extends JFrame implements ActionListener {

    private String[] recentDBServers;
    private String[] recentWebServers;

    private JPanel loginPanel;
    private JTextField usernameField;
    private JPasswordField passwordField;
    private JComboBox dbserverComboBox;
    private JComboBox webserverComboBox;


    /**
     * Creates a new LoginFrame
     */
    public LoginFrame() {
        super("Login");
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        fetchRecentServers();
        initLoginFrame();
        setVisible(true);
    }

    /**
     * Tries to log in with the entered information.
     */
    private void attemptLogin() {
        int validation;
        boolean loginOK = false;

        if (!((String)webserverComboBox.getEditor().getItem()).endsWith("/")) {
            webserverComboBox.getEditor().setItem(webserverComboBox.getEditor().getItem().toString() + "/");
            webserverComboBox.setSelectedItem(webserverComboBox.getEditor().getItem());
        }
        if (!((String)webserverComboBox.getEditor().getItem()).contains(":")) {
            webserverComboBox.getEditor().setItem("http://" + webserverComboBox.getEditor().getItem().toString());
            webserverComboBox.setSelectedItem(webserverComboBox.getEditor().getItem());
        }

        String username = usernameField.getText();
        String password = new String(passwordField.getPassword());
        String dbserver = (String)dbserverComboBox.getEditor().getItem();
        String webserver = (String)webserverComboBox.getEditor().getItem();
        validation = validateValues(username, password, dbserver, webserver);

        if (validation == 0) {
            loginOK = true;
        }
        else if (validation == 1) {
            JOptionPane.showMessageDialog(this,
                                          "Could not connect with database " + dbserver + ".",
                                          "Invalid values",
                                          JOptionPane.ERROR_MESSAGE);
            loginOK = false;

        }
        else if (validation == 2) {
            int choice = JOptionPane.showConfirmDialog(this,
                                                       "A folder named " + AoCAdmin.DATA_FOLDER + " could not\n" +
                                                       "be found on server " + webserver + ".\n" +
                                                       "Login anyway?",
                                                       "Confirm!",
                                                       JOptionPane.YES_NO_OPTION,
                                                       JOptionPane.QUESTION_MESSAGE);

            loginOK = choice == JOptionPane.YES_OPTION;

        }
        else {
            JOptionPane.showMessageDialog(this,
                                          "Validation of one or more values failed.",
                                          "Validation",
                                          JOptionPane.ERROR_MESSAGE);

        }

        if (loginOK) {
            putRecentServers();
            AoCAdmin.DATABASE_SERVER = dbserver;
            AoCAdmin.DATABASE_USERNAME = username;
            AoCAdmin.DATABASE_PASSWORD = password;
            AoCAdmin.WEB_SERVER = webserver;
            this.dispose();
            AoCAdmin.LOGIN_OK = true;
        }
    }

    /**
     * Controls whether the entered values are legal
     *
     * @param username The username to log in
     * @param password The password
     * @param dbserver The address of the database server
     * @param webserver The address of the webserver
     * @return If the information may be used to log in.
     */
    private int validateValues(String username, String password, String dbserver, String webserver) {

        /* checking if we have access to the database with the given values */
        if (!DatabaseCommunicator.connect(dbserver, username, password)) {
            return 1;
        }
        DatabaseCommunicator.disconnect();

        /* checking if the given webserver has a folder named AoCAdmin.DATA_FOLDER */
        try {
            InputStream is = ServerCommunicator.getInputStream(webserver, AoCAdmin.DATA_FOLDER);

            try {
                is.close();
            } catch (IOException e) {
            }
        } catch (IOException ioe) {
            return 2;
        } catch (IllegalArgumentException e) {
            return 2;
        }

        return 0;
    }

    /**
     * Gets the list of previously entered servers, and diaplys these in the combobox;
     * with the last used server as default selection.
     */
    private void fetchRecentServers() {
        ArrayList<String> recentDBServers = new ArrayList<String>();
        ArrayList<String> recentWebServers = new ArrayList<String>();
        BufferedReader reader;
        try {
            reader = new BufferedReader(new FileReader("recent_servers.txt"));

            String line = getRelevantLine(reader);
            while (line != null && !line.toUpperCase().startsWith("DATABASESERVERS:")) {
                line = getRelevantLine(reader);
            }
            line = getRelevantLine(reader);
            while (line != null && !line.toUpperCase().startsWith("WEBSERVERS:")) {
                recentDBServers.add(line);
                line = getRelevantLine(reader);
            }
            line = getRelevantLine(reader);
            while (line != null) {
                recentWebServers.add(line);
                line = getRelevantLine(reader);
            }
            reader.close();
        } catch (IOException e) {
        }

        this.recentDBServers = new String[recentDBServers.size()];
        this.recentWebServers = new String[recentWebServers.size()];
        recentDBServers.toArray(this.recentDBServers);
        recentWebServers.toArray(this.recentWebServers);
    }

    /**
     * Writes the list of servers to a txt-file, so that it can be
     * used on later logins.
     */
    public void putRecentServers() {
        FileWriter writer;
        String fileContent;
        String thisServer;

        try {
            writer = new FileWriter("recent_servers.txt", false);
            fileContent = "# This file contains the servers to which a previous\n" +
                          "# connection successfully has been established.\n" +
                          "\n" +
                          "DATABASESERVERS:\n";

            thisServer = (String)dbserverComboBox.getEditor().getItem();
            fileContent += thisServer + "\n";

            for (String server : recentDBServers) {
                if (!server.equalsIgnoreCase(thisServer)) {
                    fileContent += server + "\n";
                }
            }

            fileContent += "\n" +
                          "WEBSERVERS:\n";

            thisServer = (String)webserverComboBox.getEditor().getItem();
            fileContent += thisServer + "\n";
            for (String server : recentWebServers) {
                if (!server.equalsIgnoreCase(thisServer)) {
                    fileContent += server + "\n";
                }
            }
            writer.write(fileContent);
            writer.close();
        } catch (IOException e) {
        }
    }

    /**
     * Checks whether a line is empty, or contains '#' - if so, the line
     * is discarded.
     *
     * @param reader
     * @return The next line containing relevant information
     */
    public String getRelevantLine(BufferedReader reader) {
        String line = null;
        try {
            if (reader.ready()) {
                line = reader.readLine();
                while (line.startsWith("#") || line.equalsIgnoreCase("")) {
                    if (reader.ready()) {
                        line = reader.readLine();
                    }
                    else {
                        line = null;
                    }
                }
            }
        } catch (IOException e) {
            line = null;
        } catch (NullPointerException e) {

        }
        return line;
    }

    /**
     * Initializes the frame.
     */
    private void initLoginFrame() {
        JPanel contentPane;
        Box titleBox;
        JLabel titleLabel;
        Box loginBox;
        Box buttonBox;

        contentPane = new JPanel();
        contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));
        contentPane.add(Box.createVerticalGlue());

        titleBox = new Box(BoxLayout.X_AXIS);
        titleBox.add(Box.createHorizontalGlue());
        titleLabel = new JLabel("AoC Admin");
        titleLabel.setFont(new Font("Monospaced", Font.BOLD, 50));
        titleBox.add(titleLabel);
        titleBox.add(Box.createHorizontalGlue());
        contentPane.add(titleBox);
        contentPane.add(Box.createRigidArea(new Dimension(0, 20)));

        loginBox = new Box(BoxLayout.X_AXIS);
        loginBox.add(Box.createHorizontalGlue());
        loginPanel = new JPanel();
        initLoginPanel();
        loginBox.add(loginPanel);
        loginBox.add(Box.createHorizontalGlue());
        contentPane.add(loginBox);
        contentPane.add(Box.createRigidArea(new Dimension(0, 20)));

        buttonBox = new Box(BoxLayout.X_AXIS);
        buttonBox.add(Box.createHorizontalGlue());
        JButton loginButton = new JButton("Login");
        loginButton.addActionListener(this);
        buttonBox.add(loginButton);
        buttonBox.add(Box.createRigidArea(new Dimension(20, 0)));
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(this);
        buttonBox.add(cancelButton);
        buttonBox.add(Box.createHorizontalGlue());
        contentPane.add(buttonBox);
        contentPane.add(Box.createVerticalGlue());

        setContentPane(contentPane);
        pack();
        setLocationRelativeTo(null);
        setResizable(false);
    }

    /**
     * Initializes the components of the login frame.
     */
    private void initLoginPanel() {
        JLabel usernameLabel, passwordLabel, dbserverLabel, webserverLabel;
        GridLayout gridLayout;
        int fontSize = 18;
        Dimension fieldAndBoxDimension = new Dimension(200, 24);

        gridLayout = new GridLayout(4, 2, 2, 5);
        loginPanel.setLayout(gridLayout);

        usernameLabel = new JLabel("Username:");
        usernameLabel.setFont(new Font("Monospaced", Font.PLAIN, fontSize));
        loginPanel.add(usernameLabel);

        usernameField = new JTextField("", 20);
        usernameField.setPreferredSize(fieldAndBoxDimension);
        usernameField.addActionListener(this);
        loginPanel.add(usernameField);

        passwordLabel = new JLabel("Password:");
        passwordLabel.setFont(new Font("Monospaced", Font.PLAIN, fontSize));
        loginPanel.add(passwordLabel);

        passwordField = new JPasswordField("", 20);
        passwordField.setPreferredSize(fieldAndBoxDimension);
        passwordField.addActionListener(this);
        loginPanel.add(passwordField);

        dbserverLabel = new JLabel("Database server:");
        dbserverLabel.setFont(new Font("Monospaced", Font.PLAIN, fontSize));
        loginPanel.add(dbserverLabel);

        dbserverComboBox = new JComboBox(recentDBServers);
        dbserverComboBox.setPreferredSize(fieldAndBoxDimension);
        dbserverComboBox.setEditable(true);
        dbserverComboBox.getEditor().addActionListener(this);
        loginPanel.add(dbserverComboBox);

        webserverLabel = new JLabel("Web server:");
        webserverLabel.setFont(new Font("Monospaced", Font.PLAIN, fontSize));
        loginPanel.add(webserverLabel);

        webserverComboBox = new JComboBox(recentWebServers);
        webserverComboBox.setPreferredSize(fieldAndBoxDimension);
        webserverComboBox.setEditable(true);
        webserverComboBox.getEditor().addActionListener(this);
        loginPanel.add(webserverComboBox);
    }



    /* ----- Methods from interface ActionListener ----- */


    /**
     * Called when an action has occured in the login-frame,
     * i.e. enter has been pressed or a button has been clicked.
     *
     * @param e the event that occured
     */
    public void actionPerformed(ActionEvent e) {
        String actionCommand = e.getActionCommand();

        if (actionCommand.equalsIgnoreCase("Cancel")) {
            this.dispose();
            System.exit(0);
        }
        else {
            attemptLogin();
        }
    }
}
