package gui;

import gui.listeners.MenuBarListener;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;


/**
 * The main frame of the application.
 * The GUI is created from this class, and the bottom
 * components are placed here.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 * @see StatusBar
 * @see JMenuBar
 * @see Content
 */
public class MainFrame extends JFrame implements WindowListener {


    private StatusBar statusBar;
    private JMenuBar menuBar;
    private Content content;
    private JMenuItem traceMenuItem;
    private JDialog aboutDialog;
    private JDialog helpDialog;


    public MainFrame() {
        super("AoC Admin");
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setPreferredSize(new Dimension(1024, 800));
        setLayout(new GridLayout(1, 1));

        JPanel contentPane;
        contentPane = new JPanel();
        contentPane.setLayout(new BorderLayout());

        DBProgressMonitor.init(this);

        content = new Content(this);

        statusBar = new StatusBar();
        contentPane.add(statusBar, BorderLayout.SOUTH);

        aboutDialog = new JDialog(this, true);
        helpDialog = new JDialog(this);

        menuBar = new JMenuBar();
        initMenuBar();
        setJMenuBar(menuBar);

        this.addWindowListener(this);

        contentPane.add(content, BorderLayout.CENTER);
        setContentPane(contentPane);
        pack();
        setLocationRelativeTo(null);

        setVisible(true);
        SQLTrace.show();
        SQLTrace.addWindowListener(this);
        requestFocus();
    }


    /**
     * Sets the text on the statusbar.
     */
    public void printStatus(String status) {
        statusBar.setText(status);
    }

    /**
     * Initializes the menu bar with all its components and listeners.
     */
    private void initMenuBar() {
        MenuBarListener menuBarListener;
        menuBarListener = new MenuBarListener(this);

        /* File-menu */
        JMenu file = new JMenu("File");
        JMenuItem[] fileItems = new JMenuItem[4];
        fileItems[0] = new JMenuItem("New");
        fileItems[0].addActionListener(menuBarListener);
        fileItems[0].setMnemonic(KeyEvent.VK_N);
        fileItems[0].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
        fileItems[1] = new JMenuItem("Delete");
        fileItems[1].addActionListener(menuBarListener);
        fileItems[1].setMnemonic(KeyEvent.VK_D);
        fileItems[1].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
        fileItems[2] = new JMenuItem("Reload DB");
        fileItems[2].setActionCommand("reload");
        fileItems[2].setMnemonic(KeyEvent.VK_R);
        fileItems[2].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
        fileItems[2].addActionListener(menuBarListener);
        fileItems[3] = new JMenuItem("Exit");
        fileItems[3].addActionListener(menuBarListener);
        fileItems[3].setMnemonic(KeyEvent.VK_E);
        fileItems[3].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.CTRL_MASK));
        file.add(fileItems[0]);
        file.add(fileItems[1]);
        file.add(new JSeparator(JSeparator.HORIZONTAL));
        file.add(fileItems[2]);
        file.add(new JSeparator(JSeparator.HORIZONTAL));
        file.add(fileItems[3]);

        /* View-menu */
        JMenu view = new JMenu("View");
        ButtonGroup group = new ButtonGroup();
        JRadioButtonMenuItem[] viewItems = new JRadioButtonMenuItem[2];
        viewItems[0] = new JRadioButtonMenuItem("Editor");
        viewItems[0].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.CTRL_MASK));
        viewItems[0].setSelected(true);
        content.showTabPanel();
        viewItems[0].setMnemonic(KeyEvent.VK_E);
        viewItems[0].addActionListener(menuBarListener);

        group.add(viewItems[0]);
        view.add(viewItems[0]);
        viewItems[1] = new JRadioButtonMenuItem("Admin");
        viewItems[1].setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, ActionEvent.CTRL_MASK));
        viewItems[1].setSelected(false);
        viewItems[1].setMnemonic(KeyEvent.VK_A);
        viewItems[1].addActionListener(menuBarListener);
        group.add(viewItems[1]);
        view.add(viewItems[1]);
        view.add(new JSeparator(JSeparator.HORIZONTAL));
        traceMenuItem = new JCheckBoxMenuItem("SQL trace");
        traceMenuItem.setSelected(true);
        traceMenuItem.setMnemonic(KeyEvent.VK_T);
        traceMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));
        traceMenuItem.addActionListener(menuBarListener);
        view.add(traceMenuItem);


        /* Help-menu */
        JMenu help = new JMenu("Help");
        JMenuItem[] helpItems = new JMenuItem[2];
        helpItems[0] = new JMenuItem("Help");
        helpItems[0].addActionListener(menuBarListener);
        help.add(helpItems[0]);
        helpItems[1] = new JMenuItem("About");
        helpItems[1].addActionListener(menuBarListener);
        help.add(helpItems[1]);

        menuBar.add(file);
        menuBar.add(view);
        menuBar.add(help);


        /* The About-Dialog */
        aboutDialog = new AboutDialog(this, menuBarListener);


        /* The Help-Dialog */
        helpDialog = new HelpDialog(this, menuBarListener);
    }

    /**
     * Hides or shows the dialog window in question
     * 
     * @param dialog The dialog to hode or show
     * @param visible Whether it should be shown or not.
     */
    public void setDialogVisible(String dialog, boolean visible) {
        if (dialog.equalsIgnoreCase("ABOUT")) {
            aboutDialog.setVisible(visible);
        }
        else if (dialog.equalsIgnoreCase("HELP")) {
            helpDialog.setVisible(visible);
        }
    }


    public Content getContent() {
        return content;
    }

    /**
     * Methos called when the user wants to switch between Admin-
     * and Editor view.
     *
     * @param content The view to display
     */
    public void setContent(String content) {
        if (content.equalsIgnoreCase("editor")) {
            this.content.showTabPanel();
            this.statusBar.setStatus(this.content.getStatus());
        }
        else if (content.equalsIgnoreCase("admin")) {
            this.content.showAdminPanel();
            this.statusBar.setText("Admin View");
        }
    }


    public void reload() {
        this.content.reload();
    }

    /**
     * Exits the application, but saves the current object before closing.
     */
    public void exit() {
        this.content.getTabPanel().getActiveTab().saveButtonPressed();
        System.exit(0);
    }

    public StatusBar getStatusBar() {
        return statusBar;
    }

    /* Methods implemented from WindowListener */

    public void windowOpened(WindowEvent e) {}

    public void windowClosing(WindowEvent e) {
        if (e.getSource() == this) {
            exit();
        }
        else {
            traceMenuItem.setSelected(false);
        }
    }

    public void windowClosed(WindowEvent e) {}

    public void windowIconified(WindowEvent e) {}

    public void windowDeiconified(WindowEvent e) {}

    public void windowActivated(WindowEvent e) {}

    public void windowDeactivated(WindowEvent e) {}
}
