package gui;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.WindowListener;

/**
 * A static class providing a frame which shows all
 * SQL-queries executed on the database, except for
 * SELECT-queries.
 * If a query shows up in the frame, and it is not
 * followed by an error message, the user can be
 * certain that the query was indeed excetuted on
 * the database.
 *
 * @author Tarje Killingberg - 14.jul.2006
 */
public class SQLTrace {
    private static boolean isInitialized = false;
    private static long previous;
    private static JFrame traceFrame;
    private static JTextPane traceField;

    private static AttributeSet fieldAtt;
    private static AttributeSet errAtt;

    public static void init() {
        if (isInitialized) {
            return;
        }
        isInitialized = true;

        JPanel contentPane;
        JScrollPane traceScrollPane;

        traceFrame = new JFrame();
        traceFrame.setTitle("SQL Trace");
        contentPane = new JPanel();
        contentPane.setLayout(new BorderLayout());
        traceField = new JTextPane();
        traceField.setEditable(false);
        traceField.setFont(new Font("Monospaced", Font.PLAIN, 12));
        traceScrollPane = new JScrollPane(traceField);
        traceScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        traceScrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        contentPane.add(traceScrollPane, BorderLayout.CENTER);
        traceFrame.setContentPane(contentPane);
        traceFrame.setSize(600, 400);
        traceFrame.setLocation(0, 0);
        previous = System.currentTimeMillis();

        StyleContext sc = StyleContext.getDefaultStyleContext();
        AttributeSet defAtt = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, Color.BLACK);

        fieldAtt = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, Color.BLUE);
        errAtt = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, Color.RED);

        traceField.setParagraphAttributes(defAtt, false);
    }


    /**
     * Retuns whether the SQL Trace is visible.
     *
     * @return whether the SQL Trace is visible
     */
    public static boolean isVisible() {
        return traceFrame.isVisible();
    }


    /**
     * Shows the SQL Trace frame.
     */
    public static void show() {
        if (traceFrame.isVisible()) {
            return;
        }
        traceFrame.setVisible(true);
    }


    /**
     * Hides the SQL Trace franme
     */
    public static void hide() {
        if (!traceFrame.isVisible()) {
            return;
        }
        traceFrame.setVisible(false);
    }


    /**
     * Prints some text on the SQL Trace frame,
     * followed by a newline.
     *
     * @param query the text to print.
     */
    public static void println(String query) {
        if (!query.toUpperCase().startsWith("SELECT")) {
            if ((System.currentTimeMillis() - previous) / 1000 > 5) {
                printSeparator();
            }

            Document doc = traceField.getDocument();
            try {
                //Removes line breaks, to reduce the query length
                String print = query.replace("\n", "");

                //Finds the first pair of '-s
                int begin_value = print.indexOf("'");
                int end_value = print.indexOf("'", begin_value +1);

                //Make the field names easy to read..
                if (begin_value >= 0) {
                    formatQuery(doc, print.substring(0, begin_value));
                }
                else {
                    formatQuery(doc, print);
                }

                String s;

                //Search through the query looking for '-s
                while (begin_value >= 0 && begin_value < print.length()) {

                    //Make sure escapted '-s does not belong with the last '
                    end_value = avoidEscapedCharacters(print, end_value);

                    //Attributes longer than 30 characters are shortened
                    int textlength = 30;
                    if (end_value - begin_value >= textlength) {
                        s = print.substring(begin_value, begin_value + textlength) + "......";
                    }
                    else {
                        s = print.substring(begin_value, end_value) + "";
                    }

                    //Insert the formatted part of the string
                    doc.insertString(doc.getLength(), s, null);

                    //Find the next pair of '-s
                    begin_value = print.indexOf("'", end_value +1);

                    //Find the field names
                    if (begin_value != -1) {
                        formatQuery(doc, print.substring(end_value, begin_value));
                    }
                    else formatQuery(doc, print.substring(end_value));

                    end_value = print.indexOf("'", begin_value +1);
                }

                doc.insertString(doc.getLength(), "\n", null);
                traceField.setCaretPosition(doc.getLength());

            } catch (BadLocationException e) {
            }
        }
    }

    /**
     * Searches through a string looking for the first not-escaped '.
     * Used to find the ' marking the end of a text value.
     *
     * @param print The string to search
     * @param end_value The starting point of the search
     * @return The position of the '
     */
    private static int avoidEscapedCharacters(String print, int end_value) {
        //Checking for ' within the value
        boolean escape = print.substring(end_value -1, end_value +1).equals("\\'");

        while (escape) {
            end_value = print.indexOf("'", end_value +1);
            escape = print.substring(end_value -1, end_value +1).equals("\\'");
        }
        return end_value;
    }

    /**
     * Searches through a string looking for strings matching the pattern "xxxx=".
     * Used to make the field names in a query appear in another color.
     *
     * @param doc The document displaying the text
     * @param string The string to search
     * @throws BadLocationException
     */
    private static void formatQuery(Document doc, String string) throws BadLocationException {
        int header_end = string.indexOf("=");
        int header_begin = string.lastIndexOf(" ", header_end);
        int start = 0;

        if (header_end < 0) doc.insertString(doc.getLength(), string, null);

        while (header_end >= 0) {
            doc.insertString(doc.getLength(), string.substring(start, header_begin +1), null);
            doc.insertString(doc.getLength(), string.substring(header_begin +1, header_end), fieldAtt);

            start = header_end;
            header_end = string.indexOf("=", header_end +1);
            if (header_end >= 0) {
                header_begin = string.lastIndexOf(" ", header_end);
            }
            else {
                doc.insertString(doc.getLength(), string.substring(start), null);
            }
        }
    }

    /**
     * Shows SQL-exceptions in red font.
     *
     * @param exception The exception to print
     */
    public static void printException(String exception) {
        StyledDocument doc = traceField.getStyledDocument();

        try {
            doc.insertString(doc.getLength(), exception + "\n", errAtt);
        } catch (BadLocationException e) {
            e.printStackTrace();
        }
    }


    /**
     * This method is called whenever the
     * println()-method is called, and it
     * is more than five seconds since the
     * last separator.
     */
    private static void printSeparator() {
        previous = System.currentTimeMillis();
        long s = previous / 1000;
        long m = s / 60;
        long h = m / 60;
        s = s % 60;
        m = m % 60;
        h = (h + 2) % 24;

        String ss = s < 10 ? "0" + s : "" + s;
        String mm = m < 10 ? "0" + m : "" + m;
        String hh = h < 10 ? "0" + h : "" + h;

        println("");
        println("-------------------------------- " + hh + ":" + mm + ":" + ss + " --------------------------------\n");
    }


    public static void addWindowListener(WindowListener listener) {
        traceFrame.addWindowListener(listener);
    }
}
