package gui.editors.common;

import gui.editors.map.MapPanel;
import sys.datatypes.GroupElement;
import sys.datatypes.MapPosition;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.models.LevelModel;
import sys.models.treemodels.AbstractTreeModel;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

/**
 * Superclass for all panels containing a position- and level selector.
 *
 * @author Tarje Killingberg - 29.jun.2006
 */
public abstract class InfoPanel extends DPanel implements ActionListener {
    protected JTextField xPos;
    protected JTextField yPos;
    protected JTextField zPos;
    protected JComboBox levelBox;
    protected JButton changePositionButton;
    protected JPanel infoPanel;
    protected AbstractTreeModel model;
    private MapPanel mapPanel;

    private MapPosition mapPositionBackup;

    /**
     * Creates a new <code>JPanel</code> with a double buffer
     * and a flow layout.
     */
    protected InfoPanel() {
        super();
        this.xPos = new JTextField();
        this.xPos.addActionListener(this);
        this.yPos = new JTextField();
        this.yPos.addActionListener(this);
        this.zPos = new JTextField();
        this.zPos.addActionListener(this);
        this.levelBox = new JComboBox();
        this.levelBox.addActionListener(this);
        mapPositionBackup = new MapPosition();

        changePositionButton = new JButton("...");
        changePositionButton.addActionListener(this);

        infoPanel = new JPanel();
    }

    /**
     * Updates the components with the current object's positional attributes (position and level).
     *
     * @param object The object to display
     */
    public void showObject(MapItem object) {
        levelBox.removeActionListener(this);
        setMapPosition(object.getMapPosition());
        levelBox.setSelectedItem(object.getLevelModel());
        levelBox.addActionListener(this);
    }

    public void setMapPosition(MapPosition mapPosition) {
        xPos.setText(Integer.toString(mapPosition.getX()));
        xPos.setCaretPosition(0);
        yPos.setText(Integer.toString(mapPosition.getY()));
        yPos.setCaretPosition(0);
        zPos.setText(Integer.toString(mapPosition.getZ()));
        zPos.setCaretPosition(0);

        mapPositionBackup = mapPosition;
    }

    public void setModel(AbstractTreeModel model) {
        this.model = model;
        modelChanged();
    }


    /**
     * Called when the user has changed an items position
     * through selector-mappanel or textfields
     */
    public void mapPositionChanged() {
        AoCObject currentObject = model.getModelManager().getCurrentObject();
        if (currentObject instanceof MapItem) {
            model.getModelManager().fireObjectParentUpdated(currentObject);
        }
    }

    /**
     * Called whenever the LevelModel has been updated, so that the list of levels
     * needs to be refreshed.
     */
    public void modelChanged() {
        LevelModel lm = (LevelModel)model.getModelManager().getSuperLevelModel().getRoot();
        ArrayList<GroupElement> groupElements = lm.getGroupElements();
        ArrayList<GroupElement> sortedGroupElements = new ArrayList<GroupElement>(groupElements);

        Collections.sort(sortedGroupElements, new Comparator() {
            public int compare(Object o1, Object o2) {
                return o1.toString().compareToIgnoreCase(o2.toString());
            }
        });

        Object selectedItem = levelBox.getModel().getSelectedItem();

        levelBox.removeActionListener(this);
        levelBox.removeAllItems();

        for (GroupElement groupElement : sortedGroupElements){
            levelBox.addItem(groupElement);
        }

        if (selectedItem != null) {
            levelBox.getModel().setSelectedItem(selectedItem);
        }

        levelBox.addActionListener(this);
    }


    public LevelModel getLevelModel() {
        return (LevelModel)levelBox.getSelectedItem();
    }

    public AbstractTreeModel getModel() {
        return model;
    }


    /**
     * Gets the coordinates currently in the position-fields
     * wrapped in a MapPosition-object, or null if a
     * NumberFormatException is thrown (meaning there were
     * not only numbers in the fields)
     *
     * @return the MapPosition from the TextFields
     */
    public MapPosition getMapPosition() {
        boolean showErrorMsg = false;

        try {
            mapPositionBackup.setX(Integer.parseInt(xPos.getText()));
        } catch (NumberFormatException e) {
            showErrorMsg = true;
        }

        try {
            mapPositionBackup.setY(Integer.parseInt(yPos.getText()));
        } catch (NumberFormatException e) {
            showErrorMsg = true;
        }

        try {
            mapPositionBackup.setZ(Integer.parseInt(zPos.getText()));
        } catch (NumberFormatException e) {
            showErrorMsg = true;
        }

        if (showErrorMsg) {
            JOptionPane.showMessageDialog(null,
                                          "The contents of the position-fields will be\n" +
                                          "reset to the previous values with correct format",
                                          "Illegal position format",
                                          JOptionPane.ERROR_MESSAGE);

            setMapPosition(mapPositionBackup);
        }

        return mapPositionBackup;

    }

    /**
     * Closes the map frame.
     */
    public void closeMapPositionChooser() {
        mapPanel.closeFrame();
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == changePositionButton) {
            
            mapPanel = new MapPanel(true, this);
            mapPanel.setModel(((MapItem)model.getModelManager().getCurrentObject()).getLevelModel());
            MapPosition itemMP = getMapPosition();
            mapPanel.setCenter(itemMP.getX(), itemMP.getY());

        }
        else if (e.getSource().equals(levelBox)) {

            AoCObject currentObject = model.getModelManager().getCurrentObject();

            if (currentObject instanceof MapItem) {
                MapItem mi = (MapItem)currentObject;
                if (!getLevelModel().equals(mi.getLevelModel())) {
                    mi.setLevelModel(getLevelModel());

                    model.getModelManager().fireObjectParentUpdated(currentObject);
                    model.moveToLevel((LevelModel)levelBox.getSelectedItem());
                }
            }
            else if (currentObject == null) {
            }
            else if (currentObject instanceof LevelModel) {
            }
            else {
            }

        }
        else if (e.getSource().equals(xPos) || e.getSource().equals(yPos) || e.getSource().equals(zPos)) {
            mapPositionChanged();
        }
    }

    /**
     * Method to make the positional fields uneditable if
     * the object represented in this panel does not correspond with
     * the application's currently selected object.
     *
     * @param moveable Whether or not the fields should be uneditable
     */
    public void setMovable(boolean moveable) {
        this.xPos.setEnabled(moveable);
        this.yPos.setEnabled(moveable);
        this.zPos.setEnabled(moveable);
        this.levelBox.setEnabled(moveable);
        this.changePositionButton.setEnabled(moveable);
    }
}
