package gui.editors.common.selector;

import sys.models.treemodels.AbstractTreeModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;

/**
 * Abstract class used to generate different popups used in the application.
 *
 * @author Eivind Sorteberg - 04.jul.2006
 */
public abstract class AbstractSelector implements ActionListener {
    protected JPanel parent;
    protected AbstractTreeModel model;
    protected JButton ok;
    protected JButton cancel;
    protected JPanel contents;
    protected JComponent selector;

    private JDialog d;

    /**
     * Constructor. Should be called by all inheriting subclasses.
     *
     * @param parent The component to which this is added.
     * @param model The model on which this is based.
     */
    public AbstractSelector(JPanel parent, AbstractTreeModel model) {

        this.model = model;
        this.parent = parent;
        this.selector = createSelector();

        this.contents = placeComponents();

        Point p = getPosition(parent);

        d = new JDialog(getFrame(parent), true);
        d.add(contents);
        d.setLocation((int)p.getX(), (int)p.getY());
        d.setUndecorated(true);
        d.pack();
        d.setVisible(true);
    }

    /**
     * Iterates through the layers of components until it finds the JFrame
     * at the "bottom" of the application.
     *
     * NOTE: Will hang execution if the application has no JFrame.
     *
     * @param comp The starting point of the iterations.
     * @return The JFrame of the application.
     */
    private JFrame getFrame(Component comp) {
        while (!(comp instanceof JFrame)) comp = comp.getParent();

        return (JFrame)comp;
    }

    /**
     * Removes the popup and resets it, so that it can be collected
     * by the garbage collector.
     */
    public void remove() {
        d.setVisible(false);
        d = null;
    }

    /**
     * Creates the specific component used to select an alternative in the popup.
     * The kind of component created will depend on its function.
     *
     * @return The component needed.
     */
    protected abstract JComponent createSelector();

    protected abstract JComponent getSelectorComponent();

    /**
     * Sets the text string describing the specific popup's function.
     *
     * @return A describing text.
     */
    protected abstract String setTitle();

    /**
     * Places the components of the popup.
     *
     * @return A panel containing the visible components on the popup.
     */
    private JPanel placeComponents() {
        JPanel contentPane = new JPanel();
        contentPane.setLayout(new GridLayout(1, 1));
        contentPane.setBorder(BorderFactory.createRaisedBevelBorder());

        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        JPanel south = new JPanel();
        south.setLayout(new BoxLayout(south, BoxLayout.X_AXIS));

        this.ok = new JButton("OK");
        this.ok.addActionListener(this);

        this.cancel = new JButton("Cancel");
        this.cancel.addActionListener(this);

        this.ok.setPreferredSize(new Dimension(67, 24));
        this.cancel.setPreferredSize(new Dimension(67, 24));

        south.add(Box.createHorizontalGlue());
        south.add(ok);
        south.add(cancel);
        south.add(Box.createHorizontalGlue());

        panel.add(new JLabel(setTitle()), BorderLayout.NORTH);
        panel.add(south, BorderLayout.SOUTH);
        panel.add(getSelectorComponent(), BorderLayout.CENTER);

        contentPane.add(panel);

        return contentPane;
    }

    /**
     * Finds the position of the popup based on the on-screen location
     * of the application. (Popup should be centered within the application.)
     *
     * @param comp Some component on the application.
     * @return The correct position for the popup.
     */
    private Point getPosition(Component comp) {
        comp = getFrame(comp);

        int frameX = (int)comp.getLocationOnScreen().getX();
        int frameY = (int)comp.getLocationOnScreen().getY();
        int frameWidth = comp.getWidth();
        int frameHeight = comp.getHeight();
        int centerX = frameX + frameWidth / 2;
        int centerY = frameY + frameHeight / 2;
        int popupWidth = (int)contents.getPreferredSize().getWidth();
        int popupHeight = (int)contents.getPreferredSize().getHeight();
        int x = centerX - popupWidth / 2;
        int y = centerY - popupHeight / 2;

        return new Point(x, y);
    }
}
