package gui.editors.common.util;

import sys.datatypes.interfaces.AoCObject;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;

/**
 * A class for generating a panel showing where conflicting objects differ. Used
 * whenever the user's local representation of the DB-object differs from the one actually
 * in the database (to avoid overwriting other person's work).
 *
 * @see sys.models.ModelManager
 * @author Eivind Sorteberg - 19.jul.2006
 */
public class ConflictViewGenerator {

    /**
     * Generated a panel with two textpanes where one textpane shows the local representation
     * of the object, while the other shows the object stored in DB. The differing fields are
     * shown in another color.
     *
     * @param dbObject  DB representation of the object
     * @param newObject Local representation of the object
     * @return A panel showing the objects, and where they differ
     */
    public static JPanel showConflict(AoCObject dbObject, AoCObject newObject) {

        Object[][] dbAttributes = dbObject.getAttributeArray();
        Object[][] localAttributes = newObject.getAttributeArray();

        JTextPane dbInfo = new JTextPane();
        JTextPane localInfo = new JTextPane();

        StyleContext sc = StyleContext.getDefaultStyleContext();
        AttributeSet defAtt = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, Color.BLACK);
        defAtt = sc.addAttribute(defAtt, StyleConstants.FontFamily, "Courier");
        defAtt = sc.addAttribute(defAtt, StyleConstants.Size, 11);

        AttributeSet confAtt = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, Color.RED);

        AttributeSet newlineAtt = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Size, 6);

        AttributeSet titleAtt = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Bold, true);
        titleAtt = sc.addAttribute(titleAtt, StyleConstants.Size, 14);

        StyledDocument dbDoc = dbInfo.getStyledDocument();
        StyledDocument localDoc = localInfo.getStyledDocument();

        dbInfo.setParagraphAttributes(defAtt, false);
        localInfo.setParagraphAttributes(defAtt, false);


        try {
            for (int i = 0; i < dbAttributes.length; i++) {
                String newline = "\n\n";
                String title = (String)dbAttributes[i][0];
                String dbString = dbAttributes[i][1].toString();
                String localString = localAttributes[i][1].toString();

                dbDoc.insertString(dbDoc.getLength(), title, titleAtt);
                localDoc.insertString(localDoc.getLength(), title, titleAtt);

                dbDoc.insertString(dbDoc.getLength(), newline, newlineAtt);
                localDoc.insertString(localDoc.getLength(), newline, newlineAtt);

                if (dbAttributes[i][1].equals(localAttributes[i][1])) {
                    dbDoc.insertString(dbDoc.getLength(), dbString, defAtt);
                    localDoc.insertString(localDoc.getLength(), localString, defAtt);
                }
                else {
                    dbDoc.insertString(dbDoc.getLength(), dbString, confAtt);
                    localDoc.insertString(localDoc.getLength(), localString, confAtt);
                }

                dbDoc.insertString(dbDoc.getLength(), newline, newlineAtt);
                dbDoc.insertString(dbDoc.getLength(), newline, newlineAtt);
                localDoc.insertString(localDoc.getLength(), newline, newlineAtt);
                localDoc.insertString(localDoc.getLength(), newline, newlineAtt);
            }
        } catch (BadLocationException e) {
            e.printStackTrace();
        }

        dbInfo.setEditable(false);
        dbInfo.setCaretPosition(0);
        localInfo.setEditable(false);
        localInfo.setCaretPosition(0);

        JPanel dbPanel = new JPanel(new BorderLayout());
        dbPanel.add(new JLabel("Database object"), BorderLayout.NORTH);
        dbPanel.add(new JScrollPane(dbInfo), BorderLayout.CENTER);

        JPanel localPanel = new JPanel(new BorderLayout());
        localPanel.add(new JLabel("Local object"), BorderLayout.NORTH);
        localPanel.add(new JScrollPane(localInfo), BorderLayout.CENTER);

        JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, localPanel, dbPanel);
        dbPanel.setPreferredSize(new Dimension(400, 600));
        localPanel.setPreferredSize(new Dimension(400, 600));

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(split, BorderLayout.CENTER);
        split.setDividerLocation(0.5);
        panel.add(new JLabel("What do you want to do?"), BorderLayout.SOUTH);

        return panel;
    }
}
