package gui.editors.level;

import gui.editors.AbstractEditor;
import gui.editors.common.DPanel;
import gui.editors.map.MapPanel;
import gui.editors.map.MapViewPanel;
import sys.datatypes.*;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.datatypes.question.QuestionCategory;
import sys.datatypes.question.QuestionGroup;
import sys.models.LevelModel;
import sys.models.treemodels.AbstractTreeModel;
import sys.models.treemodels.LevelTreeModel;

import javax.swing.*;
import java.awt.*;

/**
 * Specific panel for the level editor panel, that is
 * for editing <code>Level</code>, <code>TeleportSource</code>,
 * <code>TeleportDestination</code> and <code>Message</code> objects.
 * Depending on the type of object selecting in the Level tree, a
 * corresponding information panel is shown.
 *
 * @author Eivind Sorteberg - 20.jun.2006
 */
public class LevelEditor extends AbstractEditor {
    private LevelInfo levelInfo;
    private TeleSrcInfo teleSrc;
    private TeleDestInfo teleDest;
    private MessageInfo msgInfo;
    private QuestionCategoryInfo questCatInfo;
    private HistoryInfo historyInfo;
    private FactInfo factInfo;
    private MapPanel mapPanel;
    private DPanel currentInfoPanel;


    /**
     * Creates a LevelEditor
     */
    public LevelEditor() {
        super();
        this.setLayout(new BorderLayout());

        this.levelInfo = new LevelInfo();
        this.teleSrc = new TeleSrcInfo();
        this.teleDest = new TeleDestInfo();
        this.msgInfo = new MessageInfo();
        this.questCatInfo = new QuestionCategoryInfo();
        this.historyInfo = new HistoryInfo();
        this.factInfo = new FactInfo();
        this.mapPanel = new MapPanel(false, null);
        this.add(mapPanel, BorderLayout.CENTER);
    }


    /**
     * Displays information on an AoCObject on the bottom of the panel
     *
     * @param source
     */
    public void showObject(AoCObject source) {
        if (source instanceof LevelModel) {
            LevelModel lm = (LevelModel)source;
            mapPanel.setModel(lm);
            setInfoPanel(levelInfo);
            levelInfo.showObject(lm);
            currentInfoPanel.setEditable(true);
        }
        else if (source instanceof MapItem) {
            MapItem mi = (MapItem)source;
            LevelModel lm = mi.getLevelModel();
            mapPanel.setModel(lm);
            mapPanel.setCenter(mi.getMapPosition().getX(), mi.getMapPosition().getY());

            if (source instanceof TeleportSource) {
                setInfoPanel(teleSrc);
                teleSrc.showObject((TeleportSource)mi);
            }
            else if (source instanceof TeleportDestination) {
                setInfoPanel(teleDest);
                teleDest.showObject((TeleportDestination)mi);
            }
            else if (source instanceof Message) {
                setInfoPanel(msgInfo);
                msgInfo.showObject((Message)mi);
            }
            else if (source instanceof QuestionCategory) {
                setInfoPanel(questCatInfo);
                questCatInfo.showObject((QuestionCategory)mi);
            }
            else if (source instanceof History) {
                setInfoPanel(historyInfo);
                historyInfo.showObject((History)mi);
            }
            else if (source instanceof Fact) {
                setInfoPanel(factInfo);
                factInfo.showObject((Fact)mi);
            }
            currentInfoPanel.setEditable(true);
        }
        else {
            if (currentInfoPanel != null) currentInfoPanel.setEditable(false);
        }
    }

    /**
     * Sets the content of the infopanel.
     * Removes the current content, and adds the new.
     *
     * @param newPanel
     */
    private void setInfoPanel(DPanel newPanel) {
        if (currentInfoPanel != newPanel) {
            clearInfoPanel();
            if (newPanel != null) {
                add(newPanel, BorderLayout.SOUTH);
            }
            currentInfoPanel = newPanel;
            revalidate();
            currentInfoPanel.repaint();
        }
    }


    /**
     * Returns the content of the infopanel currently displayed.
     *
     * @return the content of the infopanel currently displayed
     */
    private JPanel getInfoPanel() {
        return currentInfoPanel;
    }


    /**
     * Removes the current content of the infopanel
     */
    private void clearInfoPanel() {
        if (currentInfoPanel != null) {
            remove(currentInfoPanel);
        }
    }


    /**
     * Called whenever a new object is chosen in the tree.
     * Retrieves the changes made, and saves them in the object.
     *
     * @param current The selected object
     */
    public void updateObject(AoCObject current) {
        if (current == null) {
            return;
        }
        else if (current instanceof LevelModel) {
            LevelModel lm = (LevelModel)current;

            if (getInfoPanel() == levelInfo) {
                lm.setName(levelInfo.getName());

                if (!(levelInfo.getMap().equalsIgnoreCase(LevelInfo.DEFAULT_BOX_VALUE))) {
                    lm.setMap(levelInfo.getMap());
                }
            }
        }
        else if (current instanceof TeleportDestination) {
            TeleportDestination td = (TeleportDestination)current;
            if (getInfoPanel() == teleDest) {
                td.setText(teleDest.getText());
                    td.setMapPosition(teleDest.getMapPosition());
            }
        }
        else if (current instanceof TeleportSource) {
            TeleportSource ts = (TeleportSource)current;
            if (getInfoPanel() == teleSrc) {
                ts.setText(teleSrc.getText());
                ts.setMapPosition(teleSrc.getMapPosition());
                ts.setTeleportDestinationId(teleSrc.getDestinationID());
                ts.setRequirementList(teleSrc.getRequirementList());
            }
        }
        else if (current instanceof Message) {
            Message msg = (Message)current;
            if (getInfoPanel() == msgInfo) {
                msg.setText(msgInfo.getText());
                msg.setMapPosition(msgInfo.getMapPosition());
            }
        }
        else if (current instanceof QuestionCategory) {
            QuestionCategory qc = (QuestionCategory)current;
            if (getInfoPanel() == questCatInfo) {
                qc.setName(questCatInfo.getName());
                qc.setMapPosition(questCatInfo.getMapPosition());
            }
        }
        else if (current instanceof History) {
            History h = (History)current;
            if (getInfoPanel() == historyInfo) {
                h.setName(historyInfo.getName());
                h.setMapPosition(historyInfo.getMapPosition());
            }
        }
        else if (current instanceof Fact) {
            Fact f = (Fact)current;
            if (getInfoPanel() == factInfo) {
                f.setName(factInfo.getName());
                f.setMapPosition(factInfo.getMapPosition());
            }
        }
        else if (current instanceof Level) {
        }
        else if (current instanceof QuestionGroup) {

        }
        else {
        }
        if (current instanceof GroupElement) {
        }
        showObject(current);
    }


    /**
     * Returns the mappanel
     *
     * @return the mappanel
     */
    public MapPanel getMapPanel() {
        return mapPanel;
    }


    /**
     * Sets the AbstractTreeModel of this editor.
     * The model-parameter shoud be instance of LevelTreeModel
     *
     * @param model the model to set
     */
    public void setModel(AbstractTreeModel model) {
        this.model = model;
        if (model instanceof LevelTreeModel) {
            LevelTreeModel superLevelModel = (LevelTreeModel)model;
            this.levelInfo.setModel(superLevelModel);
            this.teleSrc.setModel(superLevelModel);
            this.teleDest.setModel(superLevelModel);
            this.msgInfo.setModel(superLevelModel);
            this.questCatInfo.setModel(superLevelModel);
            this.historyInfo.setModel(superLevelModel);
            this.factInfo.setModel(superLevelModel);
            this.mapPanel.setSuperTreeModel(superLevelModel);
        }
    }

    public void superLevelModelChanged() {
        this.mapPanel.levelModelChanged(MapViewPanel.MAP_AND_MAPITEMS_CHANGED);
        this.levelInfo.modelChanged();
        this.teleSrc.modelChanged();
        this.teleDest.modelChanged();
        this.msgInfo.modelChanged();
        this.questCatInfo.modelChanged();
        this.historyInfo.modelChanged();
        this.factInfo.modelChanged();
    }
}