package gui.editors.level;

import gui.editors.common.InfoPanel;
import gui.editors.common.InfoTextArea;
import gui.editors.common.PlacementPanel;
import gui.editors.common.selector.TeleportSourceRequirementSelector;
import gui.editors.common.util.InfoPanelGenerator;
import sys.datatypes.TeleportDestination;
import sys.datatypes.TeleportSource;
import sys.datatypes.question.QuestionCategory;
import sys.models.treemodels.LevelTreeModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collections;

/**
 * Contains the information for the Teleport
 * sources in the level editor view.
 *
 * @author Eivind Sorteberg - 23.jun.2006
 */
public class TeleSrcInfo extends InfoPanel {
    private JTextArea text;
    private JComboBox destBox;
    private JComboBox requirementsBox;
    private JButton addReqButton, removeReqButton;

    /**
     * Creates a TeleSrcInfo-object
     */
    public TeleSrcInfo() {
        super();
        this.setLayout(new BorderLayout());
        this.text = new JTextArea();
        this.destBox = new JComboBox();
        requirementsBox = new JComboBox();
        addReqButton = new JButton("Add");
        addReqButton.addActionListener(this);
        removeReqButton = new JButton("Remove");
        removeReqButton.addActionListener(this);
        Box buttBox1 = new Box(BoxLayout.X_AXIS);
        buttBox1.add(Box.createHorizontalGlue());
        Box buttBox2 = new Box(BoxLayout.X_AXIS);
        buttBox2.add(addReqButton);
        buttBox2.add(removeReqButton);
        buttBox2.add(Box.createHorizontalGlue());

        InfoPanelGenerator.placeComponents(infoPanel, 3, 3, new Object[]{"Text", new InfoTextArea(text), null,
                                                                         "Destination", destBox, buttBox1,
                                                                         "Requirements", requirementsBox, buttBox2});
        this.add(infoPanel, BorderLayout.NORTH);
        PlacementPanel placement = new PlacementPanel(levelBox, xPos, yPos, zPos, changePositionButton);
        this.add(placement, BorderLayout.SOUTH);
    }

    /**
     * Updates the information shown to correspond
     * to the chosen object
     *
     * @param object
     */
    public void showObject(TeleportSource object) {
        super.showObject(object);
        text.setText(object.getText());
        text.setCaretPosition(0);

        /* Selecting the correct TeleportDestination in the correct JComboBox */
        ArrayList<TeleportDestination> tds = ((LevelTreeModel)model).getAllTeleportDestinations();
        for (TeleportDestination td : tds) {
            if (object.getTeleportDestinationId() == td.getId()) {
                destBox.setSelectedItem(td);
                break;
            }
        }

        /* Adding all requirements of this TeleportSource to the correct JComboBox */
        ArrayList<QuestionCategory> requirementList = object.getRequirementList();
        QuestionCategory[] requirementArray = new QuestionCategory[requirementList.size()];
        for (int i = 0; i < requirementArray.length; i++) {
            requirementArray[i] = requirementList.get(i);
        }
        requirementsBox.setModel(new DefaultComboBoxModel(requirementArray));
    }

    /**
     * Gets the ID of the currently chosen TeleportDestination
     *
     * @return the ID of the currently chosen TeleportDestination
     */
    public int getDestinationID() {
        return ((TeleportDestination)destBox.getSelectedItem()).getId();
    }

    /**
     * Gets the text in the textfield
     *
     * @return the text in the textfield
     */
    public String getText() {
        return text.getText();
    }

    /**
     * Gets the list of requirements for the TeleportSource currently displayed.
     *
     * @return the list of requirements for the TeleportSource currently displayed.
     */
    public ArrayList<QuestionCategory> getRequirementList() {
        ArrayList<QuestionCategory> qclist = new ArrayList<QuestionCategory>();
        for (int i = 0; i < requirementsBox.getModel().getSize(); i++) {
            qclist.add((QuestionCategory)requirementsBox.getModel().getElementAt(i));
        }
        return qclist;
    }


    public void addRequirement(QuestionCategory req) {
        requirementsBox.addItem(req);
    }


    public void removeRequirement(QuestionCategory req) {
        requirementsBox.removeItem(req);
    }


    @Override
    public void modelChanged() {
        super.modelChanged();
        ArrayList<TeleportDestination> teleportDestinationList;
        teleportDestinationList = ((LevelTreeModel)model).getAllTeleportDestinations();
        Collections.sort(teleportDestinationList);

        Object selectedItem = destBox.getModel().getSelectedItem();
        destBox.removeAllItems();
        for (TeleportDestination teledest : teleportDestinationList) {
            destBox.addItem(teledest);
        }
        destBox.getModel().setSelectedItem(selectedItem);
    }


    public void actionPerformed(ActionEvent e) {
        super.actionPerformed(e);
        if (e.getSource() == addReqButton) {
            new TeleportSourceRequirementSelector(this, (LevelTreeModel)model);
        }
        else if (e.getSource() == removeReqButton) {
            requirementsBox.removeItem(requirementsBox.getSelectedItem());
        }
    }

    public void setEditable(boolean editable) {
        super.setMovable(editable);
        this.text.setEnabled(editable);
        this.destBox.setEnabled(editable);
        this.addReqButton.setEnabled(editable);
        this.removeReqButton.setEnabled(editable);
    }
}
