package gui.editors.map;

import gui.MapFrame;
import gui.editors.common.InfoPanel;
import sys.AoCAdmin;
import sys.models.LevelModel;
import sys.models.treemodels.AbstractTreeModel;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * A panel with scrollbars and a MapViewPanel, which in turn contains a map
 *
 * @author Tarje Killingberg - 23.jun.2006
 */
public class MapPanel extends JPanel {

    private MapFrame mapFrame;
    private MapViewPanel mapViewPanel;
    private JPanel scrollableMapViewPanel;
    private JComponent currentPanel;
    private LevelModel levelModel;
    private boolean framed;

    /**
     * Creates a MapPanel
     *
     * @param framed
     */
    public MapPanel(boolean framed, InfoPanel infoPanel) {
        this.framed = framed;
        JScrollBar horizontalScrollBar;
        JScrollBar verticalScrollBar;
        JEditorPane credPanel;


        setLayout(new BorderLayout());
        scrollableMapViewPanel = new JPanel();
        scrollableMapViewPanel.setLayout(new BorderLayout());
        horizontalScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);
        scrollableMapViewPanel.add(horizontalScrollBar, BorderLayout.SOUTH);
        verticalScrollBar = new JScrollBar(JScrollBar.VERTICAL);
        scrollableMapViewPanel.add(verticalScrollBar, BorderLayout.EAST);

        mapViewPanel = new MapViewPanel(horizontalScrollBar, verticalScrollBar);
        scrollableMapViewPanel.add(mapViewPanel, BorderLayout.CENTER);

        String credFilename = "aocAdmin/doc/creds.html";

        URL url;

        url = AoCAdmin.class.getClassLoader().getResource(credFilename);

        if (url == null) {
            try {
                url = new File(credFilename).toURL();
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
        }

        try {
            credPanel = new JEditorPane(url);
        } catch (IOException e) {
                credPanel = new JEditorPane("text/html",
                        "<br><br><br><center>Could not find " + credFilename + ".</center>");
            e.printStackTrace();
        }

        credPanel.setEditable(false);
        add(credPanel);
        currentPanel = credPanel;

        if (framed) {
            if (infoPanel != null) {
                JPanel contentPane;
                JPanel buttonPanel;
                JButton okButton;

                mapViewPanel.getMapViewPanelListener().setInfoPanel(infoPanel);

                contentPane = new JPanel();
                contentPane.setLayout(new BorderLayout());
                contentPane.add(this, BorderLayout.CENTER);
                buttonPanel = new JPanel();
                okButton = new JButton("OK");
                okButton.addActionListener(mapViewPanel.getMapViewPanelListener());
                buttonPanel.add(okButton);
                contentPane.add(buttonPanel, BorderLayout.SOUTH);
                mapFrame = new MapFrame(contentPane);
            }
        }
        else {
            JPanel controlPanel;
            JLabel fpsLabel;
            JSpinner fpsSpinner;
            JLabel speedLabel;
            JSlider speedSlider;

            controlPanel = new JPanel();
            controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.X_AXIS));

            controlPanel.add(Box.createHorizontalGlue());
            fpsLabel = new JLabel("FPS: ");
            controlPanel.add(fpsLabel);
            fpsSpinner = new JSpinner();
            fpsSpinner.setPreferredSize(new Dimension((int)fpsSpinner.getPreferredSize().getWidth(), 20));
            ((SpinnerNumberModel)fpsSpinner.getModel()).setMinimum(1);
            ((SpinnerNumberModel)fpsSpinner.getModel()).setMaximum(60);
            ((JSpinner.NumberEditor)fpsSpinner.getEditor()).getTextField().setEditable(false);
            fpsSpinner.setValue(mapViewPanel.getFps());
            fpsSpinner.addChangeListener(mapViewPanel.getMapViewPanelListener());
            controlPanel.add(fpsSpinner);
            controlPanel.add(Box.createRigidArea(new Dimension(10, 0)));
            speedLabel = new JLabel("Speed: ");
            controlPanel.add(speedLabel);
            speedSlider = new JSlider();
            speedSlider.setPreferredSize(new Dimension((int)speedSlider.getPreferredSize().getWidth(), 20));
            speedSlider.setMinimum(4);
            speedSlider.setMaximum(40);
            speedSlider.setMajorTickSpacing(5);
            speedSlider.setMinorTickSpacing(1);
            speedSlider.setPaintTrack(true);
            speedSlider.setSnapToTicks(true);
            speedSlider.setValue(mapViewPanel.getSpeedX());
            speedSlider.addChangeListener(mapViewPanel.getMapViewPanelListener());
            controlPanel.add(speedSlider);
            controlPanel.add(Box.createHorizontalGlue());

            add(controlPanel, BorderLayout.SOUTH);
        }

    }


    /**
     * Sets the datamodel for this MapPanel and sets this panel
     * as listener for changes in the model.
     *
     * @param model the new datasource for this MapPanel
     */
    public void setModel(LevelModel model) {

        if (this.levelModel == model) {
            return;
        }

        levelModel = model;
        levelModel.setListener(this);
        try {
            mapViewPanel.setModel(model);
            if (currentPanel != scrollableMapViewPanel) {
                remove(currentPanel);
                add(scrollableMapViewPanel, BorderLayout.CENTER);
                currentPanel = scrollableMapViewPanel;
                revalidate();
                if (framed) {
                    mapViewPanel.requestFocusInWindow();
                }

            }
        } catch (Exception e) {
            showErrorPanel(e, model);
            e.printStackTrace();
        }
    }

    /**
     * Called when the LevelModel changes.
     * Just forwards the message to MapViewPanel
     */
    public void levelModelChanged(int event) {
        try {
            if (currentPanel != scrollableMapViewPanel) {
                remove(currentPanel);
                add(scrollableMapViewPanel, BorderLayout.CENTER);
                currentPanel = scrollableMapViewPanel;
                revalidate();
            }

            mapViewPanel.levelModelChanged(event);
        } catch (Exception e) {
            showErrorPanel(e, levelModel);
            e.printStackTrace();
        }
    }


    /**
     * Gets the datamodel for this MapPanel
     */
    public LevelModel getModel() {
        return levelModel;
    }


    /**
     * Sets the given map coordinates to be the center of the MapView.
     *
     * @param x
     * @param y
     */
    public void setCenter(int x, int y) {
        mapViewPanel.setCenter(x, y);
    }


    /**
     * Creates and shows a nice and useful error-panel :)
     *
     * @param e
     */
    private void showErrorPanel(Exception e, LevelModel model) {
        this.remove(currentPanel);

        JLabel[] errorLabel;
        Box[] errorBox;
        JPanel errorPanel;

        errorPanel = new JPanel();
        errorPanel.setLayout(new BoxLayout(errorPanel, BoxLayout.Y_AXIS));
        errorLabel = new JLabel[2];
        errorLabel[0] = new JLabel("ERROR");
        errorLabel[0].setFont(new Font("Monospaced", Font.BOLD, 50));
        errorLabel[1] = new JLabel("");
        errorLabel[1].setFont(new Font("Monospaced", Font.BOLD, 15));

        if (e instanceof FileNotFoundException) {
            if (levelModel.getLevelObject().getMap().equalsIgnoreCase("undefined")) {
                errorLabel[0].setText(" ");
                errorLabel[1].setText("You have not yet chosen a map for this level");
            }
            else if (levelModel.getLevelObject().getMap().equalsIgnoreCase("nomap")) {  // levelModel is rootLevelModel
                errorLabel[0].setText(" ");
                errorLabel[1].setText("No level chosen");
            }
            else {
                errorLabel[0].setText("ERROR");
                errorLabel[1].setText(levelModel.getLevelObject().getMap() + " cannot be opened for reading");
                e.printStackTrace();
            }
        }
        else if (e instanceof SecurityException) {
            errorLabel[0].setText("ERROR");
            errorLabel[1].setText("We were denied read access to: " + levelModel.getLevelObject().getMap());
            e.printStackTrace();
        }
        else if (e instanceof NullPointerException) {
            errorLabel[0].setText("ERROR");
            errorLabel[1].setText("NullPointerException for some reason...");
            e.printStackTrace();
        }
        else {
            errorLabel[0].setText("ERROR");
            errorLabel[1].setText("For some reason... (" + e.getClass().getName() + ")");
            e.printStackTrace();
        }

        errorBox = new Box[errorLabel.length];

        for (int i = 0; i < errorBox.length; i++) {
            errorBox[i] = new Box(BoxLayout.X_AXIS);
            errorBox[i].add(Box.createHorizontalGlue());
            errorBox[i].add(errorLabel[i]);
            errorBox[i].add(Box.createHorizontalGlue());
            errorPanel.add(errorBox[i]);
        }

        errorPanel.add(Box.createVerticalGlue());
        add(errorPanel, BorderLayout.CENTER);
        currentPanel = errorPanel;
        revalidate();

    }

    public void closeFrame() {
        mapFrame.dispose();
        mapViewPanel.kill();
    }


    public void setSuperTreeModel(AbstractTreeModel treeModel) {
        mapViewPanel.setSuperTreeModel(treeModel);
    }
}
