package gui.editors.map;


import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.MapHeader;
import com.alienfactory.javamappy.loader.MapLoader;
import com.alienfactory.javamappy.viewer.LayerViewer;
import com.alienfactory.javamappy.viewer.MapViewer;
import com.alienfactory.javamappy.viewer.render.J2SE14Renderer;
import gui.listeners.MapViewPanelListener;
import sys.AoCAdmin;
import sys.datatypes.*;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.datatypes.question.QuestionCategory;
import sys.models.LevelModel;
import sys.models.treemodels.AbstractTreeModel;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;

/**
 * This is the panel that actually paints the map and the icons
 *
 * @author Tarje Killingberg - 23.jun.2006
 */
public class MapViewPanel extends JPanel implements Runnable {

    /* Constants... */
    public static final int MAP_CHANGED = 45;
    public static final int MAPITEMS_CHANGED = 46;
    public static final int MAP_AND_MAPITEMS_CHANGED = 47;

    /* Control variables */
    private int speedX = 25;     // Move-speed in X-direction when using keyboard */
    private int speedY = 25;     // Move-speed in Y-direction when using keyboard */
    private int fps = 15;        // Frames per second
    private	boolean running = false;

    /* Component variables */
    private JScrollBar horizontalScrollBar;
    private JScrollBar verticalScrollBar;
    private MapViewPanelListener mapViewPanelListener;

    /* Mappy variables */
    private Map map;
    private MapHeader mapHeader;
    private J2SE14Renderer renderer;
    private	MapViewer mapViewer;

    /* Others... */
    private LevelModel levelModel;
    private AbstractTreeModel treeModel;
    private ArrayList<MapItem> mapItemList;
    private Image doubleBuffer;
    private String currentMapName;


    /**
     * Creates a new MapViewPanel
     *
     * @param horizontalScrollBar
     * @param verticalScrollBar
     */
    public MapViewPanel(JScrollBar horizontalScrollBar, JScrollBar verticalScrollBar) {
        currentMapName = "";
        mapItemList = new ArrayList<MapItem>();
        mapViewPanelListener = new MapViewPanelListener(this);
        this.horizontalScrollBar = horizontalScrollBar;
        this.horizontalScrollBar.setBlockIncrement(150);
        this.horizontalScrollBar.setUnitIncrement(20);
        this.horizontalScrollBar.addAdjustmentListener(mapViewPanelListener);
        this.verticalScrollBar = verticalScrollBar;
        this.verticalScrollBar.setBlockIncrement(150);
        this.verticalScrollBar.setUnitIncrement(20);
        this.verticalScrollBar.addAdjustmentListener(mapViewPanelListener);
        addKeyListener(mapViewPanelListener);
        addMouseListener(mapViewPanelListener);
        addMouseWheelListener(mapViewPanelListener);
    }


    /**
     * Sets the datamodel for this MapViewPanel and scrolls the view to (0, 0).
     *
     * @param model the new datasource for this MapViewPanel
     * @throws Exception for various reasons :)
     */
    public void setModel(LevelModel model) throws Exception {
        levelModel = model;
        levelModelChanged(MAP_CHANGED);
        scrollMapTo(0, 0);

    }


    /**
     * Should be called whenever the levelModel i changed
     * Loads the map and mapitems associated with a level
     *
     * @throws Exception
     */
    public void levelModelChanged(int event) throws Exception {

        if (levelModel == null) { return; }

        if (!levelModel.getMap().equals(currentMapName)) {
                loadMap(levelModel.getLevelObject().getMap());
        }
        loadMapItems();

        if (!running) {
            running = true;
            Thread t = new Thread(this);
            t.setPriority(Thread.MAX_PRIORITY);
            t.start();
        }

    }


    /**
     * Loads the mapitems associated with a level
     */
    private void loadMapItems() {
        mapItemList = new ArrayList<MapItem>();

        for (int i = 0; i < levelModel.getGroupElements().size(); i++) {
            GroupElement mapItem = levelModel.getGroupElements().get(i);
            if (mapItem instanceof MapItem) {
                mapItemList.add((MapItem)mapItem);
            }
        }
    }


    /**
     * Loads a map
     *
     * @param mapFileName
     * @throws Exception
     */
    private void loadMap(String mapFileName) throws Exception {
        currentMapName = mapFileName;
        URL mapUrl = new URL(AoCAdmin.WEB_SERVER + AoCAdmin.MAP_FOLDER + mapFileName);
        BufferedInputStream inputStream = new BufferedInputStream(mapUrl.openStream());

        map = MapLoader.loadMap(inputStream);
        mapHeader = map.getMapHeader();
        renderer = new J2SE14Renderer(map);

        /* Because of several reasons dealing with the scrollbars and resizing, */
        /* we have to initialize the mapViewer temporarily here */
        /* it will be reinitialized properly later */
        mapViewer = new MapViewer(map, renderer, 200, 200);

        /* And this... */
        horizontalScrollBar.setMaximum(mapHeader.getMapPixelWidth() - 516);
        verticalScrollBar.setMaximum(mapHeader.getMapPixelHeight() - 729);


        try {
            inputStream.close();
        }
        catch (IOException ioe) {
        }
    }


    /**
     * Access point for the repaint-thread
     */
    public void run() {

        long refreshRate;
        long start, elapsed, sleepTime;

        while (running) {
            start = System.currentTimeMillis();
            refreshRate = 1000/fps;

            scrollMapFromKeyboard();
            super.repaint();

            elapsed = System.currentTimeMillis() - start;
            sleepTime = refreshRate - elapsed;

            if (sleepTime > 0) {
                try { Thread.sleep(sleepTime); }
                catch (InterruptedException ie) { /**/ }
            } else {
            }
        }
    }


    /**
     * Paints the map and the items on it.
     *
     * @param graphics
     */
    protected void paintComponent(Graphics graphics) {
        super.paintComponent(graphics);
        synchronized (this) {
            if (map == null) {
                return;
            }
            if (mapViewer.getViewHeight() != getHeight() ||
                    mapViewer.getViewWidth() != getWidth() ||
                    mapViewer.getMap() != map) {

                int pixelX = mapViewer.getPixelX();
                int pixelY = mapViewer.getPixelY();
                mapViewer = new MapViewer(map, renderer, getWidth(), getHeight());
                horizontalScrollBar.setMaximum(mapHeader.getMapPixelWidth() - getWidth());
                verticalScrollBar.setMaximum(mapHeader.getMapPixelHeight() - getHeight());
                scrollMapTo(pixelX, pixelY);
                doubleBuffer = createImage(getWidth(), getHeight());
            }

            Graphics2D g2d = (Graphics2D)doubleBuffer.getGraphics();

            Rectangle clip = g2d.getClipBounds();
            mapViewer.updateAnimBlocks();

            g2d.setColor(Color.BLACK);
            g2d.fillRect(0, 0, getWidth(), getHeight());

            int nViewX = horizontalScrollBar.getValue();
            int nViewY = verticalScrollBar.getValue();

            long currentTime = System.currentTimeMillis();
            StaticAnimation.setCurrenTimeMillis(currentTime);


            AffineTransform oIdentity = new AffineTransform();
            oIdentity.setToIdentity();

            AffineTransform oTransform = new AffineTransform();
            oTransform.setToTranslation(-nViewX, -nViewY);

            int numLayers = mapViewer.getMap().getLayers().length;

            for (int layer = 0; layer < numLayers; layer++) {
              g2d.setTransform(oIdentity);

              LayerViewer layerViewer = mapViewer.getLayerViewers()[layer];
              layerViewer.setPixelX(nViewX);
              layerViewer.setPixelY(nViewY);
              layerViewer.draw(g2d, true);


              g2d.setTransform(oTransform);

                StaticAnimation staticAnimation;
                for (MapItem mapItem : mapItemList) {
                    if (mapItem.getMapPosition().getZ() == layer) {
                        if (mapItem instanceof Fact) {
                            staticAnimation = new StaticAnimation(Icons.ICON_FACT);
                            staticAnimation.draw(g2d, mapItem);
                        }
                        if (mapItem instanceof History) {
                            staticAnimation = new StaticAnimation(Icons.ICON_HISTORY);
                            staticAnimation.draw(g2d, mapItem);
                        }
                        if (mapItem instanceof QuestionCategory) {
                            staticAnimation = new StaticAnimation(Icons.ICON_QUESTION);
                            staticAnimation.draw(g2d, mapItem);
                        }
                        if (mapItem instanceof Message) {
                            staticAnimation = new StaticAnimation(Icons.ICON_MESSAGE);
                            staticAnimation.draw(g2d, mapItem);
                        }
                        if (mapItem instanceof TeleportDestination) {
                            staticAnimation = new StaticAnimation(Icons.ICON_TELEPORTDEST);
                            staticAnimation.draw(g2d, mapItem);
                        }
                        if (mapItem instanceof TeleportSource) {
                            staticAnimation = new StaticAnimation(Icons.ICON_TELEPORTSOURCE);
                            staticAnimation.draw(g2d, mapItem);
                        }
                    }
                }
            }

            g2d.dispose();
            if (doubleBuffer != null) {
                graphics.drawImage(doubleBuffer, 0, 0, this);
            }
          }
    }


    /**
     * Moves the map to the specified position
     *
     * @param x
     * @param y
     */
    private void scrollMapTo(int x, int y) {
        try {
            int xmin = 0;
            int xmax = mapHeader.getMapPixelWidth() - getWidth();
            int ymin = 0;
            int ymax = mapHeader.getMapPixelHeight() - getHeight();

            if (x < xmin) { x = xmin; }
            else if (x > xmax) { x = xmax; }
            if (y < ymin) { y = ymin; }
            else if (y > ymax) { y = ymax; }

            try {
                mapViewer.setPixelX(x);
                horizontalScrollBar.setValue(mapViewer.getPixelX());
                mapViewer.setPixelY(y);
                verticalScrollBar.setValue(mapViewer.getPixelY());
            } catch (IllegalArgumentException iae) {
            }
        }
        catch (NullPointerException e) {
        }
    }


    public void setCenter(int x, int y) {
        x = x - (getWidth() != 0 ? getWidth() : 732) / 2;
        y = y - (getHeight() != 0 ? getHeight() : 518) / 2;
        scrollMapTo(x, y);
    }


    /**
     * Moves the map to the specified x-position
     *
     * @param x
     */
    public void scrollMapToX(int x) {
        scrollMapTo(x, mapViewer.getPixelY());
    }


    /**
     * Moves the map to the specified y-position
     *
     * @param y
     */
    public void scrollMapToY(int y) {
        scrollMapTo(mapViewer.getPixelX(), y);
    }


    /**
     * Moves the map by the specified amount
     *
     * @param dx
     * @param dy
     */
    public void scrollMapBy(int dx, int dy) {
        scrollMapTo(mapViewer.getPixelX() + dx, mapViewer.getPixelY() + dy);
    }


    /**
     * Checks which of the arrow-keys are pressed,
     * and moves the map as expected
     */
    private void scrollMapFromKeyboard() {
        int dx = 0, dy = 0;
        boolean[] pressedKeys = mapViewPanelListener.getPressedKeys();

        if (pressedKeys[0]) { dy -= speedY; }
        if (pressedKeys[1]) { dy += speedY; }
        if (pressedKeys[2]) { dx -= speedX; }
        if (pressedKeys[3]) { dx += speedX; }

        if (dx != 0 || dy != 0) {
            scrollMapBy(dx, dy);
        }
    }


    /**
     * Method for killing the repaint-thread when we are done with the MapViewPanel
     */
    public void kill() {
        running = false;
    }


    /**
     * Getter-method for this panels associated horizontal scrollbar
     *
     * @return guess!
     */
    public JScrollBar getHorizontalScrollBar() {
        return horizontalScrollBar;
    }


    /**
     * Getter-method for this panels associated vertical scrollbar
     *
     * @return guess!
     */
    public JScrollBar getVerticalScrollBar() {
        return verticalScrollBar;
    }


    /**
     * Called by the associated mouselistener when an interesting
     * button has been clicked in an interesting place.
     *
     * @param x
     * @param y
     */
    public void mouseClicked(int x, int y) {
        MapPanel mp = (MapPanel)getParent().getParent();
        Point p = translateToMapPoint(new Point(x, y));

        for (MapItem mapItem : mapItemList) {
            if (hit(mapItem, p)) {
                treeModel.getModelManager().fireObjectSelected((AoCObject)mapItem);
                return;
            }

        }
    }

    public boolean hit(MapItem mapItem, Point click) {
        MapPosition itemMapPosition = mapItem.getMapPosition();
        Point item = new Point(itemMapPosition.getX(), itemMapPosition.getY());

        int animNr = Icons.ICON_QUESTION;

        if (mapItem instanceof QuestionCategory) animNr = Icons.ICON_QUESTION;
        else if (mapItem instanceof Fact) animNr = Icons.ICON_FACT;
        else if (mapItem instanceof History) animNr = Icons.ICON_HISTORY;
        else if (mapItem instanceof Message) animNr = Icons.ICON_FACT;
        else if (mapItem instanceof TeleportDestination) animNr = Icons.ICON_FACT;
        else if (mapItem instanceof TeleportSource) animNr = Icons.ICON_FACT;

        Rectangle itemRect = Icons.getFrameSize(animNr);

        if (click.getX() > item.getX()
                && click.getX() < item.getX() + itemRect.getWidth()
                && click.getY() > item.getY()
                && click.getY() < item.getY() + itemRect.getHeight()) {

            return true;
        }

        return false;
    }


    /**
     * Translates a point from panel coordinates to map coordinates
     *
     * @param p the point to be translated
     * @return the translated point
     */
    public Point translateToMapPoint(Point p) {
        return new Point((int)p.getX() + mapViewer.getPixelX(), (int)p.getY() + mapViewer.getPixelY());
    }


    public int getSpeedX() {
        return speedX;
    }


    public void setSpeedX(int speedX) {
        this.speedX = speedX;
    }


    public int getSpeedY() {
        return speedY;
    }


    public void setSpeedY(int speedY) {
        this.speedY = speedY;
    }


    public int getFps() {
        return fps;
    }


    public void setFps(int fps) {
        this.fps = fps;
    }


    public MapViewPanelListener getMapViewPanelListener() {
        return mapViewPanelListener;
    }


    public Point getCurrentLocation() {
        return new Point(mapViewer.getPixelX(), mapViewer.getPixelY());
    }


    public void setSuperTreeModel(AbstractTreeModel treeModel) {
        this.treeModel = treeModel;
    }
}
