package gui.editors.question.question;

import gui.editors.common.DPanel;
import gui.editors.common.FormatBar;
import gui.editors.common.InputPane;
import gui.editors.common.Previewable;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.question.MultiChoiceAlternative;
import sys.datatypes.question.MultiChoiceQuestion;
import sys.models.MCAlternativeModel;
import sys.models.treemodels.AbstractTreeModel;
import sys.models.treemodels.QuestionTreeModel;

import javax.swing.*;
import javax.swing.table.TableCellRenderer;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;

/**
 * The panel showing the inputbox for the question, and
 * the table for the alternativesTable. A <code>FormatBar</code> is
 * used on the top of the panel, allowing the user to write in
 * both HTML and LaTeX.
 *
 * @author Eivind Sorteberg - 21.jun.2006
 * @see gui.editors.question.category.QuestionCategoryPanel
 * @see gui.editors.common.FormatBar
 */
public class MultiChoicePanel extends DPanel implements Previewable, ActionListener, TableCellRenderer {
    public static final int NOF_VISIBLE_ALTERNATIVES = 5;

    private JRadioButton html;
    private JRadioButton latex;
    private InputPane question;

    private JTable alternativesTable;
    private JButton deleteAlternative;
    private JButton newAlternative;
    private QuestionTreeModel model;

    private TableCellRenderer defaultHeaderRenderer;
    private FormatBar formatBar;

    public MultiChoicePanel() {
        super();

        this.setLayout(new BorderLayout());


        this.html = new JRadioButton("HTML");
        this.latex = new JRadioButton("LaTeX");

        formatBar = new FormatBar(html, latex, this);

        //Adding component to the center top panel (question panel)
        JPanel center = new JPanel();

        //Adding the input text area
        JPanel centertop = new JPanel(new GridLayout(1, 1));
        centertop.setBorder(BorderFactory.createTitledBorder("Question"));

        question = new InputPane();
        centertop.add(new JScrollPane(question));


        //Adding components to the center bottom panel (alternativesTable panel)
        JPanel centerbottom = new JPanel();
        centerbottom.setLayout(new BoxLayout(centerbottom, BoxLayout.Y_AXIS));
        centerbottom.setBorder(BorderFactory.createTitledBorder("Alternatives"));

        //Adding the alternativesTable table
        this.alternativesTable = new JTable();
        this.defaultHeaderRenderer = alternativesTable.getTableHeader().getDefaultRenderer();
        alternativesTable.getTableHeader().setDefaultRenderer(this);

        alternativesTable.setPreferredScrollableViewportSize(new Dimension(
                alternativesTable.getPreferredScrollableViewportSize().width,
                alternativesTable.getRowHeight() * NOF_VISIBLE_ALTERNATIVES
        ));

        this.deleteAlternative = new JButton("Delete");
        this.deleteAlternative.addActionListener(this);
        this.newAlternative = new JButton("New");
        this.newAlternative.addActionListener(this);

        centerbottom.add(Box.createVerticalGlue());
        centerbottom.add(new JScrollPane(alternativesTable));

        //Adding the table buttons
        JPanel buttons = new JPanel();
        buttons.setLayout(new BoxLayout(buttons, BoxLayout.X_AXIS));
        buttons.add(Box.createHorizontalGlue());
        buttons.add(newAlternative);
        buttons.add(deleteAlternative);

        centerbottom.add(buttons);

        //Adding the two panels with correct proportions
        GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints constr = new GridBagConstraints();
        center.setLayout(new BorderLayout());

        constr.fill = GridBagConstraints.BOTH;
        constr.weightx = 1.0;
        constr.weighty = 1.0;

        constr.gridheight = 2;
        constr.gridwidth = GridBagConstraints.REMAINDER;
        constr.gridx = 0;
        constr.gridy = 0;
        gridBag.setConstraints(centertop, constr);
        center.add(centertop, BorderLayout.CENTER);

        constr.gridheight = 1;
        constr.gridx = 0;
        constr.gridy = 2;
        constr.gridwidth = GridBagConstraints.REMAINDER;
        gridBag.setConstraints(centerbottom, constr);
        center.add(centerbottom, BorderLayout.SOUTH);

        this.add(formatBar, BorderLayout.NORTH);
        this.add(center, BorderLayout.CENTER);
        this.add(centertop, BorderLayout.CENTER);
        this.add(centerbottom, BorderLayout.SOUTH);

    }

    public String[] getText() {
        String[] ret = new String[1 + alternativesTable.getRowCount()];

        ret[0] = question.getText();

        for (int i = 0; i < alternativesTable.getRowCount(); i++) {
            MCAlternativeModel model = (MCAlternativeModel)alternativesTable.getModel();

            String color = (Boolean)model.getValueAt(i, 1) ? "#00FF00" : "#FF0000";

            ret[i + 1] = "<font color='" + color + "'>" + model.getValueAt(i, 0) + "</font>";
        }

        return ret;
    }

    public String getTitle() {
        return null;
    }

    public AbstractTreeModel getModel() {
        return this.model;
    }

    public void showObject(MultiChoiceQuestion question) {
        this.question.setText(this.html.isSelected() ? question.getText() : question.getLatex());
        this.alternativesTable.setModel(question.getAlternativeModel());
        alternativesTable.getColumnModel().getColumn(1).setMaxWidth(50);
    }

    public void saveObject(MultiChoiceQuestion question) {
        if      (this.html.isSelected())    question.setText(this.question.getText());
        else if (this.latex.isSelected())   question.setLatex(this.question.getText());

        int currentRow = alternativesTable.getEditingRow();

        if (currentRow >= 0) {
            alternativesTable.getCellEditor().stopCellEditing();
            alternativesTable.getSelectionModel().setSelectionInterval(currentRow, currentRow);
        }
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource().equals(newAlternative)) {
            MultiChoiceAlternative newAlt = ((MCAlternativeModel)alternativesTable.getModel()).newAlternative();
            model.createMultiChoiceAlternative(newAlt);
        }
        else if (e.getSource().equals(deleteAlternative)) {
            int[] selection = alternativesTable.getSelectedRows();


            MultiChoiceAlternative[] removed = ((MCAlternativeModel)alternativesTable.getModel()).deleteAlternative(selection);
            model.removeObjects(removed);

            HashMap<Integer, AoCObject> changed = ((MCAlternativeModel)alternativesTable.getModel()).sortAlternatives();
            model.updateObjects(changed);

            int rows = alternativesTable.getModel().getRowCount();
            if (selection.length > 0 && rows > 0) {
                if (selection[0] < rows) {
                    alternativesTable.getSelectionModel().setSelectionInterval(selection[0], selection[0]);
                }
                else {
                    alternativesTable.getSelectionModel().setSelectionInterval(rows - 1, rows - 1);
                }
            }
        }
    }

    public void setModel(QuestionTreeModel model) {
        this.model = model;
    }

    public InputPane getInputPane() {
        return question;
    }

    public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        Component component = defaultHeaderRenderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);

        if (component instanceof JLabel) {
            JLabel l = (JLabel)component;

            // Added to make the label appear centered on the column header
            // despite change of skin.
            l.setHorizontalAlignment(JLabel.CENTER);
        }
        return component;
    }

    public void setEditable(boolean editable) {
        this.question.setEnabled(editable);
        this.formatBar.setEditable(editable);
        this.alternativesTable.setEnabled(editable);
        this.newAlternative.setEnabled(editable);
        this.deleteAlternative.setEnabled(editable);
    }
}
