package gui.editors.question.question;

import gui.editors.common.DPanel;
import gui.editors.common.SelectorField;
import gui.editors.common.selector.ParentSelector;
import gui.listeners.QuestionTypeListener;
import sys.datatypes.question.*;
import sys.models.treemodels.QuestionTreeModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * The panel containing information about the question object
 * selected by the user from the question tree (leaf nodes).
 * The top of the panel shows information common for all the
 * question types, while the bottom depends on the type of
 * question selected by the user.
 *
 * @author Eivind Sorteberg - 20.jun.2006
 * @see gui.editors.question.QuestionEditor
 * @see QuestionInfo
 * @see QuestionTypeContent
 */
public class QuestionObjectPanel extends DPanel implements ActionListener {
    private SelectorField category;
    private JTextField value;
    private JTextArea comment;

    private JButton changeCategory;

    private JRadioButton multChoice;
    private JRadioButton dark;
    private JRadioButton input;
    private JRadioButton autoGen;

    private QuestionTypeContent questionContentPanel;
    private QuestionTreeModel model;
    private int valueBackup;

    public QuestionObjectPanel() {
        this.questionContentPanel = new QuestionTypeContent();

        this.category = new SelectorField();
        this.value = new JTextField();
        this.comment = new JTextArea();

        this.changeCategory = new JButton("...");
        this.changeCategory.addActionListener(this);

        this.multChoice = new JRadioButton("Multiple Choice");
        this.dark = new JRadioButton("Dark");
        this.input = new JRadioButton("Input");
        this.autoGen = new JRadioButton("Autogenerate");

        QuestionTypeListener radiolistener = new QuestionTypeListener(this);
        this.multChoice.addActionListener(radiolistener);
        this.input.addActionListener(radiolistener);
        this.dark.addActionListener(radiolistener);
        this.autoGen.addActionListener(radiolistener);

        QuestionInfo infoPanel = new QuestionInfo(category, value, comment, changeCategory, multChoice, dark, input, autoGen);

        this.setLayout(new BorderLayout());
        this.add(infoPanel, BorderLayout.NORTH);
        this.add(questionContentPanel, BorderLayout.CENTER);

        this.multChoice.doClick();
    }

    /**
     * Notifies the <code>QuestionTypeContent</code> whenever a
     * type of question is selected, so that the content can be changed
     * correspondingly.
     *
     * @param b The new selection made by the user.
     */
    public void switchPanel(JRadioButton b) {
        if      (b.equals(multChoice)) questionContentPanel.showMultChoice();
        else if (b.equals(input))      questionContentPanel.showInput();
        else if (b.equals(dark))       questionContentPanel.showDark();
        else if (b.equals(autoGen))    questionContentPanel.showAutoGen();
    }

    public void showObject(Question question) {
        this.category.setText(question.getParent().getName());
        this.value.setText(Integer.toString(question.getValue()));
        this.valueBackup = question.getValue();
        this.comment.setText(question.getComment());
        this.questionContentPanel.showQuestion(question);

        if (question instanceof MultiChoiceQuestion)    selectRadioButton(multChoice);
        else if (question instanceof InputQuestion)     selectRadioButton(input);
        else if (question instanceof DarkQuestion)      selectRadioButton(dark);
        else if (question instanceof AutoGenQuestion)   selectRadioButton(autoGen);
    }

    private void selectRadioButton(JRadioButton button) {
        multChoice.setSelected(button.equals(multChoice));
        multChoice.setEnabled(button.equals(multChoice));

        input.setSelected(button.equals(input));
        input.setEnabled(button.equals(input));

        dark.setSelected(button.equals(dark));
        dark.setEnabled(button.equals(dark));

        autoGen.setSelected(button.equals(autoGen));
        autoGen.setEnabled(button.equals(autoGen));
    }

    public void saveObject(Question question) {

        int valueTemp;

        try {
            valueTemp = Integer.parseInt(value.getText());
        } catch (NumberFormatException e) {
            value.setText(Integer.toString(valueBackup));
            valueTemp = valueBackup;
        }

        question.setValue(valueTemp);
        question.setComment(comment.getText());

        questionContentPanel.saveObject(question);
    }

    public void setModel(QuestionTreeModel model) {
        this.model = model;
        this.questionContentPanel.setModel(model);
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource().equals(changeCategory)) new ParentSelector(this, this.model);
    }

    public void setEditable(boolean editable) {
        questionContentPanel.setEditable(editable);
    }
}
