package gui.listeners;

import gui.editors.common.InfoPanel;
import gui.editors.common.selector.NewObjectSelector;
import gui.editors.map.MapViewPanel;
import gui.tabs.LevelTab;
import sys.datatypes.MapPosition;
import sys.models.treemodels.LevelTreeModel;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.*;

/**
 * Listener managing events on the MapViewPanel.
 *
 * @author Tarje Killingberg 24.jun.2006
 */
public class MapViewPanelListener implements ActionListener, AdjustmentListener, ChangeListener, KeyListener, MouseListener, MouseWheelListener {


    private MapViewPanel mapViewPanel;
    private InfoPanel infoPanel;

    public boolean UP = false;
    public boolean DOWN = false;
    public boolean LEFT = false;
    public boolean RIGHT = false;

    private Point pointOfLastRightClick;


    public MapViewPanelListener(MapViewPanel mapViewPanel) {
        this.mapViewPanel = mapViewPanel;
        this.infoPanel = null;
        this.pointOfLastRightClick = new Point(10, 10);
    }

    public void setInfoPanel(InfoPanel infoPanel) {
        this.infoPanel = infoPanel;
    }


    public InfoPanel getInfoPanel() {
        return infoPanel;
    }


    public boolean[] getPressedKeys() {
        return new boolean[] {UP, DOWN, LEFT, RIGHT};
    }



    /***** Methods from interface ActionListener *****/

    /**
     * Creates a new MapItem if the user clicks "New" on the right-click
     * menu, and closes the frame if the user clicks the "OK" button in the
     * MapView frame.
     *
     * @param e The ActionEvent
     */
    public void actionPerformed(ActionEvent e) {
        String actionCommand = e.getActionCommand();
        if (actionCommand.equalsIgnoreCase("New ...")) {
            Component comp = mapViewPanel;
            while (!(comp instanceof LevelTab)) {
                comp = comp.getParent();
            }
            LevelTreeModel slm = (LevelTreeModel)((LevelTab)comp).getModel();
            slm.setActivePoint(pointOfLastRightClick);

            new NewObjectSelector(((LevelTab)comp).getSuperTreePanel(), slm);
        }
        else if (actionCommand.equalsIgnoreCase("OK")) {
            infoPanel.closeMapPositionChooser();
        }
    }


    /***** Methods from interface AdjustmentListener *****/

    /**
     * Called when the value of the scrollbars is changed, that is when the
     * user scrolls the viewport using the mouse wheel or the arrow buttons.
     *
     * @param e
     */
    public void adjustmentValueChanged(AdjustmentEvent e) {
        Adjustable adjustable = e.getAdjustable();
        int adjustableValue = adjustable.getValue();
        Point currentPoint = mapViewPanel.getCurrentLocation();

        switch (adjustable.getOrientation()) {
            case Adjustable.VERTICAL:
                if (currentPoint.getY() != adjustableValue) {
                    mapViewPanel.scrollMapToY(adjustableValue);
                }
                break;
            case Adjustable.HORIZONTAL:
                if (currentPoint.getX() != adjustableValue) {
                    mapViewPanel.scrollMapToX(adjustableValue);
                }
                break;
            default:
                break;
        }
    }


    /***** Methods from interface ChangeListener *****/

    /**
     * Called when the user changes the values of either the fps spinner or the
     * speed slider.
     *
     * @param e
     */
    public void stateChanged(ChangeEvent e) {
        Object source = e.getSource();
        if (source instanceof JSpinner) {
            JSpinner spinner = (JSpinner)source;
            mapViewPanel.setFps((Integer)spinner.getValue());
        }
        else if (source instanceof JSlider) {
            JSlider slider = (JSlider)source;
            mapViewPanel.setSpeedX(slider.getValue());
            mapViewPanel.setSpeedY(slider.getValue());
        }
    }



    /***** Methods from interface KeyListener *****/

    /**
     * Called then the user presses the arrow keys. Leads to the viewport
     * being scrolled in the desired direction as long as the user holds the
     * key.
     *
     * @param e
     */
    public void keyPressed(KeyEvent e) {
        switch(e.getKeyCode()) {
            case KeyEvent.VK_UP:
                UP = true;
                break;
            case KeyEvent.VK_DOWN:
                DOWN = true;
                break;
            case KeyEvent.VK_LEFT:
                LEFT = true;
                break;
            case KeyEvent.VK_RIGHT:
                RIGHT = true;
                break;
            default:
                break;
        }
    }

    /**
     * Stops scrolling in the direction of the key being released.
     * If the key being released is the '+' or '-' keys, the object's z-position
     * is updated accordingly (if the current mapviewpanel is framed).
     *
     * @param e
     */
    public void keyReleased(KeyEvent e) {
        switch(e.getKeyCode()) {
            case KeyEvent.VK_UP:
                UP = false;
                break;
            case KeyEvent.VK_DOWN:
                DOWN = false;
                break;
            case KeyEvent.VK_LEFT:
                LEFT = false;
                break;
            case KeyEvent.VK_RIGHT:
                RIGHT = false;
                break;
            default:
                break;
        }

        if (infoPanel != null) {
            MapPosition oldPosition;
            MapPosition newPosition;
            switch (e.getKeyCode()) {
                case KeyEvent.VK_ADD:
                case KeyEvent.VK_PLUS:
                    oldPosition = infoPanel.getMapPosition();
                    newPosition = new MapPosition(oldPosition.getX(), oldPosition.getY(), oldPosition.getZ() + 1);
                    infoPanel.setMapPosition(newPosition);
                    infoPanel.mapPositionChanged();
                    break;
                case KeyEvent.VK_SUBTRACT:
                case KeyEvent.VK_MINUS:
                    oldPosition = infoPanel.getMapPosition();
                    newPosition = new MapPosition(oldPosition.getX(), oldPosition.getY(), oldPosition.getZ() - 1);
                    infoPanel.setMapPosition(newPosition);
                    infoPanel.mapPositionChanged();
                    break;
                default:
                    break;
            }
        }
    }


    public void keyTyped(KeyEvent e) {
    }


    /***** Methods from interface MouseListener *****/

    /**
     * Updates the current object's x- and y-coordinates if the current
     * MapViewPanel is framed.
     * Shows the menu containing "New" if the user right clicks and the
     * current MapViewPanel is <i>not</i> framed.
     *
     * @param e
     */
    public void mouseClicked(MouseEvent e) {
        Point clickPoint = new Point(e.getX(), e.getY());
        Point pointOfLastClick = mapViewPanel.translateToMapPoint(clickPoint);

        if (e.getButton() == MouseEvent.BUTTON1) {
            if (e.getClickCount() == 1) {
                if (infoPanel != null) {    // means we are in select-position-mode
                    int z;
                    MapPosition oldMapPosition = infoPanel.getMapPosition();
                    z = oldMapPosition.getZ();

                    infoPanel.setMapPosition(new MapPosition((int)pointOfLastClick.getX(), (int)pointOfLastClick.getY(), z));
                    infoPanel.mapPositionChanged();
                    mapViewPanel.requestFocusInWindow();
                }
                else {
                    mapViewPanel.mouseClicked(e.getX(), e.getY());
                    mapViewPanel.requestFocusInWindow();
                }
            }
            else if (e.getClickCount() == 2) {
            }
        }
        else if (e.getButton() == MouseEvent.BUTTON3 && e.getClickCount() == 1) {
            if (infoPanel == null) {        // means we are not in select-position-mode
                pointOfLastRightClick = pointOfLastClick;
                JPopupMenu popupMenu = new JPopupMenu();
                JMenuItem newItem = new JMenuItem("New ...");
                newItem.addActionListener(this);
                popupMenu.add(newItem);
                popupMenu.show(mapViewPanel, e.getX(), e.getY());
            }
        }
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }


    /***** Methods from interface MouseWheelListener *****/

    /**
     * Scrolls the viewport in the desired direction.
     * If <i>ctrl</i> is held down, the viewport is scrolled horizontally.
     * 
     * @param e
     */
    public void mouseWheelMoved(MouseWheelEvent e) {
        if (e.getScrollType() == MouseWheelEvent.WHEEL_UNIT_SCROLL) {
            int totalScrollAmount;
            if (e.isControlDown() || e.isShiftDown() || e.isAltDown()) {
                totalScrollAmount = e.getUnitsToScroll() * mapViewPanel.getHorizontalScrollBar().getUnitIncrement();
                mapViewPanel.scrollMapBy(totalScrollAmount, 0);
            }
            else {
                totalScrollAmount = e.getUnitsToScroll() * mapViewPanel.getVerticalScrollBar().getUnitIncrement();
                mapViewPanel.scrollMapBy(0, totalScrollAmount);
            }
        }
    }


}
