package gui.tabs;

import gui.ButtonPanel;
import gui.Saver;
import gui.editors.AbstractEditor;
import gui.listeners.SuperTreeSelectionListener;
import gui.tree.AbstractTreePanel;
import sys.datatypes.interfaces.AoCObject;
import sys.models.treemodels.AbstractTreeModel;

import javax.swing.*;
import java.awt.*;

/**
 * The superclass of the three different editors. Contains the
 * panels of the editor (<code>AbstractEditor</code>, <code>AbstractTreePanel</code> and
 * <code>ButtonPanel</code>). The implementing subclasses contain references
 * to their corresponding models.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 * @see gui.editors.AbstractEditor
 * @see gui.tree.AbstractTreePanel
 * @see gui.ButtonPanel
 * @see gui.tabs.LevelTab
 * @see gui.tabs.QuestionTab
 * @see gui.tabs.TextTab
 */
public abstract class AbstractTab extends JDesktopPane implements Saver {
    protected AbstractEditor editor;
    protected AbstractTreePanel tree;
    protected ButtonPanel buttons;

    protected AbstractTreeModel model;

    protected SuperTreeSelectionListener treelistener;

    public AbstractTab() {
        super();
        this.setLayout(new BorderLayout());
        treelistener = new SuperTreeSelectionListener();

        setEditor();
        setTree();
        this.buttons = new ButtonPanel(this);

        tree.addSelectionListener(treelistener);

        placeComponents();
    }

    /**
     * Sets the editor corresponding to the subclass' view model.
     * @see AbstractEditor
     */
    public abstract void setEditor();

    /**
     * Sets the tree corresponding to the subclass' view model.
     * @see AbstractTreePanel
     */
    public abstract void setTree();

    /**
     * Sets the model for the selected editor view.
     *
     * @param model The model for the selected editor view.
     */
    public abstract void setModel(AbstractTreeModel model);

    private void placeComponents() {
        JPanel r = new JPanel(new BorderLayout());
        r.add(editor, BorderLayout.CENTER);
        r.add(buttons, BorderLayout.SOUTH);

        JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                                tree,
                                r);
        
        this.add(split);
    }

    public void updateObject(AoCObject current) {
        this.editor.updateObject(current);
    }

    public void showObject(AoCObject selection) {
        this.editor.showObject(selection);
    }

    public AbstractTreePanel getSuperTreePanel() {
        return tree;
    }

    public AbstractTreeModel getModel() {
        return model;
    }

    public AbstractEditor getEditor() {
        return editor;
    }

    public void superLevelModelChanged() {
        editor.superLevelModelChanged();
    }

    /* Methods implemented from Saver */

    public void saveButtonPressed() {
        AoCObject currentObject = model.getModelManager().getCurrentObject();
        model.getModelManager().fireSaveClicked(currentObject);
    }

    public void discardButtonPressed() {
        int choice = JOptionPane.showConfirmDialog(this,
                "This will reset the currently selected object.\n" +
                        "It will not change the object's parent\n" +
                        "(position in the tree), if this has been updated.\n" +
                        "Continue?",
                "Confirm",
                JOptionPane.YES_NO_OPTION);

        if (choice == JOptionPane.YES_OPTION) {
            model.getModelManager().undoChanges(model.getModelManager().getCurrentObject());
        }
    }
}
