package gui.tree;

import gui.editors.common.selector.NewObjectSelector;
import gui.listeners.SuperTreeSelectionListener;
import sys.models.treemodels.AbstractTreeModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * The left panel in the editor. Contains a tree based on the corresponding model,
 * a list of available filtering options and buttons for adding and deleting
 * tree nodes.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 * @see gui.tabs.AbstractTab
 * @see LevelTreePanel
 * @see QuestionTreePanel
 * @see TextTreePanel
 * @see AbstractTreeModel
 */
public abstract class AbstractTreePanel extends JPanel implements ActionListener {

    protected SuperTree tree;
    protected AbstractTreeModel model;

    protected JButton addNode;
    protected JButton removeNode;

    protected JCheckBox[] filters;

    private JPanel actionPanel; //The panel on which the filter panel is placed

    public AbstractTreePanel() {
        this.setBackground(Color.WHITE);
        this.setBorder(BorderFactory.createEtchedBorder());

        tree = new SuperTree();

        JScrollPane treeScroll = new JScrollPane(tree);
        treeScroll.getVerticalScrollBar().setUnitIncrement(3);

        addNode = new JButton("Add");
        addNode.addActionListener(this);

        removeNode = new JButton("Remove");
        removeNode.addActionListener(this);

        JPanel buttonPanel = new JPanel(new GridLayout(1, 2));
        buttonPanel.setBorder(BorderFactory.createTitledBorder("Operations"));

        buttonPanel.add(addNode);
        buttonPanel.add(removeNode);

        this.actionPanel = new JPanel(new BorderLayout());

        actionPanel.add(buttonPanel, BorderLayout.NORTH);

        this.setLayout(new BorderLayout());
        this.add(new JScrollPane(tree), BorderLayout.CENTER);
        this.add(actionPanel, BorderLayout.SOUTH);

        this.setPreferredSize(new Dimension(250, 0));
    }

    /**
     * Sets the model representing the tree displayed in this panel.
     *
     * @param model
     */
    public void setModel(AbstractTreeModel model) {
        this.model = model;
        this.tree.setModel(model);
        this.model.setTree(tree);
        this.createFilters();
    }

    /**
     * Creates the filter options displayed below the tree
     */
    protected void createFilters() {
        String[] filterNames = model.getFilterNames();
        this.filters = new JCheckBox[filterNames.length];

        if (!(filterNames.length > 0)) return;

        JPanel filterPanel = new JPanel();
        filterPanel.setLayout(new GridLayout(filters.length, 1));
        filterPanel.setBorder(BorderFactory.createTitledBorder("Filters"));

        for (int i = 0; i < filterNames.length; i++) {
            filters[i] = new JCheckBox(filterNames[i]);
            filters[i].setSelected(true);
            filters[i].addActionListener(this);
            filterPanel.add(filters[i]);
        }

        this.actionPanel.add(filterPanel, BorderLayout.SOUTH);
    }

    public void addSelectionListener(SuperTreeSelectionListener superTreeSelectionListener) {
        this.tree.addTreeSelectionListener(superTreeSelectionListener);
    }

    public AbstractTreeModel getModel() {
        return model;
    }

    /**
     * Called whenever the "Add" or "Remove" buttons are pressed, and takes corresponding action.
     * If "Add" is clicked, a popup for selecting object type is displayed.
     * If the active panel has active filters, selection and deselection of these
     * are also caught in this method.
     *
     * @param e
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();

        if (source.equals(addNode)) {
            new NewObjectSelector(this, this.model);
        }
        else if (source.equals(removeNode)) {
            this.model.removeNode();
        }
        else if (source instanceof JCheckBox) {
            JCheckBox checkbox = (JCheckBox)source;
            model.fireFilterSelected(checkbox.getActionCommand(), checkbox.isSelected());
        }
    }
}
