package sys.Parser;

import com.DatabaseCommunicator;
import com.ServerCommunicator;
import sys.AoCAdmin;
import sys.datatypes.*;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.question.*;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Set;

/**
 * Converts db-info to and from objects used in the
 * application.
 *
 * @author Eivind Sorteberg
 */
public class Parser {


    public static final String HISTORY = "history";
    public static final String FACT = "fact";
    public static final String MULTIPLE_CHOICE = "multichoice";
    public static final String INPUT = "input";
    public static final String DARK = "dark";
    public static final String AUTO_GENERATE = "autogen";


    /**
     * Connects to the database if connection is not already established
     */
    public static void connect() {
        if (!DatabaseCommunicator.isConnected()) {
            DatabaseCommunicator.connect(AoCAdmin.DATABASE_SERVER, AoCAdmin.DATABASE_USERNAME, AoCAdmin.DATABASE_PASSWORD);
        }
    }

    /**
     * Disconnects from the database if connection is established
     */
    public static void disconnect() {
        if (DatabaseCommunicator.isConnected()) DatabaseCommunicator.disconnect();
    }

    /**
     * Reads all the information from a table fulfilling given specifications
     *
     * @param table The DB table to read
     * @param filter Which conditions should be fulfilled
     * @return All entries in the given table
     */
    private static ResultSet getAllEntriesFromTable(String table, String filter) {
        String query = "SELECT * FROM " + table;
        if (filter != null) query += " WHERE " + filter;
        return DatabaseCommunicator.executeQuery(query);
    }

    /**
     * Reads one entry from a table fulfilling the primary key specified
     *
     * @param table The DB table to read
     * @param id The id of the wanted entry
     * @return The wanted entry
     */
    private static ResultSet getSingleEntryFromTable(String table, int id) {
        String query = "SELECT * FROM " + table  + " WHERE ID=" + id;
        return DatabaseCommunicator.executeQuery(query);
    }

    /**
     * @return All the levels in the database.
     */
    public static HashMap<Integer, Level> getLevels() {
        connect();

        HashMap<Integer, Level> levels = new HashMap<Integer, Level>();

        ResultSet rs = getAllEntriesFromTable("Level", null);

        try {
            while (rs.next()) {
                int id = rs.getInt("ID");
                String name = rs.getString("Name");
                String map = rs.getString("Map");

                levels.put(id, new Level(id, name, map));
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return levels;
    }

    /**
     * @return All <code>TeleportSource</code> objects in the database.
     */
    public static HashMap<Integer, TeleportSource> getTeleportSources() {
        connect();

        HashMap<Integer, TeleportSource> teleportSources = new HashMap<Integer, TeleportSource>();

        ResultSet rs = getAllEntriesFromTable("TeleportSource", null);

        try {
            while (rs.next()) {
                int id = rs.getInt("ID");
                int destID = rs.getInt("TeleportDestID");
                String text = rs.getString("Text");
                int levelID = rs.getInt("LevelID");
                int xpos = rs.getInt("PosX");
                int ypos = rs.getInt("PosY");
                int zpos = rs.getInt("PosZ");

                teleportSources.put(id, new TeleportSource(id, destID, text, levelID, new MapPosition(xpos, ypos, zpos)));
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return teleportSources;
    }

    /**
     * @return All <code>TeleportDestination</code> objects in the database.
     */
    public static HashMap<Integer, TeleportDestination> getTeleportDestinations() {
        connect();

        HashMap<Integer, TeleportDestination> teleportDestinations = new HashMap<Integer, TeleportDestination>();

        ResultSet rs = getAllEntriesFromTable("TeleportDest", null);

        try {
            while (rs.next()) {
                int id = rs.getInt("ID");
                String text = rs.getString("Text");
                int levelID = rs.getInt("LevelID");
                int xpos = rs.getInt("PosX");
                int ypos = rs.getInt("PosY");
                int zpos = rs.getInt("PosZ");

                teleportDestinations.put(id, new TeleportDestination(id, text, levelID, new MapPosition(xpos, ypos, zpos)));
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return teleportDestinations;
    }

    /**
     * @return All <code>Text</code> objects in the database.
     */
    public static HashMap<Integer, Text> getTexts() {
        connect();

        HashMap<Integer, Text> texts = new HashMap<Integer, Text>();

        ResultSet rs = getAllEntriesFromTable("Text", null);

        try {
            while (rs.next()) {
                int id = rs.getInt("ID");
                int group = rs.getInt("TextGroupID");
                String name = rs.getString("Name");
                String text = rs.getString("Text");
                String latex = rs.getString("LaTeX");
                int level = rs.getInt("LevelID");
                int x = rs.getInt("PosX");
                int y = rs.getInt("PosY");
                int z = rs.getInt("PosZ");
                int sequence = rs.getInt("Sequence");

                if (rs.getString("Type").equals(FACT)) {
                    texts.put(id, new Fact(id, group, name, text, latex, level, new MapPosition(x, y, z), sequence));
                }
                else if (rs.getString("Type").equals(HISTORY)) {
                    texts.put(id, new History(id, group, name, text, latex, level, new MapPosition(x, y, z), sequence));
                }
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return texts;
    }

    /**
     * @return All <code>TextGroup</code> objects in the database.
     */
    public static HashMap<Integer, TextGroup> getTextGroups() {
        connect();

        HashMap<Integer, TextGroup> textgroups = new HashMap<Integer, TextGroup>();

        ResultSet rs = getAllEntriesFromTable("TextGroup", null);

        try {
            while(rs.next()) {
                int id = rs.getInt("ID");
                int parent = rs.getInt("ParentID");
                String name = rs.getString("Name");
                String text = rs.getString("Text");
                int sequence = rs.getInt("Sequence");

                TextGroup group = new TextGroup(id, parent, name, text, sequence);

                textgroups.put(id, group);
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return textgroups;
    }

    /**
     * @return All <code>QuestionGroup</code> objects in the database.
     */
    public static HashMap<Integer, QuestionGroup> getQuestionGroups() {
        connect();

        HashMap<Integer, QuestionGroup> groups = new HashMap<Integer, QuestionGroup>();
        ResultSet rs = getAllEntriesFromTable("QuestionGroup", null);

        try {
            while (rs.next()) {
                int id = rs.getInt("ID");
                int parent = rs.getInt("ParentID");
                String name = rs.getString("Name");
                String description = rs.getString("Description");

                QuestionGroup group = new QuestionGroup(id, parent, name, description);

                groups.put(id, group);
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return groups;
    }

    /**
     * @return All <code>QuestionCategory</code> objects in the database.
     */
    public static HashMap<Integer, QuestionCategory> getQuestionCategories() {
        connect();

        HashMap<Integer,  QuestionCategory> categories = new HashMap<Integer, QuestionCategory>();

        ResultSet rs = getAllEntriesFromTable("QuestionCategory", null);

        try {
            while (rs.next()) {
                int id = rs.getInt("ID");
                int parent_id = rs.getInt("QuestionGroupID");
                String name = rs.getString("Name");
                String description = rs.getString("Description");
                String text = rs.getString("Text");
                String latex = rs.getString("LaTeX");
                int q_req = rs.getInt("QuestionsRequired");
                int level = rs.getInt("LevelID");
                int x = rs.getInt("PosX");
                int y = rs.getInt("PosY");
                int z = rs.getInt("PosZ");

                QuestionCategory cat = new QuestionCategory(id, parent_id, name, description, text, latex, q_req, level, new MapPosition(x, y, z));

                categories.put(id, cat);
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return categories;
    }

    /**
     * @return All <code>Question</code> objects in the database.
     */
    public static HashMap<Integer, Question> getQuestions() {
        connect();

        HashMap<Integer, Question> questions = new HashMap<Integer, Question>();

        ResultSet rs = getAllEntriesFromTable("Question", null);

        try {
            while (rs.next()) {
                int id = rs.getInt("ID");
                int parent = rs.getInt("QuestionCategoryID");
                String text = rs.getString("Text");
                String latex = rs.getString("LaTeX");
                int value = rs.getInt("Value");
                String comment = rs.getString("Comment");
                String type = rs.getString("Type");

                Question question;

                if (type.equals(MULTIPLE_CHOICE)) {
                    ResultSet answer_rs = getSingleEntryFromTable("MultiChoiceQuestion", id);
                    answer_rs.next();
                    int answer = answer_rs.getInt("AnswerID");

                    question = new MultiChoiceQuestion(id, parent, text, latex, value, comment, answer);
                }
                else if (type.equals(INPUT)) {
                    ResultSet answer_rs = getSingleEntryFromTable("InputQuestion", id);
                    answer_rs.next();
                    String answer = null;
                    try {
                        answer = answer_rs.getString("Answer");
                    } catch (SQLException e) {
                    }

                    question = new InputQuestion(id, parent, text, latex, value, comment, answer);
                }
                else if (type.equals(DARK)) {
                    ResultSet answer_rs = getSingleEntryFromTable("DarkQuestion", id);
                    answer_rs.next();
                    String code = answer_rs.getString("Code");
                    String oldCode = answer_rs.getString("OldCode");
                    String testCode = answer_rs.getString("TestCode");
                    String suggSol = answer_rs.getString("SuggestedSolution");


                    question = new DarkQuestion(id, parent, text, latex, value, comment, code, oldCode, testCode, suggSol);
                }
                else if (type.equals(AUTO_GENERATE)) {
                    ResultSet answer_rs = getSingleEntryFromTable("AutoGenQuestion", id);
                    answer_rs.next();
                    String code = answer_rs.getString("Code");
                    String oldCode = answer_rs.getString("OldCode");
                    String formula = answer_rs.getString("Formula");

                    question = new AutoGenQuestion(id, parent, text, latex, value, comment, code, oldCode, formula);
                }
                else {
                    continue;
                }

                questions.put(id, question);
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return questions;
    }

    /**
     * @return All <code>MultiChoiceAlternative</code> objects in the database.
     */
    public static ArrayList<MultiChoiceAlternative> getMultiChoiceAlternatives() {
        connect();

        ArrayList<MultiChoiceAlternative> alternatives = new ArrayList<MultiChoiceAlternative>();

        ResultSet rs = getAllEntriesFromTable("MultiChoiceAlternative", null);

        try {
            while (rs.next()) {
                int questionID = rs.getInt("QuestionID");
                int alternativeID = rs.getInt("AlternativeID");
                String text = rs.getString("Text");
                String latex = rs.getString("LaTeX");

                MultiChoiceAlternative alternative = new MultiChoiceAlternative(alternativeID, questionID, text, latex);

                alternatives.add(alternative);
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return alternatives;
    }

    /**
     * @return All <code>Message</code> objects in the database.
     */
    public static HashMap<Integer, Message> getMessages() {
        connect();

        HashMap<Integer,  Message> messages = new HashMap<Integer, Message>();

        ResultSet rs = getAllEntriesFromTable("Message", null);

        try {
            while (rs.next()) {
                int id = rs.getInt("ID");
                String text = rs.getString("Text");
                int levelId = rs.getInt("LevelID");
                int x = rs.getInt("PosX");
                int y = rs.getInt("PosY");
                int z = rs.getInt("PosZ");

                Message msg = new Message(id, levelId, text, new MapPosition(x, y, z));
                messages.put(id, msg);
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return messages;
    }

    /**
     * @return All <code>TeleportRequirement</code> objects in the database.
     */
    public static Object[] getTeleportRequirements() {
        connect();

        HashMap<Integer, ArrayList<Integer>> teleportRequirements = new HashMap<Integer, ArrayList<Integer>>();
        HashMap<Integer, ArrayList<Integer>> requirementTeleports = new HashMap<Integer, ArrayList<Integer>>();

        ResultSet rs = getAllEntriesFromTable("TeleportRequirement", null);
        try {
            while(rs.next()) {
                int teleportSourceID = rs.getInt("TeleportSourceID");
                int questionCategoryID = rs.getInt("QuestionCategoryID");

                /* building the teleportRequirements-map */
                ArrayList<Integer> questionCategoryList = teleportRequirements.get(teleportSourceID);
                if (questionCategoryList == null) {
                    questionCategoryList = new ArrayList<Integer>();
                    teleportRequirements.put(teleportSourceID, questionCategoryList);
                }
                questionCategoryList.add(questionCategoryID);

                /* building the requirementTeleports-map */
                ArrayList<Integer> teleportSourceList = requirementTeleports.get(questionCategoryID);
                if (teleportSourceList == null) {
                    teleportSourceList = new ArrayList<Integer>();
                    requirementTeleports.put(questionCategoryID, teleportSourceList);
                }
                teleportSourceList.add(teleportSourceID);
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return new Object[] {teleportRequirements, requirementTeleports};
    }

    /**
     * @return All <code>User</code> objects in the database.
     */
    public static HashMap<Integer, User> getUsers() {
        connect();

        HashMap<Integer, User> users = new HashMap<Integer, User>();

        ResultSet rs = getAllEntriesFromTable("User", null);

        try {
            while (rs.next()) {
                int id = rs.getInt("ID");
                String username = rs.getString("Username");
                String password = rs.getString("Password");
                boolean admin = rs.getBoolean("Admin");
                int avatar = rs.getInt("Avatar");
                int levelId = rs.getInt("LevelID");
                int x = rs.getInt("PosX");
                int y = rs.getInt("PosY");
                int z = rs.getInt("PosZ");

                User user = new User(id, username, password, admin, avatar, levelId, new MapPosition(x, y, z));
                users.put(id, user);
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return users;
    }

    /**
     * @return All <code>UserQuestion</code> objects in the database.
     */
    public static HashMap<Integer, ArrayList<Integer>> getUserQuestions() {
        connect();

        /* Collecting the questions the users have answered correctly */
        HashMap<Integer, ArrayList<Integer>> userQuestions = new HashMap<Integer, ArrayList<Integer>>();
        ResultSet rs = getAllEntriesFromTable("UserQuestion", "Correct=1");

        try {
            while (rs.next()) {
                int userID = rs.getInt("UserID");
                int questionID = rs.getInt("QuestionID");

                ArrayList<Integer> questionIDList = userQuestions.get(userID);
                if (questionIDList == null) {
                    questionIDList = new ArrayList<Integer>();
                    userQuestions.put(userID, questionIDList);
                }

                questionIDList.add(questionID);
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return userQuestions;
    }

    /**
     * @return All Settings in the database.
     */
    public static HashMap<String, String> getSettings() {
        connect();

        HashMap<String, String> settings = new HashMap<String, String>();
        ResultSet rs = getAllEntriesFromTable("Settings", null);

        try {
            while (rs.next()) {
                String name = rs.getString("Name");
                String value = rs.getString("Value");

                settings.put(name, value);
            }
            rs.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return settings;
    }

    /**
     * Updates a setting in the database with the specified value.
     *
     * @param name The name of the setting
     * @param value The new value
     */
    public static void updateSetting(String name, String value) {
        connect();

        String query = "UPDATE Settings" +
                       " SET Value='" + value + "'" +
                       " WHERE Name='" + name + "'";

        DatabaseCommunicator.executeUpdate(query);
        disconnect();
    }



    /**
     * Takes in a list of AoCObjects and updates them in the database if they exist
     * from before. If the objects are new, they are added to the database.
     *
     * @param objects The list of changed objects
     * @return Whether the operation was successful
     */
    public static boolean updateObjects(ArrayList<AoCObject> objects) {
        connect();

        for (AoCObject object : objects) {
            if (object.getId() < 0) {
                createNewObject(object, true);
            }
            else {
                updateExistingObject(object);
            }
        }

        disconnect();
        return false;
    }


    /**
     * Updates objects already existing in the database.
     *
     * @param object The DB-object to update.
     */
    private static void updateExistingObject(AoCObject object) {
        connect();

        String query = "UPDATE " + object.getDBname() +
                " SET " + buildValueString(object.getFieldNames(), object.getFieldValues(), ",") +
                " WHERE " + buildValueString(object.getIdFields(), object.getIdValues(), "AND");

        DatabaseCommunicator.executeUpdate(query);

        if (object instanceof Question) {
            updateQuestionType((Question)object);

            if (object instanceof MultiChoiceQuestion) {
                for (MultiChoiceAlternative alternative : ((MultiChoiceQuestion)object).getAlternatives()) {
                    updateExistingObject(alternative);
                }
            }
        }

        /* if object is a teleport source, we have to update the requirements as well */
        if (object instanceof TeleportSource) {
            updateTeleportSourceRequirements((TeleportSource)object);
        }

        disconnect();
    }


    private static void updateQuestionType(Question question) {
        connect();

        String query = "UPDATE " + question.getSpecialDBname() + " SET ";
        query += buildValueString(question.getSpecialFieldNames(), question.getSpecialAttributes(), ",");
        query += " WHERE ID=" + question.getId() + ";";

        DatabaseCommunicator.executeUpdate(query);

        disconnect();
    }

    /**
     * Builds query strings using field names and values corresponding to each other.
     *
     * @param fieldNames The field names affected by the query
     * @param values The values of the fields
     * @param separator The separator. Either "," or "AND".
     * @return Query string
     */
    private static String buildValueString(String[] fieldNames, Object[] values, String separator) {
        String string = "";

        for (int i = 0; i < values.length; i++) {
            string += fieldNames[i] + "=";

            if (values[i] instanceof String) {
                String value = (String)values[i];
                while (value.endsWith("\\") || value.endsWith("\\'")) {
                    value = value.substring(0, value.length() - 1);
                }
                value = value.replaceAll("\\\\", "\\\\\\\\");
                value = value.replaceAll("'", "\\\\\'");


                string += "'" + value + "'";
            }
            else if (values[i] instanceof Integer)  {
                string += (Integer)values[i] >= 0 ? Integer.toString((Integer)values[i]) : "null";

            }
            else if (values[i] instanceof Boolean)  string += Boolean.toString((Boolean)values[i]);


            if (i < values.length - 1) {
                string += " " + separator + " ";
            }
        }
        return string;
    }


    /**
     * Creates new objects in the database. If the object is a question object, both
     * the Question table and the object's corresponding QuestionType table
     * are updated with the new object.
     *
     * @param object The new object
     */
    public static int createNewObject(AoCObject object, boolean autoIncrement) {
        connect();

        String query = "INSERT INTO " + object.getDBname() + " VALUES(";

        if (autoIncrement) query += "null" + ", ";

        Object[] attributes = object.getFieldValues();

        for (int i = 0; i < attributes.length; i++) {
            Object attribute = attributes[i];

            if (attribute instanceof String)        query += "'" + attribute + "'";
            else if (attribute instanceof Integer)  query += attribute;
            else if (attribute instanceof Boolean)  query += attribute;

            if (i < attributes.length - 1) query += ", ";
        }
        query += ");";

        //The id of the newly created object; is needed to keep mappings
        int new_id = object.getId();

        if (autoIncrement) new_id = DatabaseCommunicator.executeAutoIncrementInsert(query);
        else DatabaseCommunicator.executeInsert(query);


        //If the object is a question, it also needs to be added to the corresponding type table
        if (object instanceof Question) {
            createQuestionType((Question)object, new_id);
        }

        disconnect();
        return new_id;
    }


    /**
     * Adds a question to its corresponding question type table, while keeping its
     * reference to the question table intact.
     *
     * @param question      The question to be added
     * @param question_id   The reference into the Question table
     */
    private static void createQuestionType(Question question, int question_id) {
        connect();

        String query = "INSERT INTO " + question.getSpecialDBname() + " VALUES(" + question_id + ", ";

        Object[] attributes = question.getSpecialAttributes();

        for (int i = 0; i < attributes.length; i++) {
            Object attribute = attributes[i];

            if (attribute instanceof String)        query += "'" + attribute + "'";
            else if (attribute instanceof Integer)  query += attribute;

            if (i < attributes.length - 1) query += ", ";
        }
        query += ");";

        DatabaseCommunicator.executeInsert(query);
        disconnect();
    }


    /**
     * Performs updates in the TeleportRequirement-table when a teleportsource has changed.
     *
     * @param teleportSource The TeleportSource-object that has changed
     */
    private static void updateTeleportSourceRequirements(TeleportSource teleportSource) {
        connect();

        int teleportSourceID;
        ArrayList<QuestionCategory> requiredCategories;
        ArrayList<Integer> requiredCategoryIDs;
        String query;
        ResultSet rs;

        teleportSourceID = teleportSource.getId();
        requiredCategories = teleportSource.getRequirementList();
        requiredCategoryIDs = new ArrayList<Integer>();

        for (QuestionCategory requiredCategory : requiredCategories) {
            requiredCategoryIDs.add(requiredCategory.getId());
        }

        try {
            rs = getAllEntriesFromTable("TeleportRequirement", "TeleportSourceID=" + teleportSourceID);

            /* deleting from DB the ones that are obsolete: */
            while (rs.next()) {
                int DB_QuestionCategoryID = rs.getInt("QuestionCategoryID");
                if (!(requiredCategoryIDs.contains(DB_QuestionCategoryID))) {
                    query = "DELETE FROM " +
                            "TeleportRequirement" +
                            " WHERE " +
                            "TeleportSourceID=" + teleportSourceID +
                            " AND " +
                            "QuestionCategoryID=" + DB_QuestionCategoryID;

                    DatabaseCommunicator.executeUpdate(query);
                }
            }
            rs.close();

            rs = getAllEntriesFromTable("TeleportRequirement", "TeleportSourceID=" + teleportSourceID);

            /* rs now contains a subset of the actual requirements */
            /* removing from requiredCategoryIDs the ones already in DB: */
            while (rs.next()) {
                int DB_QuestionCategoryID = rs.getInt("QuestionCategoryID");
                if (requiredCategoryIDs.contains(DB_QuestionCategoryID)) {
                    requiredCategoryIDs.remove(new Integer(DB_QuestionCategoryID));
                }
            }
            rs.close();

            /* requiredCategoryIDs is now supposed to contain only the */
            /* requirements that need to be saved to DB */
            for (Integer requiredCategoryID1 : requiredCategoryIDs) {
                int requiredCategoryID;
                requiredCategoryID = requiredCategoryID1;

                query = "INSERT INTO " +
                        "TeleportRequirement" +
                        " VALUES(" +
                        teleportSourceID + ", " +
                        requiredCategoryID +
                        ")";

                DatabaseCommunicator.executeUpdate(query);
            }

        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
    }


    /**
     * Returns the number of a new object when an object is created with
     * a given attribute in common with an already existing object.
     *
     * @param object The new object
     * @param nameField The attribute in common
     * @return The attribute's value with a number behind
     */
    public static int getCount(AoCObject object, String nameField) {
        connect();

        int max = 0;

            String qCount = "SELECT COUNT(*) as Count FROM " + object.getDBname() + " WHERE " + nameField + " LIKE '";

            if (object instanceof Group)        qCount += ((Group)object).getName();
            else if (object instanceof Text)    qCount += ((Text)object).getName();
            else if (object instanceof User)    qCount += ((User)object).getUsername();

            qCount += "%';";
            ResultSet rs = DatabaseCommunicator.executeQuery(qCount);

            try {
                rs.next();
                max = rs.getInt("Count");
                if (max == 0) return 0;
                if (max == 1) return 1;
            } catch (SQLException e) {
                e.printStackTrace();
            }

            String query = "SELECT " + nameField + " FROM " + object.getDBname() + " WHERE " + nameField + " LIKE '";

            if (object instanceof Group)        query += ((Group)object).getName();
            else if (object instanceof Text)    query += ((Text)object).getName();
            else if (object instanceof User)    query += ((User)object).getUsername();

            query += "%';";

            rs = DatabaseCommunicator.executeQuery(query);

            try {
                while (rs.next()) {
                    String entry = rs.getString(nameField);
                    int fp = entry.lastIndexOf("(");
                    int lp = entry.lastIndexOf(")");

                    if (fp != -1 && lp != -1) {
                        int db_no = Integer.parseInt(entry.substring(fp + 1, lp));
                        if (db_no >= max) max = db_no + 1;
                    }
                }
            } catch (SQLException e) {
                e.printStackTrace();
            }

        disconnect();
        return max;
    }

    /**
     * Gets the DB representation of an object.
     * Used to control whether the user's local representation of
     * the objects corresponds with the DB representation.
     *
     * @param object The object to get from DB
     * @return The object from DB
     */
    public static AoCObject getObject(AoCObject object) {
        connect();

        String query = "SELECT * FROM " + object.getDBname() + " WHERE ";
        query += buildValueString(object.getIdFields(), object.getIdValues(), "AND");
        query += ";";
        AoCObject clone = object.makeClone();

        try {
            ResultSet rs = DatabaseCommunicator.executeQuery(query);
            rs.next();

            HashMap<String, Object> attributes = getAttributes(object, rs);

            if (object instanceof Question) { //We also need the special attributes of questions
                String q_query = "SELECT * FROM " + ((Question)object).getSpecialDBname() + " WHERE ID = " + object.getId() + ";";

                ResultSet q_rs = DatabaseCommunicator.executeQuery(q_query);
                q_rs.next();

                attributes.putAll(getSpecialAttributes((Question)object, q_rs));
            }

            clone.setAllAttributes(attributes);

        } catch (SQLException e) {
            e.printStackTrace();
        }

        disconnect();
        return clone;
    }

    /**
     * Gets all the attributes of an object as a hashmap.
     *
     * @param object A template for the attribute extraction
     * @param rs ResultSet containing the attributes
     * @return HashMap containing the attributes as Objects with the field names as keys.
     */
    private static HashMap<String, Object> getAttributes(AoCObject object, ResultSet rs) {
        try {
            String[] idFields = object.getIdFields();
            String[] otherFields = object.getFieldNames();
            Object[] values = object.getFieldValues();

            HashMap<String, Object> attributes = new HashMap<String, Object>();

            for (String idField : idFields) {
                attributes.put(idField, rs.getInt(idField));
            }

            for (int i = 0; i < otherFields.length; i++) {
                Object type = values[i];

                if      (type instanceof String)    attributes.put(otherFields[i], rs.getString(otherFields[i]));
                else if (type instanceof Integer)   attributes.put(otherFields[i], rs.getInt(otherFields[i]));
            }
            return attributes;
        } catch (SQLException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * The same as getAttributes(), but used to extract the special attributes of <code>Question</code> objects
     *
     * @param question Template for the Question
     * @param rs The resultset containing the values
     * @return The special attributes
     */
    private static HashMap<String, Object> getSpecialAttributes(Question question, ResultSet rs) {
        try {
            String[] specialFields = question.getSpecialFieldNames();
            Object[] specialAttributes = question.getSpecialAttributes();

            HashMap<String, Object> attributes = new HashMap<String, Object>();

            for (int i = 0; i < specialFields.length; i++) {
                Object type = specialAttributes[i];

                if      (type instanceof String)    attributes.put(specialFields[i], rs.getString(specialFields[i]));
                else if (type instanceof Integer)   attributes.put(specialFields[i], rs.getInt(specialFields[i]));
            }
            return attributes;
        } catch (SQLException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * Deletes a given object from the database
     *
     * @param object The object to delete
     */
    public static void deleteObject(AoCObject object) {
        connect();

        String query = "DELETE FROM " + object.getDBname() + " WHERE ";
        query += buildValueString(object.getIdFields(), object.getIdValues(), " AND ");
        query += ";";
        DatabaseCommunicator.executeUpdate(query);

        disconnect();
    }


    /**
     * Deletes potentially many rows from a table based on a single field.
     * All rows with value <code>fieldValue</code> in field <code>fieldName</code>
     * will be removed from table <code>tableName</code>.
     *
     * @param tableName name of the table
     * @param fieldName name of the field
     * @param fieldValue the value to delete
     */
    public static void deleteMultipleRows(String tableName, String fieldName, Object fieldValue) {
        connect();

        String query = "DELETE FROM " +
                       tableName +
                       " WHERE " +
                       fieldName + "=" + fieldValue;

        DatabaseCommunicator.executeUpdate(query);

        disconnect();
    }



    /* ----- web server methods ----- */

    /**
     * @return The names of all the maps on the web server
     */
    public static ArrayList<String> getAllMaps() {

        ArrayList<String> maps = new ArrayList<String>();
        BufferedReader in;
        String s;
        int a, b;

        try {
            in = new BufferedReader(new InputStreamReader(ServerCommunicator.getInputStream(AoCAdmin.WEB_SERVER, AoCAdmin.MAP_FOLDER)));

            while(in.ready()) {
                s = in.readLine();
                a = s.toUpperCase().indexOf("HREF=\"");
                b = s.toUpperCase().indexOf(">", a);
                if (a >= 0 && b >= 0) {
                    s = s.substring(a, b);
                    a = s.indexOf("\"") + 1;
                    b = s.indexOf("\"", a);
                    s = s.substring(a, b);

                    if (s.toUpperCase().endsWith(".FMP")) {
                        maps.add(s);
                    }
                }
            }
            in.close();

        } catch (IOException ioe) {
            ioe.printStackTrace();
        }

        return maps;
    }

    /**
     * Updates the id's of the given objects. Used to make
     * sure <code>MultiChoiceAlternative</code> objects always cover all
     * possible id's from 0 and up.
     *
     * @param changed The objects whose id's should be updated.
     */
    public static void updateObjectIds(HashMap<Integer, AoCObject> changed) {
        connect();

        Set<Integer> keys = changed.keySet();
        Integer sortedKeys[] = new Integer[keys.size()];
        keys.toArray(sortedKeys);
        Arrays.sort(sortedKeys);

        for (Integer i : sortedKeys) {
            AoCObject object = changed.get(i);

            String query = "UPDATE " + object.getDBname() + " SET ";

            if (object instanceof MultiChoiceAlternative) {
                MultiChoiceAlternative alt = (MultiChoiceAlternative)object;

                query +=
                        "AlternativeID = " + object.getId() + " " +
                        "WHERE " +
                        "AlternativeID = " + i + " " +
                        "AND " +
                        "QuestionID = " + alt.getQuestion_id();
            }

            query += ";";

            DatabaseCommunicator.executeUpdate(query);
        }

        disconnect();
    }
}