package sys.datatypes;

import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.question.Question;
import sys.datatypes.question.QuestionCategory;
import sys.datatypes.question.QuestionGroup;
import sys.models.LevelModel;

import java.util.HashMap;

/**
 * A GroupElement is a  member of a Group (the element's
 * parent. A GroupElement can itself be a Group.
 * The group on the top level is the root of a tree representing
 * the group, while all folowing nodes in the tree are sucessors of
 * the root.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 * @see Group
 */
public abstract class GroupElement implements Comparable<GroupElement>, AoCObject {
    protected Group parentGroup;
    private int id;
    private int parent_id;
    private boolean visible;

    protected GroupElement(int id, int parent_id) {
        this.id = id;
        this.parent_id = parent_id;
        this.visible = true;
    }

    public boolean setParent(Group parentGroup) {
        if (this.parentGroup != null) {
            this.parentGroup.removeElement(this);
        }

        this.parent_id = parentGroup.getId();
        this.parentGroup = parentGroup;
        return parentGroup.addElement(this);
    }

    public Group getParent() {
        return this.parentGroup;
    }

    public void setParent_id(int parent_id) {
        this.parent_id = parent_id;
    }

    public int getParent_id() {
        return parent_id;
    }

    public boolean isVisible() {
        return visible;
    }

    public void setVisible(boolean visible) {
        this.visible = visible;
    }

    public void delete() {
        try {
            this.getParent().removeElement(this);
        } catch (NullPointerException npe) {
        }
    }

/* Abstract methods */

    /**
     * @return The tooltip displayed when holding the mouse over the object in the tree
     */
    public abstract String getToolTipText();

    /**
     * @return The name displayed on the status bar
     */
    public abstract String getStatusName();

/* Method inherited from Comparable */

    /**
     * Method used to sort Groups' group elements. This method is called when the trees have been
     * build with the information from the database. New elements added to the tree will
     * be added to the bottom of its parents list, until the database is reloaded or the
     * application is restarted.
     *
     * NOTE: Sequence numbers of <Text> and <TextGroup> are not supported on creation of new
     * objects in this version of AoCAdmin, but existing values are used to sort the trees. To
     * define the sequence of these objects, the user currently needs to define this in the
     * database manually.
     *
     * @param g2 The <code>GroupElement</code> to compare with
     * @return Negative value if this object should be before g2; positive if g2 should be before this. 0 if undefined or equal.
     */
    public int compareTo(GroupElement g2) {
        //TextGroups are listed according to their sequencenumbers (if defined -> sequence != 0)
        if (this instanceof TextGroup && g2 instanceof TextGroup) {
            int s1 = ((TextGroup)this).getSequence();
            int s2 = ((TextGroup)g2).getSequence();

            if      (s1 > 0 && s2 > 0)      return s1 - s2;
            else if (s1 == 0 && s2 == 0)    return this.toString().compareTo(g2.toString());
            else if (s1 == 0)               return 1;
            else                            return -1;
        }

        //Texts are listed according to their sequencenumbers, but facts and histories are separated
        else if (this instanceof Text && g2 instanceof Text) {
            if (this instanceof Fact && g2 instanceof History)      return -1;  //Facts are listed above histories
            else if (this instanceof History && g2 instanceof Fact) return 1;

            else {      //If they are the same kind, the sequence number decides (if it is defined -> sequence != 0)
                int s1 = ((Text)this).getSequence();
                int s2 = ((Text)g2).getSequence();

                if      (s1 != 0 && s2 != 0)    return s1 - s2;
                else if (s1 == 0 && s2 == 0)    return this.toString().compareTo(g2.toString());
                else if (s1 == 0)               return 1;
                else                            return -1;
            }
        }

        //Groups are always listed above nongroups
        else if (this instanceof QuestionGroup || this instanceof TextGroup)    return -1;
        else if (g2 instanceof QuestionGroup || g2 instanceof TextGroup)        return 1;

        //Categories are always listed above Questions, and on top in MapItem tree
        else if (this instanceof QuestionCategory)  return -1;
        else if (g2 instanceof QuestionCategory)    return 1;

        //Then follow texts..
        else if (this instanceof Text)  return -1;
        else if (g2 instanceof Text)    return 1;

        //..then TeleportSources..
        else if (this instanceof TeleportSource && g2 instanceof TeleportSource) {
            return this.toString().compareTo(g2.toString());
        }
        else if (this instanceof TeleportSource) return -1;
        else if (g2 instanceof TeleportSource)   return 1;

        //..then TelePortDestinations..
        else if (this instanceof TeleportDestination && g2 instanceof TeleportDestination) {
            return this.toString().compareTo(g2.toString());
        }
        else if (this instanceof TeleportDestination) return -1;
        else if (g2 instanceof TeleportDestination)   return 1;

        //..and finally Messages
        else if (this instanceof Message && g2 instanceof Message) {
            return this.toString().compareTo(g2.toString());
        }
        else if (this instanceof Message) return -1;
        else if (g2 instanceof Message)   return 1;

        //Questions are sorted according to their IDs..
        else if (this instanceof Question && g2 instanceof Question) {
            return this.getId() - g2.getId();
        }

        //..as are Levels
        else if (this instanceof LevelModel && g2 instanceof LevelModel) {
            return this.getId() - g2.getId();
        }

        //Otherwise the sorting is undefined (should not happen)
        else {
            System.err.println("Undefined sorting: " + this.getClass().getName() + " <> " + g2.getClass().getName());
            return 0;
        }
    }

/* Methods inherited from AoCObject */

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public boolean differsFrom(GroupElement g2) {
        return this.getId() != g2.getId() || this.getParent_id() != g2.getParent_id();
    }

    public String[] getIdFields() {
        return new String[]{"ID"};
    }

    public Object[] getIdValues() {
        return new Object[]{getId()};
    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        this.setId((Integer)attributes.get("ID"));
    }

    public String getInfo(int coloredAttributes) {
        Object[][] attributes = getAttributeArray();

        String s = "";

        for (int i = 0; i < attributes.length; i++) {
            if (((int)Math.pow(2, i) & coloredAttributes) != 0) s += "#CONFLICT_START#\n";
            s += "\n----------------------------\n" + attributes[i][0] + "\n----------------------------\n" + attributes[i][1] + "\n";
            if (((int)Math.pow(2, i) & coloredAttributes) != 0) s += "#CONFLICT_END#\n";
        }

        return s;
    }

    public String getInfo() {
        return getInfo(0);
    }

    public HashMap<String, Object> getAllAttributes() {
        Object[][] attributeArray = getAttributeArray();
        HashMap<String, Object> attributes = new HashMap<String, Object>();

        for (Object[] anAttributeArray : attributeArray) {
            attributes.put((String) anAttributeArray[0], anAttributeArray[1]);
        }

        return attributes;
    }

    public Object[][] getAttributeArray() {
        String[] idFields = getIdFields();
        Object[] idValues = getIdValues();

        String[] otherFields = getFieldNames();
        Object[] otherValues = getFieldValues();

        Object[][] array = new Object[idFields.length + otherFields.length][2];

        for (int i = 0; i < idFields.length; i++) {
            array[i][0] = idFields[i];
            array[i][1] = idValues[i];
        }

        for (int i = 0; i < otherFields.length; i++) {
            array[i + idFields.length][0] = otherFields[i];
            array[i + idFields.length][1] = otherValues[i];
        }

        return array;
    }
}
