package sys.datatypes;

import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.datatypes.question.QuestionCategory;
import sys.models.LevelModel;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

/**
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class TeleportSource extends GroupElement implements MapItem {


    public static final String DB_NAME = "TeleportSource";

    private int teleportDestinationId;
    private String text;
    private MapPosition mapPosition;
    private LevelModel levelModel;
    private ArrayList<QuestionCategory> requirementList;


    /**
     * Complete constructor.
     * Used when creating TeleportSources from database.
     *
     * @param id
     * @param teleportDestinationId
     * @param text
     * @param levelId
     * @param mapPosition
     */
    public TeleportSource(int id, int teleportDestinationId, String text, int levelId, MapPosition mapPosition) {
        super(id, levelId);
        this.teleportDestinationId = teleportDestinationId;
        if (text == null) {
            this.text = "";
        }
        else {
            this.text = text;
        }
        this.mapPosition = mapPosition;
        levelModel = null;
        requirementList = new ArrayList<QuestionCategory>();
    }


    /**
     * Creates a new TeleportSource.
     * Used when creating TeleportSources interactively.
     *
     * @param levelId
     * @param teleportDestinationID
     * @param mapPosition
     */
    public TeleportSource(int levelId, int teleportDestinationID, MapPosition mapPosition) {
        super(-1, levelId);
        this.teleportDestinationId = teleportDestinationID;
        this.text = "";
        this.mapPosition = mapPosition;
        levelModel = null;
        requirementList = new ArrayList<QuestionCategory>();
    }


    /**
     * Gets the String-represenation of this object
     *
     * @return the String-represenation of this object
     */
    public String toString() {
        return "TeleSrc " + getId();
    }



    /* GETTERS AND SETTERS */



    public int getTeleportDestinationId() {
        return teleportDestinationId;
    }


    public String getText() {
        return text != null ? text : "";
    }


    public ArrayList<QuestionCategory> getRequirementList() {
        return requirementList;
    }


    public void setRequirementList(ArrayList<QuestionCategory> requirementList) {
        Collections.sort(requirementList, new Comparator() {
            public int compare(Object o1, Object o2) {
                return o1.toString().compareToIgnoreCase(o2.toString());
            }
        });

        this.requirementList = requirementList;
    }


    public void setTeleportDestinationId(int destID) {
        teleportDestinationId = destID;
    }


    public void setText(String text) {
        this.text = text;
    }


    @Override
    public void delete() {
        super.delete();
        try {
            this.levelModel.removeElement(this);
        } catch (NullPointerException e) {
        }
    }



    /* ----- Methods from interface AoCObject ----- */


    public String getDBname() {
        return DB_NAME;
    }


    public String[] getFieldNames() {
        return new String[]{"TeleportDestID",
                            "Text",
                            "LevelID",
                            "PosX",
                            "PosY",
                            "PosZ"};
    }


    public Object[] getFieldValues() {

        return new Object[]{this.getTeleportDestinationId(),
                            this.getText(),
                            this.getParent_id(),
                            this.getMapPosition().getX(),
                            this.getMapPosition().getY(),
                            this.getMapPosition().getZ()};

    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        super.setAllAttributes(attributes);

        this.setTeleportDestinationId((Integer)attributes.get("TeleportDestID"));
        this.setText((String)attributes.get("Text"));
        this.setParent_id((Integer)attributes.get("LevelID"));

        this.setMapPosition(new MapPosition(
                ((Integer)attributes.get("PosX")),
                ((Integer)attributes.get("PosY")),
                ((Integer)attributes.get("PosZ"))));
    }

    public String getToolTipText() {
        return "<html><b>ID:</b> " + getId() + "<br><b>Level ID:</b> " + getLevelModel().getId() + "<br><b>Destination:</b> " + getTeleportDestinationId() + "</html>";
    }

    public AoCObject makeClone() {

        TeleportSource ts;
        ts = new TeleportSource(this.getId(),
                this.getTeleportDestinationId(),
                this.getText(),
                this.getParent_id(),
                new MapPosition(getMapPosition().getX(),
                                getMapPosition().getY(),
                                getMapPosition().getZ()));

        ts.setRequirementList(new ArrayList<QuestionCategory>(this.getRequirementList()));

        return ts;
    }


    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof TeleportSource)) return true;

        TeleportSource t2 = (TeleportSource)object;

        return super.differsFrom(t2) ||
               this.getTeleportDestinationId() != t2.getTeleportDestinationId() ||
               !this.getText().equals(t2.getText()) ||
               this.getMapPosition().differsFrom(t2.getMapPosition()) ||
               !this.getRequirementList().equals(t2.getRequirementList());

    }

    /* ----- Methods from interface MapItem ----- */


    public void moveTo(int x, int y) {
        this.mapPosition.changePosition(x, y);
    }


    public void moveTo(int x, int y, int z) {
        this.mapPosition.changePosition(x, y, z);
    }


    public MapPosition getMapPosition() {
        return mapPosition;
    }


    public void setMapPosition(MapPosition mapPosition) {
        this.mapPosition = mapPosition;
    }


    public LevelModel getLevelModel() {
        return levelModel;
    }


    public void setLevelModel(LevelModel lm) {
        try {
            this.levelModel.removeElement(this);
        } catch (NullPointerException e) {
        }
        this.levelModel = lm;
        this.levelModel.addElement(this);
        setParent_id(levelModel.getId());
    }

    public String getStatusName() {
        return "Teleport Source";
    }
}
