package sys.datatypes.interfaces;

import java.util.HashMap;

/**
 * A class implementing the AoCObject interface is
 * a direct representation of a table in the AoC database.
 * This makes the database operations easier to create and
 * understand. This interface has no methods as it is used
 * only to classify database objects.
 *
 * @author Eivind Sorteberg - Date: 19.jun.2006
 */
public interface AoCObject {

    /**
     * @return The name of the corresponding table in the DB
     */
    public String getDBname();

    /**
     * @return The id of the object - should always be the same as in the DB
     */
    public int getId();

    /**
     * Sets the id of the object.
     *
     * @param Id The id of the object
     */
    public void setId(int Id);

    /**
     * The names of the fields used as id fields in the database. Most tables only have one
     * field here ("ID")
     * @return The id fields of the object
     */
    public String[] getIdFields();

    /**
     * The values of the id fields, with varchar's as Strings and numbers as integers.
     *
     * @return The id values of the object
     */
    public Object[] getIdValues();

    /**
     * The names of the fields that are not id fields in the database.
     *
     * @return The non-key field names of the object
     */
    public String[] getFieldNames();

    /**
     * The values of the fields that are not id fields.
     * @return The values of the fields that are not id fields.
     */
    public Object[] getFieldValues();

    /**
     * Makes an object with exactly the same attributes and values as the original.
     *
     * @return A copy of the original object
     */
    public AoCObject makeClone();

    /**
     * Checks whether two AoCObjects have the same values, or if some fields differ.
     *
     * @param object The object to compare with
     * @return Whether the two objects are different
     */
    public boolean differsFrom(AoCObject object);

    /**
     * @return A string describing the object
     */
    public String getInfo();

    /**
     * Returns a html formatted string where the in parameter is a
     * binary representation of which attributes should be colored in another
     * color.
     *
     * @param coloredAttributes Binary representation of which attributes should be colored differently
     * @return A string describing the object
     */
    public String getInfo(int coloredAttributes);

    /**
     * Sets all the object's attributes, with each attribute having
     * its name as key in the attributes map
     *
     * @param attributes The attributes to set
     */
    public void setAllAttributes(HashMap<String, Object> attributes);

    /**
     * Returns a HashMap containing all the object's attributes, with its DB field name as
     * key. This list does not correspond to the sequence in the database.
     *
     * @return The object's attributes and field names.
     */
    public HashMap<String, Object> getAllAttributes();

    /**
     * Returns an array containing all the object's attrributes, and its DB field names.
     * This array corresponds to the fields' sequence in the database.
     * The first row in the array contains the field names, while the second contains the
     * corresponding values.
     *
     * @return The object's attributes and field names.
     */
    public Object[][] getAttributeArray();
}
