package sys.models;

import gui.AdminPanel;
import sys.DefaultValues;
import sys.datatypes.Level;
import sys.datatypes.User;
import sys.datatypes.interfaces.AoCObject;

import javax.swing.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;

/**
 * Model for the AdminPanel
 *
 * @author Tarje Killingberg - 12.jul.2006
 */
public class AdminModel {

    //The names of the settings in the Settings table
    private static String EXAM_MODE = "ExamMode";
    private static String DO_NOT_SHOW_QUESTION_GROUP = "DoNotShowQuestionGroup";
    private static String DO_NOT_SHOW_TEXT_GROUP = "DoNotShowTextGroup";
    private static String TEXT_ROOT = "TextGroupRoot";
    private static String QUESTION_ROOT = "QuestionGroupRoot";
    private static String STYLE_SHEET = "StyleSheet";
    private static String LAST_QUESTION_ID = "LastQuestionID";

    private ModelManager modelManager;
    private HashMap<Integer, User> originalUserHashMap;
    private HashMap<Integer, User> userHashMap;
    private ArrayList<User> changedUsers;
    private UserTableModel userTableModel;
    private HashMap<String, String> settingsHashMap;
    private AdminPanel adminPanel;
    private Integer[] levelIdArray;
    private ArrayList<Integer> allQuestionIDs;


    public AdminModel(ModelManager modelManager) {
        this.modelManager = modelManager;
        changedUsers = new ArrayList<User>();
    }


    public void createNewUser() {
        User user = new User();
        modelManager.fireObjectCreated(user);
        fireModelChanged();
        userTableModel.fireUserCreated(user);
        originalUserHashMap.put(user.getId(), (User)user.makeClone());
    }


    public void fireModelChanged() {
        if (userTableModel != null) {
            userTableModel.adminModelChanged();
        }
    }


    public void deleteUsers(User[] users) {
        for (User user : users) {
            originalUserHashMap.remove(user.getId());
            modelManager.deleteObject(user);
        }
        fireModelChanged();
    }


    /**
     * Called by the UserTableModel when a user has been
     * modified in the table
     *
     * @param user
     */
    public void userChanged(User user) {
        if (!changedUsers.contains(user)) {
            changedUsers.add(user);
        }
    }


    public void saveButtonPressed() {
        AoCObject currentObject = getModelManager().getCurrentObject();
        getModelManager().fireObjectParentUpdated(currentObject);
        this.originalUserHashMap = copyUserHashMap();
    }


    public void discardButtonPressed() {
        for (User changedUser : changedUsers) {
            User originalUser = originalUserHashMap.get(changedUser.getId());
            HashMap<String, Object> attrs = originalUser.getAllAttributes();
            changedUser.setAllAttributes(attrs);
        }

        flushChangedUsers();
        fireModelChanged();
    }


    /**
     * Called by the ModelManager when saving
     * changed objects to database was successfull,
     * or by method discardButtonPressed.
     */
    public void flushChangedUsers () {
        changedUsers = new ArrayList<User>();
    }


    public void fireSettingsChanged() {
        adminPanel.settingsChanged();
    }


    public ArrayList<User> fixLevelKeyConstraints(int levelID) {
        ArrayList<User> affectedUsers = new ArrayList<User>();

        for (Integer integer : userHashMap.keySet()) {
            User user = userHashMap.get(integer);
            if (user.getLevelID() == levelID) {
                user.setLevelID(DefaultValues.USER_LEVEL_ID);
                affectedUsers.add(user);
            }
        }
        return affectedUsers;
    }



    /* ----- Getters and setters ----- */


    /**
     * Called by the ModelManager when saving
     * changed objects to database. Returns the
     * changed users.
     *
     * @return the changed users
     */
    public ArrayList<User> getChangedUsers () {
        return changedUsers;
    }


    public HashMap<Integer, User> getUserHashMap() {
        return userHashMap;
    }


    public Integer[] getLevelIdArray() {
        return levelIdArray;
    }


    public void setLevelHashMap(HashMap<Integer, Level> levels) {
        levelIdArray = new Integer[levels.keySet().size()];
        levels.keySet().toArray(levelIdArray);
        Arrays.sort(levelIdArray);
        if (adminPanel != null) {
            adminPanel.levelIdsChanged();
        }
    }


    public void setUserHashMap(HashMap<Integer, User> userHashMap) {
        this.userHashMap = userHashMap;
        this.originalUserHashMap = copyUserHashMap();
        fireModelChanged();
    }


    public ModelManager getModelManager() {
        return modelManager;
    }


    public void setUserTableModel(UserTableModel userTableModel) {
        this.userTableModel = userTableModel;
    }

    public void setAdminPanel(AdminPanel adminPanel) {
        this.adminPanel = adminPanel;
    }

    public void setSettings(HashMap<String, String> settingsHashMap) {
        this.settingsHashMap = settingsHashMap;

        int doNotShowQuestionGroupInt;
        int doNotShowTextGroupInt;
        int textRootInt;
        int questionRootInt;
        int lastQuestionInt;

        String doNotShowQuestionGroupString = settingsHashMap.get(DO_NOT_SHOW_QUESTION_GROUP);
        String doNotShowTextGroupString = settingsHashMap.get(DO_NOT_SHOW_TEXT_GROUP);
        String textRootString = settingsHashMap.get(TEXT_ROOT);
        String questionRootString = settingsHashMap.get(QUESTION_ROOT);
        String lastQuestionString = settingsHashMap.get(LAST_QUESTION_ID);
        String examMode = settingsHashMap.get(EXAM_MODE);
        String styleSheetFile = settingsHashMap.get(STYLE_SHEET);


        try {
            doNotShowQuestionGroupInt = Integer.parseInt(doNotShowQuestionGroupString);
            DefaultValues.QUESTION_GROUP = doNotShowQuestionGroupInt;
        } catch (NumberFormatException nfe) {
            JOptionPane.showMessageDialog(adminPanel,
                                          "Value for DoNotShowQuestionGroup in Settings-table\n" +
                                          "is not an integer or does not exist.\n" +
                                          "This will cause AoC Admin to behave unpredictable.\n" +
                                          "It is highly recommended that this is corrected before\n" +
                                          "continuing to use the application.",
                                          "Value missing",
                                          JOptionPane.ERROR_MESSAGE);
        }

        try {
            doNotShowTextGroupInt = Integer.parseInt(doNotShowTextGroupString);
            DefaultValues.TEXT_GROUP = doNotShowTextGroupInt;
        } catch (NumberFormatException nfe) {
            JOptionPane.showMessageDialog(adminPanel,
                                          "Value for DoNotShowTextGroup in Settings-table\n" +
                                          "is not an integer or does not exist.\n" +
                                          "This will cause AoC Admin to behave unpredictable.\n" +
                                          "It is highly recommended that this is corrected before\n" +
                                          "continuing to use the application.",
                                          "Value missing",
                                          JOptionPane.ERROR_MESSAGE);
        }
        try {
            textRootInt = Integer.parseInt(textRootString);
            DefaultValues.TEXT_ROOT = textRootInt;
        } catch (NumberFormatException nfe) {
            JOptionPane.showMessageDialog(adminPanel,
                                          "Value for " + TEXT_ROOT + " in Settings-table\n" +
                                          "is not an integer or does not exist.\n" +
                                          "This will cause AoC Admin to behave unpredictable.\n" +
                                          "It is highly recommended that this is corrected before\n" +
                                          "continuing to use the application.",
                                          "Value missing",
                                          JOptionPane.ERROR_MESSAGE);
        }
        try {
            questionRootInt = Integer.parseInt(questionRootString);
            DefaultValues.QUESTION_ROOT = questionRootInt;
        } catch (NumberFormatException nfe) {
            JOptionPane.showMessageDialog(adminPanel,
                                          "Value for " + QUESTION_ROOT + " in Settings-table\n" +
                                          "is not an integer or does not exist.\n" +
                                          "This will cause AoC Admin to behave unpredictable.\n" +
                                          "It is highly recommended that this is corrected before\n" +
                                          "continuing to use the application.",
                                          "Value missing",
                                          JOptionPane.ERROR_MESSAGE);
        }
        try {
            lastQuestionInt = Integer.parseInt(lastQuestionString);
            DefaultValues.LAST_QUESTION_ID = lastQuestionInt;
        } catch (NumberFormatException nfe) {
            JOptionPane.showMessageDialog(adminPanel,
                                          "Value for " + LAST_QUESTION_ID + " in Settings-table\n" +
                                          "is not an integer or does not exist.\n" +
                                          "This will cause AoC Admin to behave unpredictable.\n" +
                                          "It is highly recommended that this is corrected before\n" +
                                          "continuing to use the application.",
                                          "Value missing",
                                          JOptionPane.ERROR_MESSAGE);
        }

        if (styleSheetFile != null) {
            DefaultValues.STYLE_CSS_FILE = styleSheetFile;
        }
        else {
            JOptionPane.showMessageDialog(adminPanel,
                                          "A filename for the StyleSheet was not found in " +
                                          "the Settings-table.\n" +
                                          "This means that text in this application will be\n" +
                                          "displayed in standard html style.",
                                          "Value missing",
                                          JOptionPane.ERROR_MESSAGE);
        }
        if (examMode == null) {
            JOptionPane.showMessageDialog(adminPanel,
                                          "An entry with name \"ExamMode\" was not found in\n" +
                                          "the Settings-table, or the value is NULL.\n" +
                                          "The application will not start until that entry is\n" +
                                          "present, and has a decent value.",
                                          "Value missing",
                                          JOptionPane.ERROR_MESSAGE);
        }


    }


    public boolean isExamMode() {
        return settingsHashMap.get(EXAM_MODE).equalsIgnoreCase("ON");
    }

    public void setExamMode(boolean value) {
        modelManager.updateSetting(EXAM_MODE, (value ? "on" : "off"));
    }

    public String getStyleSheet() {
        return settingsHashMap.get(STYLE_SHEET);
    }

    public void setStyleSheet(String styleSheet) {
        modelManager.updateSetting(STYLE_SHEET, styleSheet);
    }

    public Integer getLastQuestionID() {
        return Integer.parseInt(settingsHashMap.get(LAST_QUESTION_ID));
    }

    public void setLastQuestionID(Integer lastQuestionID) {
        modelManager.updateSetting(LAST_QUESTION_ID, lastQuestionID.toString());
    }

    public ArrayList<Integer> getAllQuestionIDs() {
        return allQuestionIDs;
    }

    public void setAllQuestionIDs(ArrayList<Integer> questionIDs) {
        allQuestionIDs = questionIDs;
        Collections.sort(allQuestionIDs);
    }


    /* ----- Other methods ----- */


    private HashMap<Integer, User> copyUserHashMap() {
        HashMap<Integer, User> userHashMapCopy = new HashMap<Integer, User>();

        for (Integer integer : userHashMap.keySet()) {
            User user = userHashMap.get(integer);
            User userClone = (User) user.makeClone();
            userHashMapCopy.put(userClone.getId(), userClone);
        }

        return userHashMapCopy;
    }


}
