package sys.models;

import sys.datatypes.interfaces.AoCObject;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * Stores all changes object so that these can be written to the
 * DB when needed.
 *
 * @author Eivind Sorteberg - 29.jun.2006
 */
public class ChangeManager {
    private HashMap<AoCObject, ChangedObjectWrapper> changedObjects;

    public ChangeManager() {
        this.changedObjects = new HashMap<AoCObject, ChangedObjectWrapper>();
    }

    /**
     * Adds an object's attributes to a copy of the object so that
     * the object's original values are preserved.
     *
     * @param object The object to add
     */
    public void addObject(AoCObject object) {
        if (object != null) {
            ChangedObjectWrapper wrapper = new ChangedObjectWrapper(object.makeClone());
            changedObjects.put(object, wrapper);
        }
    }

    /**
     * Updates the corresponding object in the changelist if it has
     * been changed.
     * If it has not, both the original and the "changed" object are
     * removed.
     *
     * @param object The changed object.
     * @return Whether the object was changed or not
     */
    public boolean updateObject(AoCObject object) {
        ChangedObjectWrapper wrapper = changedObjects.get(object);

        if (wrapper != null) {
            AoCObject original = wrapper.getOldObject();
            if (original.differsFrom(object)) {
                wrapper.objectChanged(object);
                return true;
            }
            else {
                changedObjects.remove(object);
                return false;
            }
        }
        return false;
    }

    /**
     * Returns all the objects that have been changed.
     *
     * @return The changed objects.
     */
    public ArrayList<ChangedObjectWrapper> getChangedObjects() {
        ArrayList<ChangedObjectWrapper> objects = new ArrayList<ChangedObjectWrapper>();

        for (AoCObject aoCObject : changedObjects.keySet()) {
            objects.add(changedObjects.get(aoCObject));
        }

        return objects;
    }

    /**
     * Empties the change list.
     * Used when all the changed objects have been written to
     * DB.
     */
    public void flush() {
        this.changedObjects.clear();
    }

    /**
     * @param object The changed object
     * @return The corresponding original
     */
    public AoCObject getOriginalObject(AoCObject object) {
        return changedObjects.get(object).getOldObject();
    }

    /**
     * @param object The object (original and changed) to remove.
     */
    public void removeObject(AoCObject object) {
        changedObjects.remove(object);
    }
}
