package sys.models;

import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.question.MultiChoiceAlternative;
import sys.datatypes.question.MultiChoiceQuestion;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * Model representing the table of alternatives for a
 * <code>MultiChoiceQuestion</code>.
 *
 * @see gui.editors.question.question.MultiChoicePanel
 * @see MultiChoiceQuestion
 * @author Eivind Sorteberg - 03.jul.2006
 */
public class MCAlternativeModel implements TableModel {
    private static final int TEXT_COLUMN = 0;
    private static final int CORRECT_COLUMN = 1;

    public ArrayList<MultiChoiceAlternative> alternatives;
    public ArrayList<TableModelListener> listeners;
    private MultiChoiceQuestion question;

    public MCAlternativeModel(ArrayList<MultiChoiceAlternative> alternatives, MultiChoiceQuestion question) {
        this.alternatives = alternatives;
        this.listeners = new ArrayList<TableModelListener>();
        this.question = question;
    }

    public MCAlternativeModel(MultiChoiceQuestion question) {
        this(new ArrayList<MultiChoiceAlternative>(), question);
    }

    public boolean addAlternative(MultiChoiceAlternative alternative) {
        return this.alternatives.add(alternative);
    }

    public boolean removeAlternative(MultiChoiceAlternative alternative) {
        return this.alternatives.remove(alternative);
    }

    public ArrayList<MultiChoiceAlternative> getAlternatives() {
        return alternatives;
    }

    ////////////////////////////////////////////////
    // Methods inherited from TableModel          //
    ////////////////////////////////////////////////
    public int getRowCount() {
        return alternatives.size();
    }

    public int getColumnCount() {
        return 2;
    }

    public String getColumnName(int columnIndex) {
        switch (columnIndex) {
            case TEXT_COLUMN:
                return "Answer";
            case CORRECT_COLUMN:
                return "Correct";
            default:
                return "Illegal columnIndex";
        }
    }

    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
            case TEXT_COLUMN:
                return String.class;
            case CORRECT_COLUMN:
                return Boolean.class;
            default:
                return Object.class;
        }
    }

    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return true;
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        switch (columnIndex) {
            case TEXT_COLUMN:
                return alternatives.get(rowIndex).getText();
            case CORRECT_COLUMN:
                return alternatives.get(rowIndex).getId() == this.question.getAnswerID();
            default:
                return null;
        }
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        switch (columnIndex) {
            case TEXT_COLUMN:
                setText(rowIndex, (String)aValue);
                break;
            case CORRECT_COLUMN:
                setCorrectAnswer(alternatives.get(rowIndex));
                break;
        }
        fireModelChanged();
    }

    private void setText(int rowIndex, String value) {
        MultiChoiceAlternative alternative = alternatives.get(rowIndex);
        alternative.setText(value);
    }

    private void setCorrectAnswer(MultiChoiceAlternative alternative) {
        this.question.setAnswerID(alternative.getId());
    }

    public void addTableModelListener(TableModelListener l) {
        this.listeners.add(l);
    }

    public void removeTableModelListener(TableModelListener l) {
        this.listeners.remove(l);
    }

    public void fireModelChanged() {
        for (TableModelListener listener : listeners) {
            listener.tableChanged(new TableModelEvent(this));
        }
    }

    public MultiChoiceAlternative newAlternative() {
        int newId = 0;

        for (MultiChoiceAlternative alternative : alternatives) {
            int id = alternative.getId();

            if (id != newId) break;
            newId++;
        }

        MultiChoiceAlternative newAlternative = new MultiChoiceAlternative(newId, this.question.getId(), "", "");
        this.alternatives.add(newAlternative);
        fireModelChanged();

        return newAlternative;
    }

    public MultiChoiceAlternative[] deleteAlternative(int[] selectedRows) {
        MultiChoiceAlternative[] removed = new MultiChoiceAlternative[selectedRows.length];

        for (int i = 0 ; i < selectedRows.length; i++) {
            int row = selectedRows[i];
            if (alternatives.get(row).getId() == question.getAnswerID()) {
                question.setAnswerID(-1);
            }
            removed[i] = alternatives.remove(row);
        }
        fireModelChanged();
        return removed;
    }

    public boolean differsFrom(MCAlternativeModel m2) {
        if (this.getAlternatives().size() != m2.getAlternatives().size()) return true;

        for (int i = 0; i < this.alternatives.size(); i++) {
            if (this.getAlternatives().get(i).differsFrom(m2.getAlternatives().get(i))) return true;
        }
        return false;
    }

    public MCAlternativeModel makeClone() {
        MCAlternativeModel clone = new MCAlternativeModel(this.question);

        for (MultiChoiceAlternative alternative : alternatives) {
            clone.addAlternative((MultiChoiceAlternative)alternative.makeClone());
        }

        return clone;
    }

    public HashMap<Integer, AoCObject> sortAlternatives() {

        HashMap<Integer, AoCObject> changed = new HashMap<Integer, AoCObject>();

        for (int i = 0; i < alternatives.size(); i++) {
            MultiChoiceAlternative alternative = alternatives.get(i);
            int oldId = alternative.getId();

            if (oldId != i) {
                alternative.setId(i);

                if (question.getAnswerID() == oldId) {
                    setCorrectAnswer(alternative);
                }

                changed.put(oldId, alternative);
            }
        }

        return changed;
    }
}
