package sys.models;

import gui.DBProgressMonitor;
import gui.TabPanel;
import gui.editors.common.util.ConflictViewGenerator;
import gui.tabs.AbstractTab;
import gui.tabs.LevelTab;
import gui.tabs.QuestionTab;
import gui.tabs.TextTab;
import sys.DefaultValues;
import sys.Parser.Parser;
import sys.datatypes.*;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.datatypes.question.*;
import sys.models.treemodels.AbstractTreeModel;
import sys.models.treemodels.LevelTreeModel;
import sys.models.treemodels.QuestionTreeModel;
import sys.models.treemodels.TextTreeModel;

import javax.swing.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Manages the sys.models.
 * Contains all relevant information in the database, where each row is an object in a list.
 * Each object is one of the defined in datatypes-package.
 *
 * @author Tarje Killingberg - 19.jun.2006
 */
public class ModelManager {
    private TabPanel tabPanel;
    private LevelTreeModel superLevelModel;
    private QuestionTreeModel questionModel;
    private TextTreeModel textModel;
    private AdminModel adminModel;
    private ChangeManager changeManager = new ChangeManager();
    private AoCObject currentObject;


    private HashMap<Integer, TextGroup> textgroups;
    private HashMap<Integer, Text>      texts;
    private HashMap<Integer, QuestionGroup> questionGroups;
    private HashMap<Integer, QuestionCategory> questionCategories;
    private HashMap<Integer, Question> questions;
    private ArrayList<MultiChoiceAlternative> alternatives;
    private HashMap<Integer, Level> levels;
    private HashMap<Integer, TeleportSource> teleportSources;
    private HashMap<Integer, TeleportDestination> teleportDestinations;
    private HashMap<Integer, Message> messages;
    private HashMap<Integer, ArrayList<Integer>> teleportRequirements;
    private HashMap<Integer, ArrayList<Integer>> requirementTeleports;
    private ArrayList<String> maps;
    private HashMap<Integer, User> users;
    private HashMap<String, String> settings;


    public ModelManager(TabPanel tabPanel) {
        this.tabPanel = tabPanel;
    }

    /**
     * Builds the three tree models with information from the
     * database.
     */
    public void buildTreeModels() {
        DBProgressMonitor.reset();
        this.fetchObjects();
        DBProgressMonitor.update("Building models", 140);
        this.buildAdminModel();
        this.buildTextModel();
        this.buildQuestionModel();
        this.buildSuperLevelModel();

        DBProgressMonitor.complete();
    }

    /**
     * Reads the information from the database, and puts
     * the new objects in corresponding hashmaps.
     */
    public void fetchObjects() {
        DBProgressMonitor.update("Parsing settings", 0);
        settings = Parser.getSettings();
        DBProgressMonitor.update("Parsing text groups", 10);
        textgroups = Parser.getTextGroups();
        DBProgressMonitor.update("Parsing texts", 20);
        texts = Parser.getTexts();
        DBProgressMonitor.update("Parsing question groups", 30);
        questionGroups = Parser.getQuestionGroups();
        DBProgressMonitor.update("Parsing question categories", 40);
        questionCategories = Parser.getQuestionCategories();
        DBProgressMonitor.update("Parsing questions", 50);
        questions = Parser.getQuestions();
        DBProgressMonitor.update("Parsing multichoicequestion alternatives", 60);
        alternatives = Parser.getMultiChoiceAlternatives();
        DBProgressMonitor.update("Parsing levels", 70);
        levels = Parser.getLevels();
        DBProgressMonitor.update("Parsing teleport destinations", 80);
        teleportDestinations = Parser.getTeleportDestinations();
        DBProgressMonitor.update("Parsing teleport sources", 90);
        teleportSources = Parser.getTeleportSources();
        DBProgressMonitor.update("Parsing messages", 100);
        messages = Parser.getMessages();
        DBProgressMonitor.update("Parsing teleport requirements", 110);
        Object[] o = Parser.getTeleportRequirements();
        teleportRequirements = (HashMap<Integer, ArrayList<Integer>>)o[0];
        requirementTeleports = (HashMap<Integer, ArrayList<Integer>>)o[1];
        DBProgressMonitor.update("Parsing maps", 120);
        maps = Parser.getAllMaps();
        DBProgressMonitor.update("Parsing users", 130);
        users = Parser.getUsers();
    }

    /**
     * Builds the <code>LevelTreeModel</code> from the information
     * read from the database.
     *
     * @see LevelTreeModel
     */
    public void buildSuperLevelModel() {
        /* Defining a levelModel-root */
        LevelModel rootLevelModel = new LevelModel(0, -1000, new Level(0, -1001));

        superLevelModel = new LevelTreeModel(rootLevelModel, this);

        /* Building levelModel-tree */
        for (Integer integer : levels.keySet()) {
            Level level = levels.get(integer);
            LevelModel levelModel = new LevelModel(level.getId(), 0, level);
            levelModel.setParent(rootLevelModel);
        }

        /* Adding texts to the LevelModels */
        for (Integer integer : texts.keySet()) {
            Text text = texts.get(integer);
            Level l = levels.get(text.getLevel());
            LevelModel lm = rootLevelModel.getLevelModel(l.getId());
            text.setLevelModel(lm);
        }

        /* Adding QuestionCategories to the LevelModels */
        for (Integer integer : questionCategories.keySet()) {
            QuestionCategory questionCategory = questionCategories.get(integer);
            Level l = levels.get(questionCategory.getLevel());
            LevelModel lm = rootLevelModel.getLevelModel(l.getId());
            questionCategory.setLevelModel(lm);
        }

        /* Adding TeleportDestinations to the LevelModels */
        for (Integer integer : teleportDestinations.keySet()) {
            TeleportDestination teleportDestination = teleportDestinations.get(integer);
            Level l = levels.get(teleportDestination.getParent_id());
            LevelModel lm = rootLevelModel.getLevelModel(l.getId());
            teleportDestination.setLevelModel(lm);
        }

        /* Adding TeleportSources to the LevelModels */
        ArrayList<QuestionCategory> requirementObjects;
        ArrayList<Integer> requirementKeys;

        for (Integer integer : teleportSources.keySet()) {
            TeleportSource teleportSource = teleportSources.get(integer);
            Level l = levels.get(teleportSource.getParent_id());
            LevelModel lm = rootLevelModel.getLevelModel(l.getId());
            teleportSource.setLevelModel(lm);

            /* Adding requirements to the TeleportSources */
            requirementObjects = new ArrayList<QuestionCategory>();
            requirementKeys = teleportRequirements.get(integer);
            if (requirementKeys != null) {
                for (Integer requirementKey : requirementKeys) {
                    QuestionCategory qc = questionCategories.get(requirementKey);
                    requirementObjects.add(qc);
                }
            }
            teleportSource.setRequirementList(requirementObjects);
        }

        /* Adding Messages to the LevelModels */
        for (Integer integer : messages.keySet()) {
            Message message = messages.get(integer);
            Level l = levels.get(message.getParent_id());
            LevelModel lm = rootLevelModel.getLevelModel(l.getId());
            message.setLevelModel(lm);
        }

        superLevelModel.setMapList(maps);
        superLevelModel.sortTree();
    }

    /**
     * Builds the <code>TextTreeModel</code> from the information
     * read from the database.
     *
     * @see TextTreeModel
     */
    public void buildTextModel() {

        for (Integer integer : textgroups.keySet()) {
            TextGroup group = textgroups.get(integer);

            if (group.getId() != DefaultValues.TEXT_ROOT) {
                TextGroup parent = textgroups.get(group.getParent_id());
                group.setParent(parent);
            }
            else {
                textModel = new TextTreeModel(group, this);
                group.setParent_id(-1);
            }
        }

        for (Integer integer : texts.keySet()) {
            Text fact = texts.get(integer);

            int parent_id = fact.getParent_id();
            TextGroup parent = textgroups.get(parent_id);
            fact.setParent(parent);
        }

        textModel.sortTree();
    }

    /**
     * Builds the <code>QuestionTreeModel</code> from the information
     * read from the database.
     *
     * @see QuestionTreeModel
     */
    public void buildQuestionModel() {
        //Generating the questiongroups
        for (int i : questionGroups.keySet()) {
            QuestionGroup group = questionGroups.get(i);

            if (group.getId() != DefaultValues.QUESTION_ROOT) {
                Group parent = questionGroups.get(group.getParent_id());
                group.setParent(parent);
            }
            else {
                questionModel = new QuestionTreeModel(group, this);
                group.setParent_id(-1);
            }
        }

        //Generating the questioncategories
        for (int i : questionCategories.keySet()) {
            QuestionCategory category = questionCategories.get(i);

            int parent_id = category.getParent_id();

            Group parent = questionGroups.get(parent_id);
            category.setParent(parent);

            if (i == DefaultValues.QUESTION_CATEGORY) QuestionCategory.setDefaultCategory(category);
        }

        //Generating the questions
        for (int i : questions.keySet()) {
            Question question = questions.get(i);

            int category = question.getParent_id();

            Group parent = questionCategories.get(category);
            question.setParent(parent);
        }

        //Generating the multiple choice alternatives
        for (MultiChoiceAlternative alternative : alternatives) {
            int question_id = alternative.getQuestion_id();

            Question question = questions.get(question_id);

            if (question instanceof MultiChoiceQuestion) {
                MultiChoiceQuestion m_question = (MultiChoiceQuestion)question;

                m_question.addAlternative(alternative);
            }
            else {
                System.err.println("ERROR! Expected MultiChoiceQuestion; found: " + question.getClass().getName());
                System.err.println("QuestionID = " + question_id + "; AlternativeID = " + alternative.getId());
            }
        }

        questionModel.sortTree();
    }

    /**
     * Builds the <code>AdminModel</code> using the information
     * from the database.
     *
     * @see AdminModel
     */
    public void buildAdminModel() {
        adminModel = new AdminModel(this);
        adminModel.setUserHashMap(users);
        adminModel.setSettings(settings);
        adminModel.setLevelHashMap(levels);
        adminModel.setAllQuestionIDs(new ArrayList<Integer>(questions.keySet()));

        /* Calculating user points */
        HashMap<Integer, ArrayList<Integer>> userQuestions = Parser.getUserQuestions();
        for (Integer integer : userQuestions.keySet()) {
            ArrayList<Integer> questionList = userQuestions.get(integer);
            int points = 0;
            for (Integer questionID : questionList) {
                points += questions.get(questionID).getValue();
            }
            users.get(integer).setPoints(points);
        }
    }


    /**
     * Refreshes the information in this class from the database.
     */
    public void reload() {

    }

    public AbstractTreeModel getTextModel() {
        return this.textModel;
    }

    public AbstractTreeModel getQuestionModel() {
        return this.questionModel;
    }

    public AbstractTreeModel getSuperLevelModel() {
        return superLevelModel;
    }

    public AdminModel getAdminModel() {
        return adminModel;
    }

    /**
     * @return The object currently selected
     */
    public AoCObject getCurrentObject() {
        return currentObject;
    }

    public TabPanel getTabPanel() {
        return tabPanel;
    }

    public void updateSetting(String name, String value) {
        Parser.updateSetting(name, value);
        settings.put(name, value);
        adminModel.fireSettingsChanged();
    }

    /**
     * Called whenever a new object is created.
     * Writes the new object to the database, and notifies the
     * neccessary models.
     *
     * @param object The newly created object
     */
    public void fireObjectCreated(AoCObject object) {

        //Checks if the name of the object should be numbered (x)
        if (object instanceof Group || object instanceof Text || object instanceof User) {
            String nameField;
            int name_no;

            if (object instanceof User) { nameField = "Username"; }
            else { nameField = "Name"; }

            name_no = Parser.getCount(object, nameField);

            if (name_no > 0) {
                if (object instanceof Group) {
                    ((Group)object).setName(((Group)object).getName() + " (" + name_no + ")");
                }
                else if (object instanceof Text) {
                    ((Text)object).setName(((Text)object).getName() + " (" + name_no + ")");
                }
                else {
                    ((User)object).setUsername(((User)object).getUsername() + " (" + name_no + ")");
                }
            }
        }

        //Finds out what the object's id will be
        object.setId(Parser.createNewObject(object, true));

        //Updates the question model if needed..
        if (object instanceof Question || object instanceof QuestionCategory || object instanceof QuestionGroup) {
            GroupElement groupElement = (GroupElement)object;

            if (object instanceof QuestionGroup) {
                groupElement.setParent(questionGroups.get(groupElement.getParent_id()));
                questionGroups.put(object.getId(), (QuestionGroup)object);
            }
            else if (object instanceof Question) {
                groupElement.setParent(questionCategories.get(groupElement.getParent_id()));
                questions.put(object.getId(), (Question)object);
            }
            else {
                groupElement.setParent(questionGroups.get(groupElement.getParent_id()));
                LevelModel lm = (LevelModel)superLevelModel.getRoot();
                int levelId = ((QuestionCategory)object).getLevel();
                ((QuestionCategory)object).setLevelModel(lm.getLevelModel(levelId));
                questionCategories.put(object.getId(), (QuestionCategory)object);
            }
            questionModel.fireNodeInserted(groupElement);
        }

        //..and the text model if needed
        if (object instanceof Text || object instanceof TextGroup) {
            GroupElement groupElement = (GroupElement)object;
            groupElement.setParent(textgroups.get(groupElement.getParent_id()));

            if (object instanceof Text) {
                Text text = (Text)object;
                int levelID = text.getLevel();
                LevelModel lm = ((LevelModel)superLevelModel.getRoot()).getLevelModel(levelID);
                text.setLevelModel(lm);
                texts.put(text.getId(), text);
            }
            else {
                textgroups.put(object.getId(), (TextGroup)object);
            }
            textModel.fireNodeInserted(groupElement);
        }

        //..and finally the level model
        if (object instanceof MapItem) {

            LevelModel rootLevelModel = (LevelModel)superLevelModel.getRoot();

            if (object instanceof Level) {
                Level level = (Level)object;
                LevelModel lm = new LevelModel(level.getId(), 0, level);
                lm.setParent(rootLevelModel);
                levels.put(level.getId(), level);
                object = lm;
                HashMap<Integer, LevelModel> map = rootLevelModel.getElements();
                map.put(object.getId(), (LevelModel)object);
                adminModel.setLevelHashMap(levels);
            }
            else if (object instanceof Message) {
                Message message = (Message)object;
                int levelID = levels.get(message.getParent_id()).getId();
                LevelModel lm = rootLevelModel.getLevelModel(levelID);
                message.setLevelModel(lm);
                messages.put(message.getId(), message);
            }
            else if (object instanceof TeleportSource) {
                TeleportSource teleportSource = (TeleportSource)object;
                int levelID = levels.get(teleportSource.getParent_id()).getId();
                LevelModel lm = rootLevelModel.getLevelModel(levelID);
                teleportSource.setLevelModel(lm);
                teleportSources.put(teleportSource.getId(), teleportSource);
            }
            else if (object instanceof TeleportDestination) {
                TeleportDestination teleportDestination = (TeleportDestination)object;
                int levelID = levels.get(teleportDestination.getParent_id()).getId();
                LevelModel lm = rootLevelModel.getLevelModel(levelID);
                teleportDestination.setLevelModel(lm);
                teleportDestinations.put(teleportDestination.getId(), teleportDestination);
            }
            else if (object instanceof Text) {
            }

            if (object instanceof GroupElement) superLevelModel.fireNodeInserted((GroupElement)object); //Always true
        }

        if (object instanceof User) {
            User user = (User)object;
            users.put(user.getId(), user);
        }
    }

    /**
     * Special version of the fireObjectCreated method for MultiChoiceAlternatives (which are not
     * <code>GroupElement<code> objects, and therefor have special attributes.
     *
     * @param newAlt The Alternative to add.
     */
    public void fireObjectCreated(MultiChoiceAlternative newAlt) {
        Parser.createNewObject(newAlt, false);
    }

    /**
     * Called whenever an object is deleted.
     * Calls <code>deleteObject()</code> to
     * remove the object from the database, and
     * updates the models.
     */
    public void fireObjectDeleted() {
        if (!deleteConfirmed(currentObject)) {
            return;
        }

        if (this.currentObject instanceof QuestionCategory ||
                this.currentObject instanceof QuestionGroup ||
                this.currentObject instanceof Question ||
                this.currentObject instanceof Message ||
                this.currentObject instanceof TeleportDestination ||
                this.currentObject instanceof TeleportSource ||
                this.currentObject instanceof Text ||
                this.currentObject instanceof TextGroup ||
                this.currentObject instanceof LevelModel) {

            changeManager.removeObject(currentObject);
            deleteObject(currentObject);

            AoCObject temp = currentObject;

            AbstractTab currentTab = getTabPanel().getActiveTab();

            if (currentTab instanceof QuestionTab) {
                if (temp instanceof Question || temp instanceof QuestionCategory || temp instanceof QuestionGroup) {

                    if (temp instanceof QuestionCategory) {
                        superLevelModel.fireNodeMoved((GroupElement)temp, true);
                        superLevelModelChanged();
                    }
                    questionModel.fireNodeMoved((GroupElement)temp, true);
                }
            }
            else if (currentTab instanceof LevelTab) {
                if (temp instanceof MapItem || temp instanceof LevelModel) {
                    if (temp instanceof QuestionCategory) {
                        questionModel.fireNodeMoved((GroupElement)temp, true);
                    }
                    else if (temp instanceof Text) {
                        textModel.fireNodeMoved((GroupElement)temp, true);
                    }

                    superLevelModel.fireNodeMoved((GroupElement)temp, true);
                    superLevelModelChanged();
                }
            }
            else if (currentTab instanceof TextTab) {
                if (temp instanceof Text || temp instanceof TextGroup) {
                    if (temp instanceof Text) {
                        superLevelModel.fireNodeMoved((GroupElement)temp, true);
                        superLevelModelChanged();
                    }

                    textModel.fireNodeMoved((GroupElement)temp, true);
                }
            }

        }
    }

    /**
     * Removes deleted objects from the database.
     *
     * @param object The deleted object
     */
    public void deleteObject(AoCObject object) {
        if (object instanceof GroupElement) {
            ((GroupElement)object).delete();
        }
        int id = object.getId();

        if (object instanceof QuestionGroup) {
            questionGroups.remove(id);
        }
        else if (object instanceof QuestionCategory) {
            questionCategories.remove(id);
            deleteForeignKeyObject(object);
        }
        else if (object instanceof Question) {
            questions.remove(id);
            deleteForeignKeyObject(object);
        }
        else if (object instanceof Message) {
            messages.remove(id);
        }
        else if (object instanceof TeleportDestination) {
            teleportDestinations.remove(id);
            deleteForeignKeyObject(object);
        }
        else if (object instanceof TeleportSource) {
            teleportSources.remove(id);
            deleteForeignKeyObject(object);
        }
        else if (object instanceof Text) {
            texts.remove(id);
            deleteForeignKeyObject(object);
        }
        else if (object instanceof TextGroup) {
            textgroups.remove(id);
        }
        else if (object instanceof LevelModel) {
            levels.remove(id);
            handleLevelDependencies((LevelModel)object);
        }
        else if (object instanceof User) {
            users.remove(id);
            deleteForeignKeyObject(object);
        }

        Parser.deleteObject(object);
    }


    /**
     * Deletes objects that have foreign key constraints, or, more precisely,
     * deletes the constraints so that the object can be deleted.
     */
    private void deleteForeignKeyObject(AoCObject object) {

        String table;
        String field;
        Object value;
        int id = object.getId();


        if (object instanceof QuestionCategory) {

            /* delete this questioncategory's requirement-rows in DB*/
            ArrayList<Integer> teleportSourceIDs = requirementTeleports.remove(id);
            if (teleportSourceIDs != null) {
                for (Integer tsid : teleportSourceIDs) {
                    ArrayList<Integer> questionCategoryIDs = teleportRequirements.get(tsid);
                    questionCategoryIDs.remove(Integer.valueOf(id));
               }
            }

            table = "TeleportRequirement";
            field = "QuestionCategoryID";
            value = id;

        }
        else if (object instanceof Question) {

            /* delete this questions entries in table UserQuestion */
            table = "UserQuestion";
            field = "QuestionID";
            value = id;
            Parser.deleteMultipleRows(table, field, value);

            if (object instanceof MultiChoiceQuestion) {
                table = "MultiChoiceAlternative";
                Parser.deleteMultipleRows(table, field, value);
            }

            /* delete this user's entries in table UserQuestion */
            Question q = (Question)object;
            table = q.getSpecialDBname();
            field = "ID";
        }
        else if (object instanceof TeleportDestination) {

            /* delete the sources that point to this destination */
            Iterator iter = teleportSources.keySet().iterator();
            ArrayList<TeleportSource> tsList = new ArrayList<TeleportSource>();
            while (iter.hasNext()) {
                TeleportSource ts = teleportSources.get(iter.next());
                if (ts.getTeleportDestinationId() == id) {
                    tsList.add(ts);
                }
            }

            for (TeleportSource teleSource : tsList) {
                teleportSources.remove(teleSource.getId());
                deleteForeignKeyObject(teleSource);
            }

            table = "TeleportSource";
            field = "TeleportDestID";
            value = id;

        }
        else if (object instanceof TeleportSource) {

            /* delete this source's requirement-rows in DB*/
            ArrayList<Integer> questionCategoryIDs = teleportRequirements.remove(id);
            if (questionCategoryIDs != null) {
                for (Integer qcid : questionCategoryIDs) {
                    ArrayList<Integer> teleportSourceIDs = requirementTeleports.get(qcid);
                    teleportSourceIDs.remove(Integer.valueOf(id));
                }
            }

            table = "TeleportRequirement";
            field = "TeleportSourceID";
            value = id;

        }
        else if (object instanceof Text) {

            table = "UserText";
            field = "TextID";
            value = id;

        }
        else if (object instanceof User) {

            /* delete this user's entries in table UserText */
            table = "UserText";
            field = "UserID";
            value = id;

            Parser.deleteMultipleRows(table, field, value);

            /* delete this user's entries in table UserQuestion */
            table = "UserQuestion";
            field = "UserID";
            value = id;

        }
        else {
            System.err.println("No content in method ModelManager.deleteForeignKeyObject()\n" +
                               "written for objects of type " + object.getClass().getName());
            return;
        }

        Parser.deleteMultipleRows(table, field, value);

    }

    /**
     * Handles dependencies (foreign key constraints) when
     * deleting a level.
     * Messages, teleportsources and teleportdestinations are deleted,
     * while questioncategories, texts and users are moved to level
     * <code>DefaultValues.QUESTION_CATEGORY_LEVEL_ID</code>,
     * <code>DefaultValues.TEXT_LEVEL_ID</code> and
     * <code>DefaultValues.USER_LEVEL_ID</code>
     *
     * @param levelModel
     */
    public void handleLevelDependencies(LevelModel levelModel) {
        Object[] affectedObjectLists;
        affectedObjectLists = levelModel.fixDependencies();
        ArrayList<AoCObject> toDelete = (ArrayList<AoCObject>)affectedObjectLists[0];
        ArrayList<AoCObject> toUpdate = (ArrayList<AoCObject>)affectedObjectLists[1];

        for (AoCObject object : toDelete) {
            deleteObject(object);
        }
        toUpdate.addAll(adminModel.fixLevelKeyConstraints(levelModel.getId()));
        Parser.updateObjects(toUpdate);

    }

    public void fireObjectDeleted(AoCObject object) {
        Parser.deleteObject(object);
    }

    /**
     * Shows a dialog window to make sure the user really wants to delete an object.
     *
     * @param object The object about to be deleted
     * @return Confirmation or cancellation
     */
    public boolean deleteConfirmed(AoCObject object) {
        boolean confirmed = true;
        String message = "";

        if (object instanceof QuestionGroup) {
            message = "Questioncategories associated with this\n" +
                      "questiongroup will be moved to questiongroup: " + ((QuestionGroup)object).getParent() + ".";
        }
        else if (object instanceof QuestionCategory) {
            message = "Questions associated with this questioncategory\n" +
                      "will be moved to questioncategory: " + questionCategories.get(DefaultValues.QUESTION_CATEGORY) + ".\n" +
                      "In addition, this category will be removed\n" +
                      "as requirement for teleportsources.";
        }
        else if (object instanceof Question) {
            message = "This question will be deleted.";
        }
        else if (object instanceof Message) {
            message = "This message will be deleted.";
        }
        else if (object instanceof TeleportDestination) {
            message = "Teleportsources pointing to this destination\n" +
                      "will also be removed.";
        }
        else if (object instanceof TeleportSource) {
            message = "The requirementmappings from this teleportsource\n" +
                      "to questioncategories will also be removed.";
        }
        else if (object instanceof Text) {
            message = "The mappings from this text to users will\n" +
                      "also be removed";
        }
        else if (object instanceof TextGroup) {
            message = "Texts associated with this textgroup will be\n" +
                      "moved to textgroup: " + ((TextGroup)object).getParent() + ".";
        }
        else if (object instanceof LevelModel) {
            message = "When deleting levels, associated mapitems are changed/removed.\n" +
                      "In particular: messages, teleportsources and\n" +
                      "teleportdestinations are deleted, and\n" +
                      "questioncategories and texts are moved to level " + DefaultValues.MAPITEM_LEVEL_ID;
        }

        if (!message.equals("")) {
            int choice = JOptionPane.showConfirmDialog(tabPanel,
                                                       message + "\n" +
                                                       "Proceed?",
                                                       "Confirm",
                                                       JOptionPane.YES_NO_OPTION,
                                                       JOptionPane.QUESTION_MESSAGE);

            if (choice == JOptionPane.NO_OPTION) {
                confirmed = false;
            }
        }

        return confirmed;
    }


    /**
     * Updates the currently selected object with the information written
     * in its textfields, and selects the new object
     *
     * @param object The new object to be selected
     */
    public void fireObjectSelected(AoCObject object) {
        if (object.equals(currentObject)) return;   //To prevent an object being removed from the list because it is selected twice in a row

        AoCObject temp = currentObject;
        this.currentObject = object;

        this.tabPanel.selectionChanged(temp, object);

        if (temp != null) this.changeManager.updateObject(temp);

        this.saveChangedObjectsToDatabase();
        this.changeManager.addObject(object);

        if (object instanceof MapItem) {
            showTreePath((GroupElement)object);
        }
    }

    /**
     * Selects an object in its corresponding tree.
     *
     * @param object The object to select
     */
    private void showTreePath(GroupElement object) {
        if (object instanceof QuestionCategory) {
            questionModel.showPathOfNode(object);
        }
        else if (object instanceof Text) {
            textModel.showPathOfNode(object);
        }
        superLevelModel.showPathOfNode(object);
    }

    public ChangeManager getChangeModelManager() {
        return changeManager;
    }

    /**
     * Resets an object to its previous state. Is called when
     * the user clicks the "Discard" button.
     *
     * @param object The object to reset.
     */
    public void undoChanges(AoCObject object) {
        AoCObject original = changeManager.getOriginalObject(object);
        object.setAllAttributes(original.getAllAttributes());

        if (object instanceof MultiChoiceQuestion) {
            ((MultiChoiceQuestion)object).resetAlternatives((MultiChoiceQuestion)original);
        }

        if (object.equals(currentObject)) this.tabPanel.undoChange(object);
    }

    /**
     * Method to be called whenever the user wants the changed object to be written to
     * the database.
     *
     * <code>ConflictView</code> is used if the user's local representation of an object does
     * not correspond to the object in the database, as this indicates that another user has
     * updated the object in the database since last update.
     *
     * @return If the DB write was successful
     */
    public boolean saveChangedObjectsToDatabase() {
        ArrayList<ChangedObjectWrapper> changedObjectWrappers = this.changeManager.getChangedObjects();
        ArrayList<AoCObject> objectsToSave = new ArrayList<AoCObject>();

        for (ChangedObjectWrapper objectWrapper : changedObjectWrappers) {
            AoCObject originalObject = objectWrapper.getOldObject();
            AoCObject newObject = objectWrapper.getNewObject();

            if (newObject != null) {
                AoCObject dbObject = Parser.getObject(originalObject);

                if (originalObject.differsFrom(dbObject) && newObject.differsFrom(dbObject)) {
                    int overWrite = JOptionPane.showOptionDialog(
                            tabPanel,
                            newObject + " was updated in the DB, do you want to overwrite the changes in the Database?",
                            "Conflict detected",
                            JOptionPane.YES_NO_CANCEL_OPTION,
                            JOptionPane.WARNING_MESSAGE,
                            null,
                            new Object[]{"Yes", "No", "Details"},
                            JOptionPane.NO_OPTION
                    );

                    if (overWrite == JOptionPane.CANCEL_OPTION) {
                        JPanel panel = ConflictViewGenerator.showConflict(dbObject, newObject);

                        overWrite = JOptionPane.showOptionDialog(
                            tabPanel,
                            panel,
                            "Conflict detected",
                            JOptionPane.YES_NO_OPTION,
                            JOptionPane.WARNING_MESSAGE,
                            null,
                            new Object[]{"Keep local object", "Keep database object"},
                            JOptionPane.NO_OPTION
                        );
                    }

                    if (overWrite == JOptionPane.YES_OPTION) {
                                            JOptionPane.showOptionDialog(
                                                    tabPanel,
                                                    newObject + " was written\nto the Database.",
                                                    "Object updated",
                                                    JOptionPane.OK_OPTION,
                                                    JOptionPane.INFORMATION_MESSAGE,
                                                    null,
                                                    new Object[]{"OK"},
                                                    null);
                    }
                    else {
                        JOptionPane.showOptionDialog(
                                tabPanel,
                                "The object was updated with the information\nfrom the Database.",
                                "Object updated",
                                JOptionPane.OK_OPTION,
                                JOptionPane.INFORMATION_MESSAGE,
                                null,
                                new Object[]{"OK"},
                                null);

                        newObject.setAllAttributes(dbObject.getAllAttributes());
                        if (newObject.equals(currentObject)) tabPanel.undoChange(newObject);

                        continue;
                    }
                }

                objectsToSave.add(newObject);
            }
        }
        objectsToSave.addAll(adminModel.getChangedUsers());

        boolean success = Parser.updateObjects(objectsToSave);

        this.changeManager.flush();
        this.adminModel.flushChangedUsers();

        return success;
    }


    /**
     * Fired whenever an object is updated and its new information needs to be shown
     * immediately. Updates the object with its new information first, and then
     * shows the object with its new information.
     *
     * @param object The object whose LevelModel or parent group has been changed
     */
    public void fireObjectParentUpdated(AoCObject object) {
        if (!(object instanceof LevelModel)) {
            this.tabPanel.selectionChanged(object, object);
        }
        this.changeManager.updateObject(object);
        this.saveChangedObjectsToDatabase();
        this.changeManager.addObject(object);
   }

    /**
     * Called whenever a level has changed its map.
     *
     * @param model The level that has changed its map
     */
    public void fireLevelMapChanged(LevelModel model) {
        this.tabPanel.updateLevelModel(model);
    }

    /**
     * Fired whenever the save button is clicked. Stores the visible information
     * into the current object, and saves all changed objects to the database.
     *
     * @param object The currently selected object
     */
    public void fireSaveClicked(AoCObject object) {
        this.tabPanel.selectionChanged(object, object);
        getChangeModelManager().updateObject(currentObject);
        saveChangedObjectsToDatabase();
        getChangeModelManager().addObject(currentObject);
    }

    /**
     * Called whenever the ids of objects have been updated.
     * Currently used for <code>MultiChoiceAlternative</code> objects.
     *
     * @param changed The changed objects
     */
    public void fireObjectIdsUpdated(HashMap<Integer, AoCObject> changed) {
        Parser.updateObjectIds(changed);
    }

    /**
     * Called whenever an <code>MapItem</code> changes its level.
     *
     * @param levelModel The new level
     */
    public void fireLevelChanged(LevelModel levelModel) {
        if (this.currentObject instanceof MapItem) {
            superLevelModel.fireNodeMoved((GroupElement)this.currentObject, false);
        }
        else {
            System.err.println("ModelManager.fireLevelChanged called with illegal parameter: " + currentObject.getClass().getName());
        }
    }

    /**
     * Sets the visible parameter of all objects defined by c.
     * Used when filters are applied.
     *
     * @param c The kind of objects affected
     * @param visible The visibility of the objects.
     */
    public void setObjectsVisible(Class c, boolean visible) {
        HashMap<Integer, ?> location;
        Class sc = c.getSuperclass();
        if      (sc.equals(Question.class))             location = questions;
        else if (sc.equals(Text.class))                 location = texts;
        else {
            if (c.equals(QuestionCategory.class))          location = questionCategories;
            else if (c.equals(QuestionGroup.class))        location = questionGroups;
            else if (c.equals(TextGroup.class))            location = textgroups;
            else if (c.equals(TeleportSource.class))       location = teleportSources;
            else if (c.equals(TeleportDestination.class))  location = questionCategories;
            else if (c.equals(Message.class))              location = messages;
            else return;
        }

        for (Object o : location.values()) {
            if (o.getClass().equals(c)) {
                ((GroupElement)o).setVisible(visible);
            }
        }

        superLevelModel.fireFiltersChanged();
        questionModel.fireFiltersChanged();
        textModel.fireFiltersChanged();
    }

    /**
     * Called whenever the LeveTreeModel is updated (adding
     * or removing levels)
     */
    public void superLevelModelChanged() {
        tabPanel.superLevelModelChanged();
    }
}