package sys.models;

import sys.datatypes.User;

import javax.swing.table.AbstractTableModel;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * TableModel for the user administration table
 *
 * @author Tarje Killingberg - 20.jun.2006
 */
public class UserTableModel extends AbstractTableModel {


    private String[] columnNames = {"ID", "User", "Admin", "Level", "X-pos", "Y-pos", "Z-pos", "Points"};
    private HashMap<Integer, User> orgUserHashMap;
    private HashMap<Integer, User> userHashMap;
    private ArrayList<Integer> userIdList;
    private AdminModel adminModel;

    /* filter variables */
    boolean showAdmin;
    boolean showNonAdmin;
    String value;
    String field;



    /**
     * Creates a new UserTableModel
     */
    public UserTableModel() {

    }


    public User getUser(int index) {
        if (index < 0 || index >= userIdList.size()) {
            return null;
        }
        return userHashMap.get(userIdList.get(index));
    }


    public User[] getUsers(int[] indices) {
        User[] users = new User[indices.length];
        for (int i = 0; i < indices.length; i++) {
            users[i] = userHashMap.get(userIdList.get(indices[i]));
        }
        return users;
    }


    public void filterChanged(boolean showAdmin,
                              boolean showNonAdmin,
                              String value,
                              String field) {

        this.showAdmin = showAdmin;
        this.showNonAdmin = showNonAdmin;
        this.value = value;
        this.field = field;

        filter();
    }


    private void filter() {
        if (orgUserHashMap == null) {
            return;
        }

        userHashMap = new HashMap<Integer, User>();
        Iterator iter = orgUserHashMap.keySet().iterator();

        while (iter.hasNext()) {
            User user = orgUserHashMap.get(iter.next());
            if ((user.isAdmin() && showAdmin) || (!user.isAdmin() && showNonAdmin)) {
                if (field.equalsIgnoreCase("Usernames")) {
                    if (user.getUsername().toUpperCase().indexOf(value.toUpperCase()) >= 0) {
                        userHashMap.put(user.getId(), user);
                    }
                }
                else if (field.equalsIgnoreCase("Levels")) {
                    if (value.equals("")) {
                        userHashMap.put(user.getId(), user);
                    }
                    else {
                        try {
                            int levelID = Integer.parseInt(value);
                            if (user.getLevelID() == levelID) {
                                userHashMap.put(user.getId(), user);
                            }
                        } catch (NumberFormatException nfe) {
                        }
                    }
                }
            }
        }
        userIdList = new ArrayList<Integer>(userHashMap.keySet());
        fireTableDataChanged();
    }


    public void adminModelChanged() {
        filter();
    }

    public void fireUserCreated(User user) {
        int userIndex = userIdList.indexOf(user.getId());
        fireTableRowsInserted(userIndex, userIndex);
    }



    /* ----- Getters and setters ----- */


    public void setAdminModel(AdminModel adminModel) {
        this.adminModel = adminModel;
        this.adminModel.setUserTableModel(this);
        this.setUserHashMap(adminModel.getUserHashMap());
    }


    private void setUserHashMap(HashMap<Integer, User> orgUserHashMap) {
        this.orgUserHashMap = orgUserHashMap;
        this.userIdList = new ArrayList<Integer>(orgUserHashMap.keySet());
        filter();
    }



    /* ----- Methods from abstract superclass AbstractTableModel ----- */


    /**
     * Returns the number of rows in the model. A
     * <code>JTable</code> uses this method to determine how many rows it
     * should display.  This method should be quick, as it
     * is called frequently during rendering.
     *
     * @return the number of rows in the model
     * @see #getColumnCount
     */
    public int getRowCount() {
        if (userIdList == null) {
            return 0;
        }
        return userIdList.size();
    }


    /**
     * Returns the number of columns in the model. A
     * <code>JTable</code> uses this method to determine how many columns it
     * should create and display by default.
     *
     * @return the number of columns in the model
     * @see #getRowCount
     */
    public int getColumnCount() {
        return columnNames.length;
    }


    /**
     * Returns the value for the cell at <code>columnIndex</code> and
     * <code>rowIndex</code>.
     *
     * @param    rowIndex    the row whose value is to be queried
     * @param    columnIndex the column whose value is to be queried
     * @return the value Object at the specified cell
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        User user = userHashMap.get(userIdList.get(rowIndex));
        switch(columnIndex) {
            case 0:
                return user.getId();
            case 1:
                return user.getUsername();
            case 2:
                return user.isAdmin();
            case 3:
                return user.getLevelID();
            case 4:
                return user.getMapPosition().getX();
            case 5:
                return user.getMapPosition().getY();
            case 6:
                return user.getMapPosition().getZ();
            case 7:
                return user.getPoints();
            default:
                return null;
        }
    }


    /**
     * Returns a default name for the column using spreadsheet conventions:
     * A, B, C, ... Z, AA, AB, etc.  If <code>column</code> cannot be found,
     * returns an empty string.
     *
     * @param column the column being queried
     * @return a string containing the default name of <code>column</code>
     */
    @Override
    public String getColumnName(int column) {
        return columnNames[column];
    }


    /**
     * Returns <code>Object.class</code> regardless of <code>columnIndex</code>.
     *
     * @param columnIndex the column being queried
     * @return the Object.class
     */
    @Override
    public Class<?> getColumnClass(int columnIndex) {
        return getValueAt(0, columnIndex).getClass();
    }


    /**
     * Returns false.  This is the default implementation for all cells.
     *
     * @param rowIndex    the row being queried
     * @param columnIndex the column being queried
     * @return false
     */
    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        switch(columnIndex) {
            case 0:
            case 7:
                return false;
            default:
                return true;
        }
    }


    /**
     * This empty implementation is provided so users don't have to implement
     * this method if their data model is not editable.
     *
     * @param aValue      value to assign to cell
     * @param rowIndex    row of cell
     * @param columnIndex column of cell
     */
    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        User user = userHashMap.get(userIdList.get(rowIndex));
        switch(columnIndex) {
            case 1:
                user.setUsername((String)aValue);
                break;
            case 2:
                user.setAdmin(((Boolean)aValue));
                break;
            case 3:
                user.setLevelID(((Integer)aValue));
                break;
            case 4:
                user.getMapPosition().setX((Integer)aValue);
                break;
            case 5:
                user.getMapPosition().setY((Integer)aValue);
                break;
            case 6:
                user.getMapPosition().setZ((Integer)aValue);
                break;
            default:
                break;
        }
        fireTableCellUpdated(rowIndex, columnIndex);
        adminModel.userChanged(user);
    }


}
