package sys.models.treemodels;

import gui.tree.SuperTree;
import sys.datatypes.Group;
import sys.datatypes.GroupElement;
import sys.datatypes.interfaces.AoCObject;
import sys.models.LevelModel;
import sys.models.ModelManager;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

/**
 * The model used to show the trees. Implements and defines all
 * methods specified in the <code>TreeModel</code> interface, except
 * <code>isLeaf(Object node)</code>, which may be specific for each of the three
 * different trees.
 *
 * @author Eivind Sorteberg - 20.jun.2006
 */
public abstract class AbstractTreeModel implements TreeModel {
    protected HashMap<String, Boolean> filters;

    private Group root;
    protected ModelManager modelManager;
    protected SuperTree tree;
    private ArrayList<TreeModelListener> listeners;


    public AbstractTreeModel(Group root, ModelManager modelManager) {
        this.root = root;
        this.modelManager = modelManager;
        this.listeners = new ArrayList<TreeModelListener>();

        this.createFilters();
    }

    /**
     * Creates the filters that may be used for this model, if any
     */
    private void createFilters() {
        filters = new HashMap<String, Boolean>();
        for (String filter : getFilterNames()) {
            filters.put(filter, true);
        }
    }


    public ArrayList<TreeModelListener> getListeners() {
        return listeners;
    }


    public void setTree(SuperTree tree) {
        this.tree = tree;
    }

    /**
     * Sorts the entire tree by recursively sorting each subtree, starting
     * with the root.
     */
    public void sortTree() {
        root.sortGroupElements();
        root.sortSubTree();
    }

    /**
     * Called whenever a node is inserted in the tree. Notifies the tree
     * that it needs to draw the new node, and selects the new node.
     *
     * @param node The new node
     */
    public void fireNodeInserted(GroupElement node) {
        if (node.isVisible()) {
            for (TreeModelListener listener : listeners) {
                listener.treeNodesInserted(
                        new TreeModelEvent(
                                node,
                                getPathOfNode(node, false),
                                new int[]{node.getParent().getNumberOfElements() - 1},
                                new GroupElement[]{node})
                );
            }

            this.tree.setSelectionPath(this.getPathOfNode(node, true));
            this.tree.scrollPathToVisible(this.getPathOfNode(node, true));
        }
        else modelManager.fireObjectSelected(node);
    }

    /**
     * Called whenever a node is moved or removed in the tree.
     * Notifies its listeners that it has been changed, and refreshes
     * the selection in the tree to a valid node.
     *
     * @param node The node that has been moved or removed
     * @param delete Whether the node has been deleted (true if it is moved)
     */
    public void fireNodeMoved(GroupElement node, boolean delete) {
        for (TreeModelListener listener : listeners) {
            listener.treeStructureChanged(
                    new TreeModelEvent(
                            node,
                            new Object[]{this.getRoot()}
                    )
            );
        }

        TreePath path;

        if (node instanceof LevelModel) {
            path = getPathOfNode(root.getElement(0), delete);
        }
        else {
            path = getPathOfNode(node, !delete);
            this.tree.expandPath(path);
        }
        this.tree.setSelectionPath(path);
        this.tree.scrollPathToVisible(path);
    }



    /**
     * Returns the nodes visited when traversing the tree from root to a
     * given node.
     *
     * @param node The node to find
     * @param includeNode Whether the node should be included in the path
     * @return The path from root -> node
     */
    public TreePath getPathOfNode(GroupElement node, boolean includeNode) {
        ArrayList<GroupElement> path = new ArrayList<GroupElement>();
        if (includeNode) path.add(node);

        while (!(root.equals(node))) {
            node = node.getParent();
            path.add(node);
        }

        GroupElement[] ret = new GroupElement[path.size()];

        //Reverses the list so that it is root -> leaf (or parent)
        for (int i = 0; i < ret.length; i++) {
            ret[ret.length - i - 1] = path.get(i);
        }

        return new TreePath(ret);
    }


    /**
     * Abstract method for returning the names of the possible
     * new objects created in this model.
     *
     * @see gui.editors.common.selector.NewObjectSelector
     * @return An array containing the possible object names
     */
    public abstract Object[] getObjectNames();


    /**
     * Creates a new object of the type given as parameter.
     * The legal type strings are defined by the implementing class.
     *
     * @param type The type of object to create.
     */
    public abstract void createObject(String type);

    /**
     * This method is called whenever a node is selected in one of the trees. This
     * is a good starting point for tracing how information is displayed and
     * processed.
     *
     * @param object The selected tree node.
     */
    public void fireNodeSelected(AoCObject object) {
        modelManager.fireObjectSelected(object);
    }

    public ModelManager getModelManager() {
        return modelManager;
    }

    /**
     * Moves the ModelManager's currently selected object to the group given
     * as in parameter, and notifies the tree that its structure has been changed.
     *
     * @param group The new parent group of the object.
     */
    public void moveObject(Group group) {
        GroupElement object = (GroupElement)modelManager.getCurrentObject();
        object.setParent(group);
        modelManager.fireObjectParentUpdated(object);
        this.fireNodeMoved(object, false);
    }

    /**
     * Fired whenever a node changes its level
     *
     * @param levelModel The object's new level
     */
    public void moveToLevel(LevelModel levelModel) {
        modelManager.fireLevelChanged(levelModel);
    }

    /**
     * Returns a list of objects which the currently selected object may be assigned to.
     *
     * @param object The currently selected object
     * @return A list of Groups
     */
    public ArrayList<Group> getPotentialParents(GroupElement object) {
        ArrayList<Group> parents = new ArrayList<Group>();
        findGroups(this.getRoot(), parents, object);

        Comparator<Group> c = new Comparator<Group>() {
            public int compare(Group g1, Group g2) {
                return g1.getName().compareTo(g2.getName());
            }
        };

        Collections.sort(parents, c);

        return parents;
    }


    /**
     * Recursive method that finds the potential parents for the selected object; that is <code>Group</code> for
     * <code>GroupElement</code> objects. This method also makes sure that the child of a node
     * can not be the same node's parent, as this leads to resolving the new tree being impossible.
     *
     * @param node The current "root" of this search level.
     * @param parents The list of possible parents.
     * @param object The node looking for a parent.
     */
    protected void findGroups(Object node, ArrayList<Group> parents, GroupElement object) {
        if (node instanceof Group && !node.equals(object)) {
            parents.add((Group)node);

            for (int i = 0; i < ((Group)node).getNumberOfElements(); i++) {
                    findGroups(((Group)node).getElement(i), parents, object);
            }
        }
    }


    /**
     * Removes the ModelManager's currently selected node, if any
     */
    public void removeNode() {
        modelManager.fireObjectDeleted();
    }



    /* -----  Methods from interace TreeModel ----- */


    public void addTreeModelListener(TreeModelListener l) {
        this.listeners.add(l);
    }


    public Object getChild(Object parent, int index) {
        Group group = (Group)parent;

        int pos = 0;

        for (int i = 0; i < group.getGroupElements().size(); i++) {
            GroupElement groupElement = group.getElement(i);

            if (groupElement.isVisible()) {
                if (pos == index) return groupElement;
                pos++;
            }
        }

        return ((Group)parent).getElement(index);
    }


    public int getChildCount(Object parent) {
        if (!(parent instanceof Group)) return 0;

        Group group = (Group)parent;

        int number = 0;

        for (GroupElement groupElement : group.getGroupElements()) {
            if (groupElement.isVisible()) number++;
        }

        return number;
    }


    public int getIndexOfChild(Object parent, Object child) {
        for (int i = 0; i < getChildCount(parent); i++) {
            Object element = getChild(parent, i);

            if (element.equals(child)) return i;
        }

        return -1;
    }


    public Object getRoot() {
        return root;
    }


    /**
     * Determines whether a node in the model is a leaf or not. Because of the different
     * properties of the trees, this method's body may be specified in the implementing
     * subclasses.
     *
     * @param node The node to check
     * @return Whether the node is a leaf or not
     */
    public boolean isLeaf(Object node) {
        return !(node instanceof Group);
    }

    public void removeTreeModelListener(TreeModelListener l) {
        this.listeners.remove(l);
    }


    public void valueForPathChanged(TreePath path, Object newValue) {
    }

    public void showPathOfNode(GroupElement node) {
        TreePath path = getPathOfNode(node, true);
        this.tree.setSelectionPath(path);
        this.tree.scrollPathToVisible(path);
    }

    /**
     * @return The names of the filters that can be applied to this model
     */
    public abstract String[] getFilterNames();

    /**
     * Called whenever a filter is activated or deactivated
     *
     * @param filter The filter to use
     * @param value Whether the filter should be activated or deactivated
     */
    public abstract void fireFilterSelected(String filter, boolean value);

    /**
     * Notifies the listeners that nodes have been removed or added when a filter
     * is activated or deactivated
     */
    public void fireFiltersChanged() {
        for (TreeModelListener listener : listeners) {
            listener.treeStructureChanged(
                    new TreeModelEvent(
                            this,
                            new Object[]{root}
                    )
            );
        }
    }
}
