package sys.models.treemodels;

import sys.datatypes.*;
import sys.datatypes.interfaces.MapItem;
import sys.datatypes.question.QuestionCategory;
import sys.models.LevelModel;
import sys.models.ModelManager;

import javax.swing.*;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.util.*;

/**
 * Model for the LevelEditor-view.
 *
 * @author Tarje Killingberg - 19.jun.2006
 */
public class LevelTreeModel extends AbstractTreeModel {


    private static final String level = "Level";
    private static final String questionCategory = "Question category";
    private static final String history = "History";
    private static final String fact = "Fact";
    private static final String message = "Message";
    private static final String teleportSource = "Teleport source";
    private static final String teleportDestionation = "Teleport destination";

    private Point activePoint;
    private ArrayList<String> mapList;


    public LevelTreeModel(Group root, ModelManager modelManager) {
        super(root, modelManager);
        activePoint = new Point(10, 10);
        mapList = new ArrayList<String>();
    }


    public ArrayList<QuestionCategory> getAllTeleportRequirementCategories() {
        ArrayList<QuestionCategory> questionCategories = new ArrayList<QuestionCategory>();

        HashMap<Integer, LevelModel> levelModels = ((LevelModel)getRoot()).getElements();
        Iterator iter = levelModels.keySet().iterator();
        while (iter.hasNext()) {
            LevelModel lm = levelModels.get(iter.next());
            ArrayList<GroupElement> ges = lm.getGroupElements();
            for (int i = 0; i < ges.size(); i++) {
                GroupElement ge = ges.get(i);
                if (ge instanceof QuestionCategory) {
                    questionCategories.add((QuestionCategory)ge);
                }
            }

        }
        
        return questionCategories;
    }


    public ArrayList<TeleportDestination> getAllTeleportDestinations() {
        ArrayList<TeleportDestination> dests = new ArrayList<TeleportDestination>();

        HashMap<Integer, LevelModel> levelModels = ((LevelModel)getRoot()).getElements();
        Iterator iter = levelModels.keySet().iterator();
        while (iter.hasNext()) {
            LevelModel lm = levelModels.get(iter.next());
            ArrayList<GroupElement> ges = lm.getGroupElements();
            for (int i = 0; i < ges.size(); i++) {
                GroupElement ge = ges.get(i);
                if (ge instanceof TeleportDestination) {
                    dests.add((TeleportDestination)ge);
                }
            }
        }

        return dests;
    }


    /**
     * Moves the ModelManager's currently selected object to the group given
     * as in parameter, and notifies the tree that its structure has been changed.
     *
     * @param group The new parent group of the object.
     */
    public void moveObject(Group group) {
    }


    /**
     * This method returns an existing TeleportDestination-object,
     * or null if none exists. The object returned is the first
     * we come over, but not neccessarily the same each time, so
     * noone should rely on the value returned by this method.
     *
     * @return an existing TeleportDestination-object,
     *         or null if none exists
     */
    private TeleportDestination getSuitableTeleportDestination() {
        HashMap<Integer, LevelModel> levelModels = ((LevelModel)getRoot()).getElements();
        Iterator iter = levelModels.keySet().iterator();
        while (iter.hasNext()) {
            LevelModel lm = levelModels.get(iter.next());
            ArrayList<GroupElement> ges = lm.getGroupElements();
            for (int i = 0; i < ges.size(); i++) {
                GroupElement ge = ges.get(i);
                if (ge instanceof TeleportDestination) {
                    return (TeleportDestination)ge;
                }
            }

        }
        return null;
    }


    public ArrayList<QuestionCategory> getPotentialTeleportRequirementCategories(GroupElement groupElement) {

        ArrayList<QuestionCategory> potentialCategories = new ArrayList<QuestionCategory>();
        if (groupElement instanceof TeleportSource) {
            potentialCategories = getAllTeleportRequirementCategories();
            TeleportSource teleportSource = (TeleportSource)groupElement;
            ArrayList<QuestionCategory> currentCategories = teleportSource.getRequirementList();
            for (int i = 0; i < currentCategories.size(); i++) {
                potentialCategories.remove(currentCategories.get(i));
            }
        }

        return potentialCategories;
    }


    public ArrayList<Group> getPotentialParents(GroupElement object) {
        return null;
    }


    public Point getActivePoint() {
        return this.activePoint;
    }


    public void setActivePoint(Point p) {
        this.activePoint = p != null ? p : new Point(10, 10);
    }


    public ArrayList<String> getMapList() {
        return mapList;
    }


    public void setMapList(ArrayList<String> mapList) {

        Collections.sort(mapList, new Comparator() {
            public int compare(Object o1, Object o2) {
                return o1.toString().compareToIgnoreCase(o2.toString());
            }
        });

        this.mapList = mapList;
    }



    /* Methods from abstract superclass AbstractTreeModel */


    public Object[] getObjectNames() {

        return new Object[]{level,
                            new JSeparator(),
                            questionCategory,
                            new JSeparator(),
                            history,
                            fact,
                            new JSeparator(),
                            message,
                            teleportSource,
                            teleportDestionation};

    }


    public void createObject(String type) {
        GroupElement object;
        LevelModel lm = null;
        Object selectedNode = null;

        /* make sure we have a levelmodel to place the new object in */
        if (!type.equals(level)) {
            try {
                selectedNode = tree.getSelectionPath().getLastPathComponent();
            } catch (NullPointerException npe) {
                JOptionPane.showMessageDialog(this.getModelManager().getTabPanel(),
                                              "Select an item in the tree first.",
                                              "Error - No item chosen.",
                                              JOptionPane.ERROR_MESSAGE);
                return;
            }
        }

        /* getting the levelmodel */
        if (selectedNode instanceof LevelModel) {
            lm = (LevelModel)selectedNode;
        }
        else if (selectedNode instanceof MapItem) {
            MapItem mi = (MapItem)selectedNode;
            lm = mi.getLevelModel();
        }

        /* creating the object */
        if (type.equals(level)) {
            object = new Level(-1, "New Level", "undefined");
        }
        else {
            if (activePoint == null) { activePoint = new Point(10, 10); }

            int levelID = lm.getLevelObject().getId();
            int x = (int)activePoint.getX();
            int y = (int)activePoint.getY();
            setActivePoint(null);

            if (type.equals(questionCategory)) {
                object = new QuestionCategory(levelID, new MapPosition(x, y));
            }
            else if (type.equals(history)) {
                object = new History(levelID, new MapPosition(x, y));
            }
            else if (type.equals(fact)) {
                object = new Fact(levelID, new MapPosition(x, y));
            }
            else if (type.equals(message)) {
                object = new Message(levelID, new MapPosition(x, y));
            }
            else if (type.equals(teleportSource)) {
                TeleportDestination td = getSuitableTeleportDestination();
                if (td == null) {
                    JOptionPane.showMessageDialog(this.getModelManager().getTabPanel(),
                                                  "Create a destination first.",
                                                  "Error - No destinations exist.",
                                                  JOptionPane.ERROR_MESSAGE);

                    return;
                }
                object = new TeleportSource(levelID, td.getId(),  new MapPosition(x, y));
            }
            else if (type.equals(teleportDestionation)) {
                object = new TeleportDestination(levelID, new MapPosition(x, y));
            }
            else {
                System.err.println("ERROR! Illegal type; found: " + type.getClass().getName());
                return;
            }
        }

        Boolean visible = filters.get(type);

        if (visible != null) {
            object.setVisible(visible);
        }


        modelManager.fireObjectCreated(object);
    }



    /* ----- overridden methods ----- */


    @Override
    public boolean isLeaf(Object node) {
        return node instanceof MapItem && !(node instanceof Level);
    }


    @Override
    public void fireNodeInserted(GroupElement node) {

        int[] childIndices;
        for (TreeModelListener listener : getListeners()) {
            if (node instanceof LevelModel) {
                childIndices = new int[]{((LevelModel)getRoot()).getNumberOfElements() - 1};
            }
            else {
                childIndices = new int[]{((MapItem)node).getLevelModel().getNumberOfElements() - 1};
            }
            listener.treeNodesInserted(
                    new TreeModelEvent(node,
                            getPathOfNode(node, false),
                            childIndices,
                            new GroupElement[]{node}));
        }

        this.modelManager.superLevelModelChanged();
        this.tree.setSelectionPath(this.getPathOfNode(node, true));
        this.tree.scrollPathToVisible(this.getPathOfNode(node, true));
    }


    @Override
    public TreePath getPathOfNode(GroupElement node, boolean includeNode) {
        ArrayList<GroupElement> path = new ArrayList<GroupElement>();
        if (includeNode) path.add(node);

        while (!(node.equals(getRoot()))) {
            if (node instanceof LevelModel) {
                node = (Group)getRoot();
            }
            else if (node instanceof MapItem) {
                node = ((MapItem)node).getLevelModel();
            }

            path.add(node);
        }

        GroupElement[] ret = new GroupElement[path.size()];

        for (int i = 0; i < ret.length; i++) {
            ret[ret.length - i - 1] = path.get(i);
        }

        return new TreePath(ret);
    }

    public String[] getFilterNames() {
        return new String[0];

    }

    public void fireFilterSelected(String filter, boolean visible) {
        Class c;
        if (teleportSource.equals(filter))              c = TeleportSource.class;
        else if (teleportDestionation.equals(filter))   c = TeleportDestination.class;
        else if (message.equals(filter))                c = Message.class;
        else if (questionCategory.equals(filter))       c = QuestionCategory.class;
        else if (history.equals(filter))                c = History.class;
        else if (fact.equals(filter))                   c = Fact.class;
        else return;

        modelManager.setObjectsVisible(c, visible);
    }
}