package sys.models.treemodels;

import sys.datatypes.Group;
import sys.datatypes.GroupElement;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.question.*;
import sys.models.ModelManager;

import javax.swing.*;
import javax.swing.tree.TreePath;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

/**
 * Model for the QuestionEditor-view.
 *
 * @author Tarje Killingberg - 19.jun.2006
 */
public class QuestionTreeModel extends AbstractTreeModel {
    private static final String questionGroup = "Question Group";
    private static final String questionCategory = "Question Category";
    private static final String multiChoiceQuestion = "Multiple Choice Question";
    private static final String inputQuestion = "Input Question";
    private static final String darkQuestion = "Dark Question";
    private static final String autogenQuestion = "Autogenerating Question";

    public QuestionTreeModel(QuestionGroup root, ModelManager modelManager) {
        super(root, modelManager);
    }

    public Object[] getObjectNames() {
        return new Object[]{
                questionGroup,
                new JSeparator(JSeparator.HORIZONTAL),
                questionCategory,
                new JSeparator(),
                multiChoiceQuestion,
                inputQuestion,
                darkQuestion,
                autogenQuestion
        };
    }

    public String[] getFilterNames() {
        return new String[] {
                multiChoiceQuestion,
                inputQuestion,
                darkQuestion,
                autogenQuestion
        };
    }

    public void createObject(String type) {
        GroupElement object;

        if (type.equals(questionGroup)) {
            object = new QuestionGroup();
        }
        else if (type.equals(questionCategory)) {
            object = new QuestionCategory();
        }
        else {
            if (type.equals(multiChoiceQuestion))       object = new MultiChoiceQuestion();
            else if (type.equals(inputQuestion))        object = new InputQuestion();
            else if (type.equals(darkQuestion))         object = new DarkQuestion();
            else if (type.equals(autogenQuestion))      object = new AutoGenQuestion();

            else {
                System.err.println("ERROR! Illegal question type; found: " + type);
                return;
            }
        }

        Boolean visible = filters.get(type);

        if (visible != null) {
            object.setVisible(visible);
        }

        TreePath selectionPath = tree.getSelectionPath();
        if (selectionPath != null) setParent(selectionPath, object);

        modelManager.fireObjectCreated(object);
    }

    /**
     * Sets the parent of the new node, while making sure that this parent
     * is a legal parent.
     *
     * @param selectionPath
     * @param newObject
     */
    private void setParent(TreePath selectionPath, GroupElement newObject) {
        GroupElement node = (GroupElement)selectionPath.getLastPathComponent();

        while(node.getParent() != null) {
            if (newObject instanceof Question) {
                if (node instanceof QuestionCategory) {
                    newObject.setParent_id(node.getId());
                    return;
                }
            }
            else if (newObject instanceof QuestionCategory) {
                if (node instanceof QuestionGroup) {
                    newObject.setParent_id(node.getId());
                    return;
                }
            }
            else if (newObject instanceof QuestionGroup) {
                if (node instanceof Group && !(node instanceof QuestionCategory)) {
                    newObject.setParent_id(node.getId());
                    return;
                }
            }
            node = node.getParent();
        }
    }

    public void createMultiChoiceAlternative(MultiChoiceAlternative newAlt) {
        modelManager.fireObjectCreated(newAlt);
    }

    /**
     * Method user for
     * @param removed
     */
    public void removeObjects(MultiChoiceAlternative[] removed) {
        for (AoCObject object : removed) {
            modelManager.fireObjectDeleted(object);
        }
    }


    public void updateObjects(HashMap<Integer, AoCObject> changed) {
        modelManager.fireObjectIdsUpdated(changed);
    }

    /**
     * Special case for Question objects, which can only be assigned to QuestionCategory objects
     *
     * @param object
     * @return All potential parents for the selected object
     */
    public ArrayList<Group> getPotentialParents(GroupElement object) {
        ArrayList<Group> parents = new ArrayList<Group>();

        if (object instanceof Group) {
            findQuestionGroups(this.getRoot(), parents, object);
        }
        else {
            //Object is question
            findQuestionCategories(this.getRoot(), parents);
        }

        Comparator<Group> c = new Comparator<Group>() {
            public int compare(Group g1, Group g2) {
                return g1.getName().compareTo(g2.getName());
            }
        };

        Collections.sort(parents, c);

        return parents;
    }

    private void findQuestionGroups(Object node, ArrayList<Group> parents, GroupElement object) {
        if (node instanceof QuestionGroup && !node.equals(object)) {
            parents.add((QuestionGroup)node);

            for (int i = 0; i < ((QuestionGroup)node).getNumberOfElements(); i++) {
                findQuestionGroups(((QuestionGroup)node).getElement(i), parents, object);
            }
        }
    }

    private void findQuestionCategories(Object node, ArrayList<Group> parents) {
        if (node instanceof Group) {
            if (node instanceof QuestionCategory) parents.add((QuestionCategory)node);

            else {
                for (int i = 0; i < ((Group)node).getNumberOfElements(); i++) {
                    findQuestionCategories(((Group)node).getElement(i), parents);
                }
            }
        }
    }

    public void fireFilterSelected(String filter, boolean visible) {
        filters.put(filter, visible);

        Class c;
        if (multiChoiceQuestion.equals(filter)) {
            c = MultiChoiceQuestion.class;
        }
        else if (inputQuestion.equals(filter)) {
            c = InputQuestion.class;
        }
        else if (darkQuestion.equals(filter)) {
            c = DarkQuestion.class;
        }
        else if (autogenQuestion.equals(filter)) {
            c = AutoGenQuestion.class;
        }
        else return;

        modelManager.setObjectsVisible(c, visible);
    }
}
